@@ -8,17 +8,20 @@
 #ifndef COMPOSITEBAMREADER_H
 #define COMPOSITEBAMREADER_H
 
+#include "pbbam/Config.h"
+
 #include <deque>
 #include <functional>
 #include <memory>
 #include <string>
 #include <vector>
+
+#include "pbbam/BaiIndexCache.h"
 #include "pbbam/BaiIndexedBamReader.h"
 #include "pbbam/BamFile.h"
 #include "pbbam/BamHeader.h"
 #include "pbbam/BamReader.h"
 #include "pbbam/BamRecord.h"
-#include "pbbam/Config.h"
 #include "pbbam/DataSet.h"
 #include "pbbam/GenomicInterval.h"
 #include "pbbam/PbiIndexedBamReader.h"
@@ -41,9 +44,6 @@ public:
 public:
     CompositeMergeItem(std::unique_ptr<BamReader> rdr);
     CompositeMergeItem(std::unique_ptr<BamReader> rdr, BamRecord rec);
-    CompositeMergeItem(CompositeMergeItem&&) = default;
-    CompositeMergeItem& operator=(CompositeMergeItem&&) = default;
-    ~CompositeMergeItem() = default;
 };
 
 /// \internal
@@ -57,7 +57,7 @@ template <typename CompareType>
 struct CompositeMergeItemSorter
     : public std::function<bool(const CompositeMergeItem&, const CompositeMergeItem&)>
 {
-    bool operator()(const CompositeMergeItem& lhs, const CompositeMergeItem& rhs);
+    bool operator()(const CompositeMergeItem& lhs, const CompositeMergeItem& rhs) const;
 };
 
 }  // namespace internal
@@ -76,9 +76,60 @@ public:
     /// \name Contstructors & Related Methods
     /// \{
 
+    /// \brief Constructs composite %BAM reader, that can be queried on genomic interval.
+    ///
+    /// \note Using this constructor means that an interval must be provided, via
+    ///       reader.Interval(i), before iterating.
+    ///
+    /// \param[in] bamFiles   input BamFile objects
+    ///
+    /// \throws std::runtime_error if either file (*.bam or *.bai) fails to open
+    ///         for reading, or if the interval is invalid
+    ///
+    GenomicIntervalCompositeBamReader(const std::vector<BamFile>& bamFiles);
+    GenomicIntervalCompositeBamReader(const std::vector<BamFile>& bamFiles,
+                                      const BaiIndexCache& cache);
+
+    /// \brief Constructs composite %BAM reader, that can be queried on genomic interval.
+    ///
+    /// \note Using this constructor means that an interval must be provided, via
+    ///       reader.Interval(i), before iterating.
+    ///
+    /// \param[in] dataset      input DataSet
+    ///
+    /// \throws std::runtime_error if either file (*.bam or *.bai) fails to open
+    ///         for reading, or if the interval is invalid
+    ///
+    GenomicIntervalCompositeBamReader(const DataSet& dataset);
+    GenomicIntervalCompositeBamReader(const DataSet& dataset, const BaiIndexCache& cache);
+
+    /// \brief Constructs composite %BAM reader, limiting record results to
+    ///        only those overalpping a GenomicInterval.
+    ///
+    /// \param[in] interval   genomic interval of interest
+    /// \param[in] bamFiles   input BamFile objects
+    ///
+    /// \throws std::runtime_error on failure to open/read underlying %BAM or
+    ///         BAI files.
+    ///
     GenomicIntervalCompositeBamReader(const GenomicInterval& interval,
                                       const std::vector<BamFile>& bamFiles);
+    GenomicIntervalCompositeBamReader(const GenomicInterval& interval,
+                                      const std::vector<BamFile>& bamFiles,
+                                      const BaiIndexCache& cache);
+
+    /// \brief Constructs composite %BAM reader, limiting record results to
+    ///        only those overalpping a GenomicInterval.
+    ///
+    /// \param[in] interval genomic interval of interest
+    /// \param[in] dataset  input data source(s)
+    ///
+    /// \throws std::runtime_error on failure to open/read underlying %BAM or
+    ///         BAI files.
+    ///
     GenomicIntervalCompositeBamReader(const GenomicInterval& interval, const DataSet& dataset);
+    GenomicIntervalCompositeBamReader(const GenomicInterval& interval, const DataSet& dataset,
+                                      const BaiIndexCache& cache);
 
     /// \}
 
@@ -109,6 +160,7 @@ private:
     void UpdateSort();
 
 private:
+    BaiIndexCache indexCache_;
     GenomicInterval interval_;
     std::deque<internal::CompositeMergeItem> mergeItems_;
     std::vector<std::string> filenames_;
@@ -152,7 +204,12 @@ public:
     /// \{
 
     PbiFilterCompositeBamReader(const PbiFilter& filter, const std::vector<BamFile>& bamFiles);
+    PbiFilterCompositeBamReader(const PbiFilter& filter, const std::vector<BamFile>& bamFiles,
+                                const PbiIndexCache& cache);
+
     PbiFilterCompositeBamReader(const PbiFilter& filter, const DataSet& dataset);
+    PbiFilterCompositeBamReader(const PbiFilter& filter, const DataSet& dataset,
+                                const PbiIndexCache& cache);
 
     /// \}
 
@@ -180,6 +237,7 @@ private:
     void UpdateSort();
 
 private:
+    PbiIndexCache indexCache_;
     container_type mergeQueue_;
     std::vector<std::string> filenames_;
     uint32_t numReads_;
