@@ -7,14 +7,17 @@
 #ifndef DATASET_H
 #define DATASET_H
 
+#include "pbbam/Config.h"
+
 #include <chrono>
 #include <memory>
 #include <set>
 #include <string>
 #include <vector>
+
 #include "pbbam/BamFile.h"
-#include "pbbam/Config.h"
 #include "pbbam/DataSetTypes.h"
+#include "pbbam/GenomicInterval.h"
 
 namespace PacBio {
 namespace BAM {
@@ -114,11 +117,10 @@ public:
     ///
     DataSet(const std::vector<std::string>& filenames);
 
-    DataSet(const DataSet& other);
-    DataSet(DataSet&&) = default;
-    DataSet& operator=(const DataSet& other);
-    DataSet& operator=(DataSet&&) = default;
-    ~DataSet() = default;
+    DataSet(const DataSet&);
+    DataSet(DataSet&&) noexcept = default;
+    DataSet& operator=(const DataSet&);
+    DataSet& operator=(DataSet&&) noexcept = default;
 
     /// \brief Creates a DataSet from "raw" XML data.
     ///
@@ -154,7 +156,7 @@ public:
     /// \throws std::runtime_error if file could be opened or if DataSet
     ///         elements could not be converted to XML
     ///
-    void Save(const std::string& outputFilename);
+    void Save(const std::string& outputFilename) const;
 
     /// \brief Saves dataset XML to output stream, e.g. std::cout,
     ///        std::stringstream.
@@ -164,7 +166,7 @@ public:
     /// \throws std::runtime_error if DataSet elements could not be converted to
     ///         XML
     ///
-    void SaveToStream(std::ostream& out);
+    void SaveToStream(std::ostream& out) const;
 
     /// \}
 
@@ -344,6 +346,17 @@ public:
     ///
     std::vector<BamFile> BamFiles() const;
 
+    /// \brief Returns all filenames for BamFiles(), with paths resolved.
+    ///
+    //  Unlike BamFiles(), this does not actually open the BAM files.
+    //  (BamFile(fn) would read the header.)
+    //
+    /// \returns vector of std::string
+    ///
+    /// \sa DataSet::BamFiles
+    ///
+    std::vector<std::string> BamFilenames() const;
+
     /// \brief Returns this dataset's primary FASTA resources, with relative
     ///        filepaths already resolved.
     ///
@@ -356,6 +369,11 @@ public:
     ///
     std::vector<std::string> FastaFiles() const;
 
+    ///
+    /// \returns (absolute) path for dataset
+    ///
+    const std::string& Path() const;
+
     /// \brief Returns all primary external resource filepaths, with relative
     ///        paths resolved.
     ///
@@ -391,6 +409,15 @@ public:
     ///
     std::set<std::string> SequencingChemistries() const;
 
+    /// \brief Return a minimal list of genomic intervals covered by filters.
+    ///
+    /// \returns vector of genomic intervals
+    ///
+    /// \throws std::runtime_error if DataSet contains invalid or non-sensical
+    ///         filters, such as rname appearing twice, etc.
+    ///
+    std::vector<GenomicInterval> GenomicIntervals() const;
+
     /// \}
 
 public:
@@ -753,7 +780,6 @@ public:
 
 private:
     std::unique_ptr<DataSetBase> d_;
-    std::string path_;
 };
 
 /// \name DataSet Timestamp Utilities
@@ -806,6 +832,4 @@ PBBAM_EXPORT std::string ToIso8601(const time_t& t);
 }  // namespace BAM
 }  // namespace PacBio
 
-#include "pbbam/internal/DataSet.inl"
-
 #endif  // DATASET_H
