@@ -7,131 +7,114 @@
 #ifndef DATASETTYPES_H
 #define DATASETTYPES_H
 
-#include <string>
-#include "pbbam/BamFile.h"
 #include "pbbam/Config.h"
+
+#include <string>
+
+#include "pbbam/BamFile.h"
 #include "pbbam/DataSetXsd.h"
 #include "pbbam/internal/DataSetBaseTypes.h"
 
 namespace PacBio {
 namespace BAM {
 
-/// \brief The DataSetMetadata class represents the %DataSetMetadata child
-///        element in DataSetXML.
 ///
-/// A few top-level elements are built-in, but as pbbam is not primarily a
-/// DataSetXML API, most of the metadata hierarchy needs to be manually managed.
+/// \brief The DNABarcode class represents a %DNABarcode element in
+///        DataSetXML, consisting of a Name and optional UniqueId.
 ///
-class PBBAM_EXPORT DataSetMetadata : public internal::DataSetElement
+class PBBAM_EXPORT DNABarcode : public internal::DataSetElement
 {
 public:
-    /// \name Constructors & Related Methods
-    /// \{
+    DNABarcode(const std::string& name);
+    DNABarcode(const std::string& name, const std::string& uuid);
+    DNABarcode(const std::string& name, const internal::FromInputXml& fromInputXml);
+    DNABarcode(const std::string& name, const std::string& uuid,
+               const internal::FromInputXml& fromInputXml);
 
-    /// \brief Constructs a DataSetMetadata with required fields.
-    DataSetMetadata(const std::string& numRecords, const std::string& totalLength);
+    const std::string& Name() const;
+    std::string& Name();
+    DNABarcode& Name(const std::string& name);
 
-    /// \}
+    const std::string& UniqueId() const;
+    std::string& UniqueId();
+    DNABarcode& UniqueId(const std::string& name);
+};
+
+/// \brief The DNABarcodes class represents an %DNABarcodes element in DataSetXML.
+///
+/// The DNABarcodes element is essentially just a list of DNABarcode
+/// objects.
+///
+class PBBAM_EXPORT DNABarcodes : public internal::DataSetElement
+{
+public:
+    DNABarcodes();
+    DNABarcodes(const internal::FromInputXml& fromInputXml);
 
 public:
-    /// \name Operators
-    /// \{
-
-    /// \brief Merges DataSetMetadata contents.
-    ///
-    /// Adds contents of \p other to this metadata object
-    ///
-    /// \param[in] other  some other metadata to add to this one
-    /// \returns reference to this object
-    ///
-    DataSetMetadata& operator+=(const DataSetMetadata& other);
-
-    /// \}
+    void Add(const DNABarcode& barcode);
+    void Remove(const DNABarcode& barcode);
 
 public:
-    /// \name Child Elements
-    /// \{
+    using value_type = DNABarcode;
+    using iterator_type = internal::DataSetElementIterator<value_type>;
+    using const_iterator_type = internal::DataSetElementConstIterator<value_type>;
 
-    /// \brief Fetches the text of the NumRecords element.
-    ///
-    /// \returns const reference to element text (empty string if not present)
-    ///
-    const std::string& NumRecords() const;
+    const value_type& operator[](size_t index) const;
+    value_type& operator[](size_t index);
 
-    /// \brief Fetches the text of the TotalLength element.
-    ///
-    /// \returns const reference to element text (empty string if not present)
-    ///
-    const std::string& TotalLength() const;
+    iterator_type begin();
+    const_iterator_type begin() const;
+    const_iterator_type cbegin() const;
+    iterator_type end();
+    const_iterator_type end() const;
+    const_iterator_type cend() const;
+};
 
-    /// \brief Fetches the Provenance element.
-    ///
-    /// \returns const reference to child element
-    /// \throws std::runtime_error if element does not exist
-    ///
-    const PacBio::BAM::Provenance& Provenance() const;
+class PBBAM_EXPORT BioSample : public internal::DataSetElement
+{
+public:
+    BioSample(const std::string& name);
+    BioSample(const std::string& name, const internal::FromInputXml& fromInputXml);
 
-    /// \}
+    const PacBio::BAM::DNABarcodes& DNABarcodes() const;
+    PacBio::BAM::DNABarcodes& DNABarcodes();
+    BioSample& DNABarcodes(const PacBio::BAM::DNABarcodes& barcodes);
+
+    const std::string& Name() const;
+    std::string& Name();
+    BioSample& Name(const std::string& name);
+};
+
+/// \brief The DNABarcodes class represents an %DNABarcodes element in DataSetXML.
+///
+/// The DNABarcodes element is essentially just a list of DNABarcode
+/// objects.
+///
+class PBBAM_EXPORT BioSamples : public internal::DataSetElement
+{
+public:
+    BioSamples();
+    BioSamples(const internal::FromInputXml& fromInputXml);
 
 public:
-    /// \name Child Elements
-    /// \{
-
-    /// \brief Fetches the text of the NumRecords element.
-    ///
-    /// This element will be created if it does not yet exist.
-    ///
-    /// \returns non-const reference to element text
-    ///
-    std::string& NumRecords();
-
-    /// \brief Fetches the text of the TotalLength element.
-    ///
-    /// This element will be created if it does not yet exist.
-    ///
-    /// \returns non-const reference to element text
-    ///
-    std::string& TotalLength();
-
-    /// \brief Fetches Provenance element.
-    ///
-    /// This element will be created if it does not yet exist.
-    ///
-    /// \returns non-const reference to child element
-    ///
-    PacBio::BAM::Provenance& Provenance();
-
-    /// \}
+    void Add(const BioSample& sample);
+    void Remove(const BioSample& sample);
 
 public:
-    /// \name Child Elements
-    /// \{
+    using value_type = BioSample;
+    using iterator_type = internal::DataSetElementIterator<value_type>;
+    using const_iterator_type = internal::DataSetElementConstIterator<value_type>;
 
-    /// \brief Sets the text of the NumRecords element.
-    ///
-    /// This element will be created if it does not yet exist.
-    ///
-    /// \returns reference to this metadata object
-    ///
-    DataSetMetadata& NumRecords(const std::string& numRecords);
+    const value_type& operator[](size_t index) const;
+    value_type& operator[](size_t index);
 
-    /// \brief Sets the text of the TotalLength element.
-    ///
-    /// This element will be created if it does not yet exist.
-    ///
-    /// \returns reference to this metadata object
-    ///
-    DataSetMetadata& TotalLength(const std::string& totalLength);
-
-    /// \brief Sets the Provenance child element.
-    ///
-    /// This element will be created if it does not yet exist.
-    ///
-    /// \returns reference to this metadata object
-    ///
-    DataSetMetadata& Provenance(const PacBio::BAM::Provenance& provenance);
-
-    /// \}
+    iterator_type begin();
+    const_iterator_type begin() const;
+    const_iterator_type cbegin() const;
+    iterator_type end();
+    const_iterator_type end() const;
+    const_iterator_type cend() const;
 };
 
 /// \brief The ExtensionElement class represents an %ExtensionElement element in
@@ -141,6 +124,7 @@ class PBBAM_EXPORT ExtensionElement : public internal::DataSetElement
 {
 public:
     ExtensionElement();
+    ExtensionElement(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The Extensions class represents an %Extensions element in DataSetXML.
@@ -148,11 +132,27 @@ public:
 /// The Extensions element is essentially just a list of ExtensionElement
 /// objects.
 ///
-class PBBAM_EXPORT Extensions : public internal::DataSetListElement<ExtensionElement>
+class PBBAM_EXPORT Extensions : public internal::DataSetElement
 {
 public:
     /// \brief Creates an empty extensions list.
     Extensions();
+    Extensions(const internal::FromInputXml& fromInputXml);
+
+public:
+    using value_type = ExtensionElement;
+    using iterator_type = internal::DataSetElementIterator<value_type>;
+    using const_iterator_type = internal::DataSetElementConstIterator<value_type>;
+
+    const value_type& operator[](size_t index) const;
+    value_type& operator[](size_t index);
+
+    iterator_type begin();
+    const_iterator_type begin() const;
+    const_iterator_type cbegin() const;
+    iterator_type end();
+    const_iterator_type end() const;
+    const_iterator_type cend() const;
 };
 
 class ExternalResources;
@@ -177,6 +177,9 @@ public:
     ///
     ExternalResource(const std::string& metatype, const std::string& filename);
 
+    ExternalResource(const std::string& metatype, const std::string& filename,
+                     const internal::FromInputXml& fromInputXml);
+
 public:
     /// \brief Fetches the resource's ExternalResources child element.
     ///
@@ -223,11 +226,12 @@ public:
 /// The ExternalResources element is essentially just a list of ExternalResource
 /// elements.
 ///
-class PBBAM_EXPORT ExternalResources : public internal::DataSetListElement<ExternalResource>
+class PBBAM_EXPORT ExternalResources : public internal::DataSetElement
 {
 public:
     /// \brief Creates an empty resource list.
     ExternalResources();
+    ExternalResources(const internal::FromInputXml& fromInputXml);
 
     /// \brief Merges \p other resource list with this one.
     ExternalResources& operator+=(const ExternalResources& other);
@@ -246,6 +250,21 @@ public:
     ///             relative filepaths and will be removed in the near future.
     ///
     std::vector<BamFile> BamFiles() const;
+
+public:
+    using value_type = ExternalResource;
+    using iterator_type = internal::DataSetElementIterator<value_type>;
+    using const_iterator_type = internal::DataSetElementConstIterator<value_type>;
+
+    const value_type& operator[](size_t index) const;
+    value_type& operator[](size_t index);
+
+    iterator_type begin();
+    const_iterator_type begin() const;
+    const_iterator_type cbegin() const;
+    iterator_type end();
+    const_iterator_type end() const;
+    const_iterator_type cend() const;
 };
 
 /// \brief The FileIndex class represents a %FileIndex element in DataSetXML.
@@ -261,6 +280,9 @@ public:
     ///        resource ID.
     ///
     FileIndex(const std::string& metatype, const std::string& filename);
+
+    FileIndex(const std::string& metatype, const std::string& filename,
+              const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The FileIndices class represents a %FileIndices element in DataSetXML.
@@ -269,11 +291,12 @@ public:
 /// providing information about a data file's index files (e.g. for %BAM files
 /// this will usually be *.bai and/or *.pbi).
 ///
-class PBBAM_EXPORT FileIndices : public internal::DataSetListElement<FileIndex>
+class PBBAM_EXPORT FileIndices : public internal::DataSetElement
 {
 public:
     /// \brief Creates an empty index list.
     FileIndices();
+    FileIndices(const internal::FromInputXml& fromInputXml);
 
 public:
     /// \brief Adds a FileIndex to this list.
@@ -281,6 +304,21 @@ public:
 
     /// \brief Removes a FileIndex from this list.
     void Remove(const FileIndex& index);
+
+public:
+    using value_type = FileIndex;
+    using iterator_type = internal::DataSetElementIterator<value_type>;
+    using const_iterator_type = internal::DataSetElementConstIterator<value_type>;
+
+    const value_type& operator[](size_t index) const;
+    value_type& operator[](size_t index);
+
+    iterator_type begin();
+    const_iterator_type begin() const;
+    const_iterator_type cbegin() const;
+    iterator_type end();
+    const_iterator_type end() const;
+    const_iterator_type cend() const;
 };
 
 /// \brief The Filter class represents a %Filter element in DataSetXML.
@@ -298,6 +336,7 @@ class PBBAM_EXPORT Filter : public internal::DataSetElement
 public:
     /// \brief Creates an empty filter.
     Filter();
+    Filter(const internal::FromInputXml& fromInputXml);
 
 public:
     /// \brief Fetches the filter's property list element.
@@ -332,11 +371,12 @@ public:
 /// purpose, each filter is considered separately (logical OR) to consider which
 /// data passes, e.g. filter1 || filter2 || filter3.
 ///
-class PBBAM_EXPORT Filters : public internal::DataSetListElement<Filter>
+class PBBAM_EXPORT Filters : public internal::DataSetElement
 {
 public:
     /// \brief Creates an empty filter list.
     Filters();
+    Filters(const internal::FromInputXml& fromInputXml);
 
     /// \brief Merges \p other filter list with this one.
     Filters& operator+=(const Filters& other);
@@ -347,6 +387,21 @@ public:
 
     /// \brief Removes a filter from this list.
     void Remove(const Filter& filter);
+
+public:
+    using value_type = Filter;
+    using iterator_type = internal::DataSetElementIterator<value_type>;
+    using const_iterator_type = internal::DataSetElementConstIterator<value_type>;
+
+    const value_type& operator[](size_t index) const;
+    value_type& operator[](size_t index);
+
+    iterator_type begin();
+    const_iterator_type begin() const;
+    const_iterator_type cbegin() const;
+    iterator_type end();
+    const_iterator_type end() const;
+    const_iterator_type cend() const;
 };
 
 /// \brief The ParentTool class represents a %ParentTool element in DataSetXML.
@@ -356,6 +411,7 @@ class PBBAM_EXPORT ParentTool : public internal::BaseEntityType
 public:
     /// \brief Creates an empty %ParentTool element.
     ParentTool();
+    ParentTool(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The Property class represents a %Property element in DataSetXML.
@@ -375,6 +431,8 @@ class PBBAM_EXPORT Property : public internal::DataSetElement
 public:
     /// \brief Constructs a filter property.
     Property(const std::string& name, const std::string& value, const std::string& op);
+    Property(const std::string& name, const std::string& value, const std::string& op,
+             const internal::FromInputXml& fromInputXml);
 
 public:
     /// \brief Fetches the value of property's Name attribute.
@@ -442,11 +500,12 @@ public:
 ///
 /// The Properties element is essentially a list of Property elements.
 ///
-class PBBAM_EXPORT Properties : public internal::DataSetListElement<Property>
+class PBBAM_EXPORT Properties : public internal::DataSetElement
 {
 public:
     /// \brief Creates an empty property list.
     Properties();
+    Properties(const internal::FromInputXml& fromInputXml);
 
 public:
     /// \brief Adds a property to this list.
@@ -454,6 +513,21 @@ public:
 
     /// \brief Removes a property from this list.
     void Remove(const Property& property);
+
+public:
+    using value_type = Property;
+    using iterator_type = internal::DataSetElementIterator<value_type>;
+    using const_iterator_type = internal::DataSetElementConstIterator<value_type>;
+
+    const value_type& operator[](size_t index) const;
+    value_type& operator[](size_t index);
+
+    iterator_type begin();
+    const_iterator_type begin() const;
+    const_iterator_type cbegin() const;
+    iterator_type end();
+    const_iterator_type end() const;
+    const_iterator_type cend() const;
 };
 
 /// \brief The Provenance class represents a %Provenance element in DataSetXML.
@@ -463,6 +537,7 @@ class PBBAM_EXPORT Provenance : public internal::DataSetElement
 public:
     /// \brief Creates a empty provenance element.
     Provenance();
+    Provenance(const internal::FromInputXml& fromInputXml);
 
 public:
     /// \brief Fetches the value of CreatedBy attribute.
@@ -592,6 +667,152 @@ public:
     Provenance& ParentTool(const PacBio::BAM::ParentTool& tool);
 };
 
+/// \brief The DataSetMetadata class represents the %DataSetMetadata child
+///        element in DataSetXML.
+///
+/// A few top-level elements are built-in, but as pbbam is not primarily a
+/// DataSetXML API, most of the metadata hierarchy needs to be manually managed.
+///
+class PBBAM_EXPORT DataSetMetadata : public internal::DataSetElement
+{
+public:
+    /// \name Constructors & Related Methods
+    /// \{
+
+    DataSetMetadata();
+    DataSetMetadata(const internal::FromInputXml& fromInputXml);
+
+    /// \brief Constructs a DataSetMetadata with required fields.
+    DataSetMetadata(const std::string& numRecords, const std::string& totalLength);
+    DataSetMetadata(const std::string& numRecords, const std::string& totalLength,
+                    const internal::FromInputXml& fromInputXml);
+
+    /// \}
+
+public:
+    /// \name Operators
+    /// \{
+
+    /// \brief Merges DataSetMetadata contents.
+    ///
+    /// Adds contents of \p other to this metadata object
+    ///
+    /// \param[in] other  some other metadata to add to this one
+    /// \returns reference to this object
+    ///
+    DataSetMetadata& operator+=(const DataSetMetadata& other);
+
+    /// \}
+
+public:
+    /// \name Child Elements
+    /// \{
+
+    /// \brief Fetches the text of the NumRecords element.
+    ///
+    /// \returns const reference to element text (empty string if not present)
+    ///
+    const std::string& NumRecords() const;
+
+    /// \brief Fetches the text of the TotalLength element.
+    ///
+    /// \returns const reference to element text (empty string if not present)
+    ///
+    const std::string& TotalLength() const;
+
+    /// \brief Fetches the Provenance element.
+    ///
+    /// \returns const reference to child element
+    /// \throws std::runtime_error if element does not exist
+    ///
+    const PacBio::BAM::Provenance& Provenance() const;
+
+    /// \brief Fetches the BioSamples element.
+    ///
+    /// \returns const reference to child element
+    /// \throws std::runtime_error if element does not exist
+    ///
+    const PacBio::BAM::BioSamples& BioSamples() const;
+
+    /// \}
+
+public:
+    /// \name Child Elements
+    /// \{
+
+    /// \brief Fetches the text of the NumRecords element.
+    ///
+    /// This element will be created if it does not yet exist.
+    ///
+    /// \returns non-const reference to element text
+    ///
+    std::string& NumRecords();
+
+    /// \brief Fetches the text of the TotalLength element.
+    ///
+    /// This element will be created if it does not yet exist.
+    ///
+    /// \returns non-const reference to element text
+    ///
+    std::string& TotalLength();
+
+    /// \brief Fetches Provenance element.
+    ///
+    /// This element will be created if it does not yet exist.
+    ///
+    /// \returns non-const reference to child element
+    ///
+    PacBio::BAM::Provenance& Provenance();
+
+    /// \brief Fetches BioSamples element.
+    ///
+    /// This element will be created if it does not yet exist.
+    ///
+    /// \returns non-const reference to child element
+    ///
+    PacBio::BAM::BioSamples& BioSamples();
+
+    /// \}
+
+public:
+    /// \name Child Elements
+    /// \{
+
+    /// \brief Sets the text of the NumRecords element.
+    ///
+    /// This element will be created if it does not yet exist.
+    ///
+    /// \returns reference to this metadata object
+    ///
+    DataSetMetadata& NumRecords(const std::string& numRecords);
+
+    /// \brief Sets the text of the TotalLength element.
+    ///
+    /// This element will be created if it does not yet exist.
+    ///
+    /// \returns reference to this metadata object
+    ///
+    DataSetMetadata& TotalLength(const std::string& totalLength);
+
+    /// \brief Sets the Provenance child element.
+    ///
+    /// This element will be created if it does not yet exist.
+    ///
+    /// \returns reference to this metadata object
+    ///
+    DataSetMetadata& Provenance(const PacBio::BAM::Provenance& provenance);
+
+    /// \brief Sets the BioSamples child element.
+    ///
+    /// This element will be created if it does not yet exist.
+    ///
+    /// \returns reference to this metadata object
+    ///
+    DataSetMetadata& BioSamples(const PacBio::BAM::BioSamples& samples);
+
+    /// \}
+};
+
 class SubDataSets;
 
 /// \brief The DataSetBase class provides the attributes & child elements shared
@@ -609,14 +830,19 @@ public:
     ///        XML element name (e.g. SubreadSet)
     ///
     static std::shared_ptr<DataSetBase> Create(const std::string& typeName);
+    static std::shared_ptr<DataSetBase> Create(const std::string& typeName,
+                                               const internal::FromInputXml& fromInputXml);
 
 public:
     /// \brief Creates an empty, generic DataSetBase.
     DataSetBase();
+    DataSetBase(const internal::FromInputXml& fromInputXml);
 
 protected:
     /// \brief Creates a DataSetBase with key values initialized.
     DataSetBase(const std::string& metatype, const std::string& label, const XsdType& xsd);
+    DataSetBase(const std::string& metatype, const std::string& label,
+                const internal::FromInputXml& fromInputXml, const XsdType& xsd);
 
     /// \brief Returns a new DataSetBase containing a deep copy of contents
     DataSetBase* DeepCopy() const;
@@ -741,8 +967,51 @@ public:
     ///
     NamespaceRegistry& Namespaces();
 
+public:
+    /// \brief Saves dataset XML to file.
+    ///
+    /// \param[in] outputFilename destination for XML contents
+    ///
+    /// \throws std::runtime_error if file could be opened or if DataSet
+    ///         elements could not be converted to XML
+    ///
+    void Save(const std::string& outputFilename);
+
+    /// \brief Saves dataset XML to output stream, e.g. std::cout,
+    ///        std::stringstream.
+    ///
+    /// \param[out] out destination for XML contents
+    ///
+    /// \throws std::runtime_error if DataSet elements could not be converted to
+    ///         XML
+    ///
+    void SaveToStream(std::ostream& out);
+
+public:
+    ///
+    /// \returns true if dataset was read from XML input
+    ///
+    bool FromInputXml() const;
+
+    ///
+    /// \brief Indicate that dataset was read from XML input
+    ///
+    void FromInputXml(bool ok);
+
+    ///
+    /// \returns (absolute) path for dataset
+    ///
+    const std::string& Path() const;
+
+    ///
+    /// \brief Set dataset path
+    ///
+    void Path(const std::string& path);
+
 private:
     NamespaceRegistry registry_;
+    std::string path_;
+    bool fromInputXml_ = false;
 };
 
 /// \brief The AlignmentSet class represents an %AlignmentSet root element in
@@ -753,6 +1022,7 @@ class PBBAM_EXPORT AlignmentSet : public DataSetBase
 public:
     /// \brief Creates an empty AlignmentSet dataset.
     AlignmentSet();
+    AlignmentSet(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The BarcodeSet class represents a %BarcodeSet root element in
@@ -763,6 +1033,7 @@ class PBBAM_EXPORT BarcodeSet : public DataSetBase
 public:
     /// \brief Creates an empty BarcodeSet dataset.
     BarcodeSet();
+    BarcodeSet(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The ConsensusAlignmentSet class represents a %ConsensusAlignmentSet
@@ -773,6 +1044,7 @@ class PBBAM_EXPORT ConsensusAlignmentSet : public DataSetBase
 public:
     /// \brief Creates an empty ConsensusAlignmentSet dataset.
     ConsensusAlignmentSet();
+    ConsensusAlignmentSet(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The ConsensusReadSet class represents a %ConsensusReadSet root
@@ -783,6 +1055,7 @@ class PBBAM_EXPORT ConsensusReadSet : public DataSetBase
 public:
     /// \brief Creates an empty ConsensusReadSet dataset.
     ConsensusReadSet();
+    ConsensusReadSet(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The ContigSet class represents a %ContigSet root element in
@@ -793,6 +1066,7 @@ class PBBAM_EXPORT ContigSet : public DataSetBase
 public:
     /// \brief Creates an empty ContigSet dataset.
     ContigSet();
+    ContigSet(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The HdfSubreadSet class represents a %HdfSubreadSet root element in
@@ -803,6 +1077,7 @@ class PBBAM_EXPORT HdfSubreadSet : public DataSetBase
 public:
     /// \brief Creates an empty HdfSubreadSet dataset.
     HdfSubreadSet();
+    HdfSubreadSet(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The ReferenceSet class represents a %ReferenceSet root element in
@@ -813,6 +1088,7 @@ class PBBAM_EXPORT ReferenceSet : public DataSetBase
 public:
     /// \brief Creates an empty ReferenceSet dataset.
     ReferenceSet();
+    ReferenceSet(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The SubDataSets class represents a %DataSets list element in
@@ -820,11 +1096,12 @@ public:
 ///
 /// The SubDataSets element is essentially a list of DataSets.
 ///
-class PBBAM_EXPORT SubDataSets : public internal::DataSetListElement<DataSetBase>
+class PBBAM_EXPORT SubDataSets : public internal::DataSetElement
 {
 public:
     /// \brief Creates an empty list of sub-datasets.
     SubDataSets();
+    SubDataSets(const internal::FromInputXml& fromInputXml);
 
 public:
     /// \brief Adds \p other sub-dataset to this list.
@@ -839,6 +1116,21 @@ public:
 
     /// \brief Removes a sub-dataset from this list.
     void Remove(const DataSetBase& subdataset);
+
+public:
+    using value_type = DataSetBase;
+    using iterator_type = internal::DataSetElementIterator<value_type>;
+    using const_iterator_type = internal::DataSetElementConstIterator<value_type>;
+
+    const value_type& operator[](size_t index) const;
+    value_type& operator[](size_t index);
+
+    iterator_type begin();
+    const_iterator_type begin() const;
+    const_iterator_type cbegin() const;
+    iterator_type end();
+    const_iterator_type end() const;
+    const_iterator_type cend() const;
 };
 
 /// \brief The SubreadSet class represents a %SubreadSet root element in
@@ -849,6 +1141,7 @@ class PBBAM_EXPORT SubreadSet : public DataSetBase
 public:
     /// \brief Creates an empty SubreadSet dataset.
     SubreadSet();
+    SubreadSet(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The TranscriptSet class represents a %TranscriptSet root element in
@@ -859,6 +1152,7 @@ class PBBAM_EXPORT TranscriptSet : public DataSetBase
 public:
     /// \brief Creates an empty TranscriptSet dataset.
     TranscriptSet();
+    TranscriptSet(const internal::FromInputXml& fromInputXml);
 };
 
 /// \brief The TranscriptAlignmentSet class represents a %TranscriptAlignmentSet
@@ -869,11 +1163,49 @@ class PBBAM_EXPORT TranscriptAlignmentSet : public DataSetBase
 public:
     /// \brief Creates an empty TranscriptAlignmentSet dataset.
     TranscriptAlignmentSet();
+    TranscriptAlignmentSet(const internal::FromInputXml& fromInputXml);
 };
 
+enum class XmlElementType
+{
+    GENERIC_ELEMENT,
+    DATASET_METADATA,
+    BIOSAMPLE,
+    BIOSAMPLES,
+    DNA_BARCODE,
+    DNA_BARCODES,
+    EXTENSION,
+    EXTENSIONS,
+    EXTERNAL_RESOURCE,
+    EXTERNAL_RESOURCES,
+    FILE_INDEX,
+    FILE_INDICES,
+    FILTER,
+    FILTERS,
+    PARENT_TOOL,
+    PROPERTY,
+    PROPERTIES,
+    PROVENANCE,
+
+    GENERIC_DATASET,
+    ALIGNMENT_SET,
+    BARCODE_SET,
+    CONSENSUS_ALIGNMENT_SET,
+    CONSENSUS_READ_SET,
+    CONTIG_SET,
+    HDF_SUBREAD_SET,
+    REFERENCE_SET,
+    SUBREAD_SET,
+    TRANSCRIPT_SET,
+    TRANSCRIPT_ALIGNMENT_SET,
+    SUBDATASETS
+};
+
+/// \returns the enum value for the requested XML element
+///          (generic if not a built-in element type)
+XmlElementType ElementTypeFromName(const std::string& name);
+
 }  // namespace BAM
 }  // namespace PacBio
 
-#include "internal/DataSetTypes.inl"
-
 #endif  // DATASETTYPES_H
