@@ -7,6 +7,8 @@
 #ifndef FASTASEQUENCE_H
 #define FASTASEQUENCE_H
 
+#include "pbbam/Config.h"
+
 #include <string>
 
 namespace PacBio {
@@ -29,11 +31,6 @@ public:
     explicit FastaSequence(std::string name, std::string bases);
 
     FastaSequence() = default;
-    FastaSequence(const FastaSequence&) = default;
-    FastaSequence(FastaSequence&&) = default;
-    FastaSequence& operator=(const FastaSequence&) = default;
-    FastaSequence& operator=(FastaSequence&&) = default;
-    ~FastaSequence() = default;
 
     /// \}
 
@@ -47,14 +44,33 @@ public:
     ///
     const std::string& Name() const;
 
+    ///
+    /// \brief
+    ///
+    /// \param name
+    /// \return FastaSequence&
+    ///
+    FastaSequence& Name(std::string name);
+
     ///
     /// \brief Bases
     /// \return
     ///
     const std::string& Bases() const;
 
+    ///
+    /// \brief
+    ///
+    /// \param bases
+    /// \return FastaSequence&
+    ///
+    FastaSequence& Bases(std::string bases);
+
     /// \}
 
+    bool operator==(const FastaSequence& other) const;
+    bool operator!=(const FastaSequence& other) const;
+
 private:
     std::string name_;
     std::string bases_;
@@ -63,6 +79,4 @@ private:
 }  // namespace BAM
 }  // namespace PacBio
 
-#include "internal/FastaSequence.inl"
-
 #endif  // FASTASEQUENCE_H
