@@ -7,13 +7,15 @@
 #ifndef FASTASEQUENCEQUERY_H
 #define FASTASEQUENCEQUERY_H
 
-#include "pbbam/DataSet.h"
-#include "pbbam/FastaSequence.h"
-#include "pbbam/internal/QueryBase.h"
+#include "pbbam/Config.h"
 
 #include <memory>
 #include <string>
 
+#include "pbbam/DataSet.h"
+#include "pbbam/FastaSequence.h"
+#include "pbbam/internal/QueryBase.h"
+
 namespace PacBio {
 namespace BAM {
 
@@ -40,7 +42,7 @@ public:
     bool GetNext(FastaSequence& seq) override;
 
 private:
-    struct FastaSequenceQueryPrivate;
+    class FastaSequenceQueryPrivate;
     std::unique_ptr<FastaSequenceQueryPrivate> d_;
 };
 
