@@ -7,11 +7,16 @@
 #ifndef PBIFILTERTYPES_H
 #define PBIFILTERTYPES_H
 
-#include <boost/optional.hpp>
+#include "pbbam/Config.h"
+
 #include <cstddef>
 #include <cstdint>
 #include <memory>
 #include <string>
+#include <unordered_map>
+
+#include <boost/optional.hpp>
+
 #include "pbbam/Compare.h"
 #include "pbbam/PbiFile.h"
 #include "pbbam/PbiFilter.h"
@@ -26,18 +31,17 @@ namespace internal {
 /// Provides basic container for value/compare-type pair
 ///
 template <typename T>
-struct FilterBase
+class FilterBase
 {
 public:
     T value_;
-    boost::optional<std::vector<T> > multiValue_;
+    boost::optional<std::vector<T>> multiValue_;
     Compare::Type cmp_;
 
 protected:
     FilterBase(T value, const Compare::Type cmp);
-    FilterBase(std::vector<T> values, const Compare::Type cmp = Compare::EQUAL);
+    FilterBase(std::vector<T> values, const Compare::Type cmp = Compare::CONTAINS);
 
-protected:
     bool CompareHelper(const T& lhs) const;
 
 private:
@@ -50,14 +54,14 @@ private:
 /// Dispatches the lookup to BarcodeLookupData
 ///
 template <typename T, PbiFile::BarcodeField field>
-struct BarcodeDataFilterBase : public FilterBase<T>
+class BarcodeDataFilterBase : public FilterBase<T>
 {
-protected:
-    BarcodeDataFilterBase(T value, const Compare::Type cmp);
-    BarcodeDataFilterBase(std::vector<T> values, const Compare::Type cmp = Compare::EQUAL);
-
 public:
     bool Accepts(const PbiRawData& idx, const size_t row) const;
+
+protected:
+    BarcodeDataFilterBase(T value, const Compare::Type cmp);
+    BarcodeDataFilterBase(std::vector<T> values, const Compare::Type cmp = Compare::CONTAINS);
 };
 
 /// \internal
@@ -65,14 +69,14 @@ public:
 /// Dispatches the lookup to BasicLookupData
 ///
 template <typename T, PbiFile::BasicField field>
-struct BasicDataFilterBase : public FilterBase<T>
+class BasicDataFilterBase : public FilterBase<T>
 {
-protected:
-    BasicDataFilterBase(T value, const Compare::Type cmp);
-    BasicDataFilterBase(std::vector<T> values, const Compare::Type cmp = Compare::EQUAL);
-
 public:
     bool Accepts(const PbiRawData& idx, const size_t row) const;
+
+protected:
+    BasicDataFilterBase(T value, const Compare::Type cmp);
+    BasicDataFilterBase(std::vector<T> values, const Compare::Type cmp = Compare::CONTAINS);
 };
 
 /// \internal
@@ -80,14 +84,14 @@ public:
 /// Dispatches the lookup to MappedLookupData
 ///
 template <typename T, PbiFile::MappedField field>
-struct MappedDataFilterBase : public FilterBase<T>
+class MappedDataFilterBase : public FilterBase<T>
 {
-protected:
-    MappedDataFilterBase(T value, const Compare::Type cmp);
-    MappedDataFilterBase(std::vector<T> values, const Compare::Type cmp = Compare::EQUAL);
-
 public:
     bool Accepts(const PbiRawData& idx, const size_t row) const;
+
+protected:
+    MappedDataFilterBase(T value, const Compare::Type cmp);
+    MappedDataFilterBase(std::vector<T> values, const Compare::Type cmp = Compare::CONTAINS);
 };
 
 }  // namespace internal
@@ -99,7 +103,7 @@ public:
 ///
 /// \sa BamRecord::AlignedEnd
 ///
-struct PbiAlignedEndFilter
+class PbiAlignedEndFilter
     : public internal::MappedDataFilterBase<uint32_t, PbiFile::MappedField::A_END>
 {
 public:
@@ -118,7 +122,7 @@ public:
 ///
 /// \sa BamRecord::AlignedEnd, BamRecord::AlignedStart
 ///
-struct PbiAlignedLengthFilter : public internal::FilterBase<uint32_t>
+class PbiAlignedLengthFilter : public internal::FilterBase<uint32_t>
 {
 public:
     /// \brief Creates a filter on aligned length.
@@ -128,7 +132,6 @@ public:
     ///
     PbiAlignedLengthFilter(const uint32_t length, const Compare::Type cmp = Compare::EQUAL);
 
-public:
     /// \brief Performs the actual index lookup.
     ///
     /// Most client code should not need to use this method directly.
@@ -143,7 +146,7 @@ public:
 ///
 /// \sa BamRecord::AlignedStart
 ///
-struct PbiAlignedStartFilter
+class PbiAlignedStartFilter
     : public internal::MappedDataFilterBase<uint32_t, PbiFile::MappedField::A_START>
 {
 public:
@@ -162,7 +165,7 @@ public:
 ///
 /// \sa BamRecord::AlignedStrand
 ///
-struct PbiAlignedStrandFilter
+class PbiAlignedStrandFilter
     : public internal::MappedDataFilterBase<Strand, PbiFile::MappedField::STRAND>
 {
 public:
@@ -183,7 +186,7 @@ public:
 ///
 /// \sa BamRecord::BarcodeForward, BamRecord::BarcodeReverse
 ///
-struct PbiBarcodeFilter
+class PbiBarcodeFilter
 {
 public:
     /// \brief Creates a single-value barcode filter.
@@ -193,17 +196,18 @@ public:
     ///
     PbiBarcodeFilter(const int16_t barcode, const Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' barcode filter.
+    /// \brief Creates a whitelisted or blacklisted barcode filter.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly, in either bc_forward or bc_reverse.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no barcodes in the blacklist.
     ///
-    /// \param[in] whitelist  barcode IDs to compare on
+    /// \param[in] barcodes barcode IDs
+    /// \param[in] cmp      compare type
     ///
-    PbiBarcodeFilter(std::vector<int16_t> whitelist, const Compare::Type cmp = Compare::EQUAL);
+    PbiBarcodeFilter(std::vector<int16_t> barcodes, const Compare::Type cmp = Compare::CONTAINS);
 
-public:
     /// \brief Performs the actual index lookup.
     ///
     /// Most client code should not need to use this method directly.
@@ -221,7 +225,7 @@ private:
 ///
 /// \sa BamRecord::BarcodeForward
 ///
-struct PbiBarcodeForwardFilter
+class PbiBarcodeForwardFilter
     : public internal::BarcodeDataFilterBase<int16_t, PbiFile::BarcodeField::BC_FORWARD>
 {
 public:
@@ -232,16 +236,18 @@ public:
     ///
     PbiBarcodeForwardFilter(const int16_t bcFwdId, const Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' forward barcode filter.
+    /// \brief Creates a whitelisted or blacklisted forward barcode filter.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly, in bc_forward.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
     ///
-    /// \param[in] whitelist  barcode IDs to compare on
+    /// \param[in] barcodes barcode IDs
+    /// \param[in] cmp      compare type
     ///
-    PbiBarcodeForwardFilter(std::vector<int16_t> whitelist,
-                            const Compare::Type cmp = Compare::EQUAL);
+    PbiBarcodeForwardFilter(std::vector<int16_t> barcodes,
+                            const Compare::Type cmp = Compare::CONTAINS);
 };
 
 /// \brief The PbiBarcodeQualityFilter class provides a PbiFilter-compatible
@@ -251,7 +257,7 @@ public:
 ///
 /// \sa BamRecord::BarcodeQuality
 ///
-struct PbiBarcodeQualityFilter
+class PbiBarcodeQualityFilter
     : public internal::BarcodeDataFilterBase<uint8_t, PbiFile::BarcodeField::BC_QUALITY>
 {
 public:
@@ -270,7 +276,7 @@ public:
 ///
 /// \sa BamRecord::BarcodeReverse
 ///
-struct PbiBarcodeReverseFilter
+class PbiBarcodeReverseFilter
     : public internal::BarcodeDataFilterBase<int16_t, PbiFile::BarcodeField::BC_REVERSE>
 {
 public:
@@ -281,16 +287,18 @@ public:
     ///
     PbiBarcodeReverseFilter(const int16_t bcRevId, const Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' reverse barcode filter.
+    /// \brief Creates a whitelisted or blacklisted reverse barcode filter.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly, in bc_reverse.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
     ///
-    /// \param[in] whitelist  barcode IDs to compare on
+    /// \param[in] barcodes barcode IDs
+    /// \param[in] cmp      compare type
     ///
-    PbiBarcodeReverseFilter(std::vector<int16_t> whitelist,
-                            const Compare::Type cmp = Compare::EQUAL);
+    PbiBarcodeReverseFilter(std::vector<int16_t> barcodes,
+                            const Compare::Type cmp = Compare::CONTAINS);
 };
 
 /// \brief The PbiBarcodesFilter class provides a PbiFilter-compatible filter on
@@ -302,7 +310,7 @@ public:
 ///
 /// \sa BamRecord::Barcodes
 ///
-struct PbiBarcodesFilter
+class PbiBarcodesFilter
 {
 public:
     /// \brief Creates a barcodes filter from a std::pair of IDs.
@@ -325,7 +333,6 @@ public:
     PbiBarcodesFilter(const int16_t bcForward, const int16_t bcReverse,
                       const Compare::Type cmp = Compare::EQUAL);
 
-public:
     /// \brief Performs the actual index lookup.
     ///
     /// Most client code should not need to use this method directly.
@@ -343,7 +350,7 @@ private:
 ///
 /// Example: \include code/PbiIdentityFilter.txt
 ///
-struct PbiIdentityFilter : public internal::FilterBase<float>
+class PbiIdentityFilter : public internal::FilterBase<float>
 {
 public:
     /// \brief Creates a read identity filter.
@@ -353,7 +360,6 @@ public:
     ///
     PbiIdentityFilter(const float identity, const Compare::Type cmp = Compare::EQUAL);
 
-public:
     /// \brief Performs the actual index lookup.
     ///
     /// Most client code should not need to use this method directly.
@@ -370,7 +376,7 @@ public:
 ///
 /// Example: \include code/PbiLocalContextFilter.txt
 ///
-struct PbiLocalContextFilter
+class PbiLocalContextFilter
     : public internal::BasicDataFilterBase<LocalContextFlags, PbiFile::BasicField::CONTEXT_FLAG>
 {
 public:
@@ -384,7 +390,7 @@ public:
 ///
 /// \sa BamRecord::MapQuality
 ///
-struct PbiMapQualityFilter
+class PbiMapQualityFilter
     : public internal::MappedDataFilterBase<uint8_t, PbiFile::MappedField::MAP_QUALITY>
 {
 public:
@@ -403,7 +409,7 @@ public:
 ///
 /// \sa BamRecord::MovieName
 ///
-struct PbiMovieNameFilter
+class PbiMovieNameFilter
 {
 public:
     /// \brief Creates a single-value movie name filter.
@@ -415,18 +421,19 @@ public:
     ///
     PbiMovieNameFilter(const std::string& movieName, const Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' movie name filter.
+    /// \brief Creates a whitelisted or blacklisted movie name filter.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
     ///
-    /// \param[in] whitelist    movie names to compare on
+    /// \param[in] movieNames   movie names
+    /// \param[in] cmp          compare type
     ///
-    PbiMovieNameFilter(const std::vector<std::string>& whitelist,
-                       const Compare::Type cmp = Compare::EQUAL);
+    PbiMovieNameFilter(const std::vector<std::string>& movieNames,
+                       const Compare::Type cmp = Compare::CONTAINS);
 
-public:
     /// \brief Performs the actual index lookup.
     ///
     /// Most client code should not need to use this method directly.
@@ -445,7 +452,7 @@ private:
 ///
 /// \sa BamRecord::NumDeletedBases
 ///
-struct PbiNumDeletedBasesFilter
+class PbiNumDeletedBasesFilter
     : public internal::MappedDataFilterBase<size_t, PbiFile::MappedField::N_DEL>
 {
 public:
@@ -464,7 +471,7 @@ public:
 ///
 /// \sa BamRecord::NumInsertedBases
 ///
-struct PbiNumInsertedBasesFilter
+class PbiNumInsertedBasesFilter
     : public internal::MappedDataFilterBase<size_t, PbiFile::MappedField::N_INS>
 {
 public:
@@ -483,8 +490,7 @@ public:
 ///
 /// \sa BamRecord::NumMatches
 ///
-struct PbiNumMatchesFilter
-    : public internal::MappedDataFilterBase<size_t, PbiFile::MappedField::N_M>
+class PbiNumMatchesFilter : public internal::MappedDataFilterBase<size_t, PbiFile::MappedField::N_M>
 {
 public:
     /// \brief Creates a filter on the number of matched bases.
@@ -502,7 +508,7 @@ public:
 ///
 /// \sa BamRecord::NumMismatches
 ///
-struct PbiNumMismatchesFilter
+class PbiNumMismatchesFilter
     : public internal::MappedDataFilterBase<size_t, PbiFile::MappedField::N_MM>
 {
 public:
@@ -522,7 +528,7 @@ public:
 ///
 /// \sa BamRecord::QueryEnd
 ///
-struct PbiQueryEndFilter : public internal::BasicDataFilterBase<int32_t, PbiFile::BasicField::Q_END>
+class PbiQueryEndFilter : public internal::BasicDataFilterBase<int32_t, PbiFile::BasicField::Q_END>
 {
 public:
     /// \brief Creates a filter on query end position.
@@ -542,7 +548,7 @@ public:
 ///
 /// \sa BamRecord::QueryEnd, BamRecord::QueryStart
 ///
-struct PbiQueryLengthFilter : public internal::FilterBase<int32_t>
+class PbiQueryLengthFilter : public internal::FilterBase<int32_t>
 {
 public:
     /// \brief Creates a filter on query length
@@ -552,7 +558,6 @@ public:
     ///
     PbiQueryLengthFilter(const int32_t length, const Compare::Type cmp = Compare::EQUAL);
 
-public:
     /// \brief Performs the actual index lookup.
     ///
     /// Most client code should not need to use this method directly.
@@ -567,7 +572,7 @@ public:
 ///
 /// \sa BamRecord::FullName
 ///
-struct PbiQueryNameFilter
+class PbiQueryNameFilter
 {
 public:
     /// \brief Creates a single-value query name filter.
@@ -579,21 +584,22 @@ public:
     ///
     PbiQueryNameFilter(const std::string& qname, const Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' query name filter.
+    /// \brief Creates a whitelisted or blacklisted query name filter.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
     ///
-    /// \param[in] whitelist    query names to compare on
+    /// \param[in] queryNames   query names
+    /// \param[in] cmp          compare type
     ///
-    PbiQueryNameFilter(const std::vector<std::string>& whitelist,
-                       const Compare::Type cmp = Compare::EQUAL);
+    PbiQueryNameFilter(const std::vector<std::string>& queryNames,
+                       const Compare::Type cmp = Compare::CONTAINS);
 
     PbiQueryNameFilter(const PbiQueryNameFilter& other);
     ~PbiQueryNameFilter();
 
-public:
     /// \brief Performs the actual index lookup.
     ///
     /// Most client code should not need to use this method directly.
@@ -612,7 +618,7 @@ private:
 ///
 /// \sa BamRecord::QueryStart
 ///
-struct PbiQueryStartFilter
+class PbiQueryStartFilter
     : public internal::BasicDataFilterBase<int32_t, PbiFile::BasicField::Q_START>
 {
 public:
@@ -631,7 +637,7 @@ public:
 ///
 /// \sa BamRecord::ReadAccuracy
 ///
-struct PbiReadAccuracyFilter
+class PbiReadAccuracyFilter
     : public internal::BasicDataFilterBase<Accuracy, PbiFile::BasicField::READ_QUALITY>
 {
 public:
@@ -652,8 +658,7 @@ public:
 ///     BamRecord::ReadGroupId,
 ///     BamRecord::ReadGroupNumericId
 ///
-struct PbiReadGroupFilter
-    : public internal::BasicDataFilterBase<int32_t, PbiFile::BasicField::RG_ID>
+class PbiReadGroupFilter
 {
 public:
     /// \brief Creates a filter on read group (numeric) ID value
@@ -683,37 +688,55 @@ public:
     ///
     PbiReadGroupFilter(const ReadGroupInfo& rg, const Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' filter on read group numeric IDs.
+    /// \brief Creates a whitelisted or blacklisted filter on read group numeric IDs.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
     ///
-    /// \param[in] whitelist    read group IDs to compare on
+    /// \param[in] rgIds    numeric read group IDs
+    /// \param[in] cmp      compare type
     ///
-    PbiReadGroupFilter(std::vector<int32_t> whitelist, const Compare::Type cmp = Compare::EQUAL);
+    PbiReadGroupFilter(const std::vector<int32_t>& rgIds,
+                       const Compare::Type cmp = Compare::CONTAINS);
 
-    /// \brief Creates a 'whitelisted' filter on read group printable IDs.
+    /// \brief Creates a whitelisted or blacklisted filter on read group string IDs.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
     ///
-    /// \param[in] whitelist    read group ID strings to compare on
+    /// \param[in] rgIds    read group ID strings
+    /// \param[in] cmp      compare type
     ///
-    PbiReadGroupFilter(const std::vector<std::string>& whitelist,
+    PbiReadGroupFilter(const std::vector<std::string>& rgIds,
+                       const Compare::Type cmp = Compare::CONTAINS);
+
+    /// \brief Creates a whitelisted or blacklisted filter on read group objects.
+    ///
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
+    ///
+    /// \param[in] readGroups   ReadGroupInfo objects
+    /// \param[in] cmp          compare type
+    ///
+    PbiReadGroupFilter(const std::vector<ReadGroupInfo>& readGroups,
                        const Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' filter using read group objects.
+    /// \brief Performs the actual index lookup.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly.
+    /// Most client code should not need to use this method directly.
     ///
-    /// \param[in] whitelist    read group objects to compare on
-    ///
-    PbiReadGroupFilter(const std::vector<ReadGroupInfo>& whitelist,
-                       const Compare::Type cmp = Compare::EQUAL);
+    bool Accepts(const PbiRawData& idx, const size_t row) const;
+
+private:
+    // RGID number => barcode(s) filter
+    std::unordered_map<int32_t, boost::optional<std::vector<std::pair<int16_t, int16_t>>>> lookup_;
+    Compare::Type cmp_;
 };
 
 /// \brief The PbiReferenceEndFilter class provides a PbiFilter-compatible
@@ -723,7 +746,7 @@ public:
 ///
 /// \sa BamRecord::ReferenceEnd
 ///
-struct PbiReferenceEndFilter
+class PbiReferenceEndFilter
     : public internal::MappedDataFilterBase<uint32_t, PbiFile::MappedField::T_END>
 {
 public:
@@ -742,7 +765,7 @@ public:
 ///
 /// \sa BamRecord::ReferenceId
 ///
-struct PbiReferenceIdFilter
+class PbiReferenceIdFilter
     : public internal::MappedDataFilterBase<int32_t, PbiFile::MappedField::T_ID>
 {
 public:
@@ -753,15 +776,17 @@ public:
     ///
     PbiReferenceIdFilter(const int32_t tId, const Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' reference ID filter.
+    /// \brief Creates a whitelisted or blacklisted reference ID filter.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
     ///
-    /// \param[in] whitelist    reference IDs to compare on
+    /// \param[in] tIds reference IDs
+    /// \param[in] cmp  compare type
     ///
-    PbiReferenceIdFilter(std::vector<int32_t> whitelist, const Compare::Type cmp = Compare::EQUAL);
+    PbiReferenceIdFilter(std::vector<int32_t> tIds, const Compare::Type cmp = Compare::CONTAINS);
 };
 
 /// \brief The PbiReferenceNameFilter class provides a PbiFilter-compatible
@@ -771,7 +796,7 @@ public:
 ///
 /// \sa BamRecord::ReferenceName
 ///
-struct PbiReferenceNameFilter
+class PbiReferenceNameFilter
 {
 public:
     /// \brief Creates a single-value reference name filter.
@@ -781,18 +806,19 @@ public:
     ///
     PbiReferenceNameFilter(std::string rname, Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' reference name filter.
+    /// \brief Creates a whitelisted or blacklisted reference name filter.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
     ///
-    /// \param[in] whitelist    reference names to compare on
+    /// \param[in] rnames   reference names
+    /// \param[in] cmp      compare type
     ///
-    PbiReferenceNameFilter(std::vector<std::string> whitelist,
-                           const Compare::Type cmp = Compare::EQUAL);
+    PbiReferenceNameFilter(std::vector<std::string> rnames,
+                           const Compare::Type cmp = Compare::CONTAINS);
 
-public:
     /// \brief Performs the actual index lookup.
     ///
     /// Most client code should not need to use this method directly.
@@ -803,13 +829,14 @@ private:
     mutable bool initialized_ = false;
     mutable PbiFilter subFilter_;
     std::string rname_;
-    boost::optional<std::vector<std::string> > rnameWhitelist_;
+    boost::optional<std::vector<std::string>> rnameWhitelist_;
     Compare::Type cmp_;
 
-private:
     // marked const so we can delay setup of filter in Accepts(), once we have
     // access to PBI/BAM input. modified values marked mutable accordingly
     void Initialize(const PbiRawData& idx) const;
+
+    void Validate() const;
 };
 
 /// \brief The PbiReferenceStartFilter class provides a PbiFilter-compatible
@@ -819,7 +846,7 @@ private:
 ///
 /// \sa BamRecord::ReferenceStart
 ///
-struct PbiReferenceStartFilter
+class PbiReferenceStartFilter
     : public internal::MappedDataFilterBase<uint32_t, PbiFile::MappedField::T_START>
 {
 public:
@@ -838,7 +865,7 @@ public:
 ///
 /// \sa BamRecord::HoleNumber
 ///
-struct PbiZmwFilter : public internal::BasicDataFilterBase<int32_t, PbiFile::BasicField::ZMW>
+class PbiZmwFilter : public internal::BasicDataFilterBase<int32_t, PbiFile::BasicField::ZMW>
 {
 public:
     /// \brief Creates a single-value ZMW hole number filter.
@@ -848,15 +875,17 @@ public:
     ///
     PbiZmwFilter(const int32_t zmw, const Compare::Type cmp = Compare::EQUAL);
 
-    /// \brief Creates a 'whitelisted' ZMW hole number filter.
+    /// \brief Creates a whitelisted or blacklisted ZMW hole number filter.
     ///
-    /// \note There is no compare type parameter here, it is always
-    ///       Compare::EQUAL. Records will match at least one value from the
-    ///       whitelist, exactly.
+    /// \note If compare type is Compare::CONTAINS, accepted records will match
+    ///       at least one value from whitelist, in bc_forward.
+    ///       If compare type is Compare::NOT_CONTAINS, accepted records will
+    ///       match no values in the blacklist.
     ///
-    /// \param[in] whitelist    ZMW hole numbers to compare on
+    /// \param[in] zmws ZMW hole numbers
+    /// \param[in] cmp  compare type
     ///
-    PbiZmwFilter(std::vector<int32_t> whitelist, const Compare::Type cmp = Compare::EQUAL);
+    PbiZmwFilter(std::vector<int32_t> zmws, const Compare::Type cmp = Compare::CONTAINS);
 };
 
 // ----------------------------------------------
@@ -872,8 +901,9 @@ enum class FilterHash
     BOOST_HASH_COMBINE,
 };
 
-struct PbiZmwModuloFilter
+class PbiZmwModuloFilter
 {
+public:
     PbiZmwModuloFilter(const uint32_t denominator, const uint32_t value,
                        const FilterHash hashtype = FilterHash::UNSIGNED_LONG_CAST,
                        const Compare::Type = Compare::EQUAL);
