@@ -7,6 +7,10 @@
 #ifndef RECORDTYPE_H
 #define RECORDTYPE_H
 
+#include "pbbam/Config.h"
+
+#include <string>
+
 namespace PacBio {
 namespace BAM {
 
@@ -35,10 +39,16 @@ enum class RecordType
 ///
 /// \param[in] type
 ///
-inline bool IsCcsOrTranscript(const RecordType type)
-{
-    return type == RecordType::CCS || type == RecordType::TRANSCRIPT;
-}
+bool IsCcsOrTranscript(const RecordType type);
+
+///
+/// \brief Returns string representation of RecordType
+///
+/// \param type
+/// \return std::string
+/// \throws std::runtime_error if type is unrecognized
+///
+std::string ToString(const RecordType type);
 
 }  // namespace BAM
 }  // namespace PacBio
