@@ -7,8 +7,10 @@
 #ifndef SAMTAGCODEC_H
 #define SAMTAGCODEC_H
 
-#include <string>
 #include "pbbam/Config.h"
+
+#include <string>
+
 #include "pbbam/TagCollection.h"
 
 namespace PacBio {
@@ -33,6 +35,15 @@ public:
     ///
     static TagCollection Decode(const std::string& tagString);
 
+    /// \brief Creates SAM-formatted string from a Tag.
+    ///
+    /// \param[in] name 2-character tag name
+    /// \param[in] tag  Tag instance containing data
+    ///
+    /// \return SAM-formatted string
+    ///
+    static std::string Encode(const std::string& name, const PacBio::BAM::Tag& tag);
+
     /// \brief Creates SAM-formatted string from a TagCollection.
     ///
     /// \param[in] tags     TagCollection containing tag data
@@ -49,10 +60,7 @@ public:
 ///
 /// \return formatted tag string
 ///
-inline std::string MakeSamTag(std::string tag, std::string value)
-{
-    return '\t' + std::move(tag) + ':' + std::move(value);
-}
+std::string MakeSamTag(std::string tag, std::string value);
 
 }  // namespace BAM
 }  // namespace PacBio
