@@ -3,14 +3,24 @@
 #ifndef PBBAM_STRINGUTILITIES_H
 #define PBBAM_STRINGUTILITIES_H
 
-#include <algorithm>
-#include <sstream>
+#include "pbbam/Config.h"
+
 #include <string>
 #include <vector>
 
 namespace PacBio {
 namespace BAM {
 
+///
+/// \brief Joins tokens into a single string
+///
+/// \param tokens   input strings
+/// \param delim    delimiter character
+///
+/// \return joined string
+///
+std::string Join(const std::vector<std::string>& tokens, const char delim);
+
 /// \brief Splits a string into tokens
 ///
 /// \param[in] line     input string
@@ -18,15 +28,7 @@ namespace BAM {
 ///
 /// \returns vector of tokens
 ///
-inline std::vector<std::string> Split(const std::string& line, const char delim = '\t')
-{
-    std::vector<std::string> tokens;
-    std::istringstream lineStream(line);
-    std::string token;
-    while (std::getline(lineStream, token, delim))
-        tokens.push_back(token);
-    return tokens;
-}
+std::vector<std::string> Split(const std::string& line, const char delim = '\t');
 
 /// \brief Remove all whitespace from input string (start, end, & internal)
 ///
@@ -34,13 +36,7 @@ inline std::vector<std::string> Split(const std::string& line, const char delim
 ///
 /// \returns new string with no whitespace
 ///
-inline std::string RemoveAllWhitespace(std::string input)
-{
-    input.erase(
-        std::remove_if(input.begin(), input.end(), [](const char c) { return std::isspace(c); }),
-        input.end());
-    return input;
-}
+std::string RemoveAllWhitespace(std::string input);
 
 }  // namespace BAM
 }  // namespace PacBio
