@@ -7,11 +7,13 @@
 #ifndef TAG_H
 #define TAG_H
 
-#include <boost/variant.hpp>
+#include "pbbam/Config.h"
+
 #include <cstdint>
 #include <string>
 #include <vector>
-#include "pbbam/Config.h"
+
+#include <boost/variant.hpp>
 
 namespace PacBio {
 namespace BAM {
@@ -166,11 +168,6 @@ public:
     Tag(std::vector<float> value);
 
     Tag() = default;
-    Tag(const Tag&) = default;
-    Tag(Tag&&) = default;
-    Tag& operator=(const Tag&) = default;
-    Tag& operator=(Tag&&) = default;
-    ~Tag() = default;
 
     Tag& operator=(boost::blank value);
     Tag& operator=(int8_t value);
@@ -413,6 +410,4 @@ private:
 }  // namespace BAM
 }  // namespace PacBio
 
-#include "pbbam/internal/Tag.inl"
-
 #endif  // TAG_H
