@@ -7,9 +7,11 @@
 #ifndef TAGCOLLECTION_H
 #define TAGCOLLECTION_H
 
+#include "pbbam/Config.h"
+
 #include <map>
 #include <string>
-#include "pbbam/Config.h"
+
 #include "pbbam/Tag.h"
 
 namespace PacBio {
@@ -24,7 +26,7 @@ class PBBAM_EXPORT TagCollection : public std::map<std::string, Tag>
 {
 public:
     /// \returns true if the collection contains a tag with \p name
-    inline bool Contains(const std::string& name) const { return count(name) != 0; }
+    bool Contains(const std::string& name) const;
 };
 
 }  // namespace BAM
