@@ -7,8 +7,11 @@
 #ifndef ZMWGROUPQUERY_H
 #define ZMWGROUPQUERY_H
 
+#include "pbbam/Config.h"
+
 #include <cstdint>
 #include <vector>
+
 #include "pbbam/internal/QueryBase.h"
 
 namespace PacBio {
@@ -49,7 +52,7 @@ public:
     bool GetNext(std::vector<BamRecord>& records) override;
 
 private:
-    struct ZmwGroupQueryPrivate;
+    class ZmwGroupQueryPrivate;
     std::unique_ptr<ZmwGroupQueryPrivate> d_;
 };
 
