@@ -3,6 +3,8 @@
 #ifndef PBBAM_VCF_VCFFILE_H
 #define PBBAM_VCF_VCFFILE_H
 
+#include "pbbam/Config.h"
+
 #include <string>
 
 #include <pbbam/vcf/VcfHeader.h>
@@ -15,13 +17,6 @@ class VcfFile
 public:
     explicit VcfFile(std::string fn);
 
-    VcfFile() = delete;
-    VcfFile(const VcfFile&) = default;
-    VcfFile(VcfFile&&) = default;
-    VcfFile& operator=(const VcfFile&) = default;
-    VcfFile& operator=(VcfFile&&) = default;
-    ~VcfFile() = default;
-
 public:
     const std::string& Filename() const;
     const VcfHeader& Header() const;
@@ -34,6 +29,4 @@ private:
 }  // namespace VCF
 }  // namespace PacBio
 
-#include "pbbam/vcf/internal/VcfFile.inl"
-
 #endif  // PBBAM_VCF_VCFFILE_H
