@@ -3,9 +3,10 @@
 #ifndef PBBAM_VCF_VCFSORT_H
 #define PBBAM_VCF_VCFSORT_H
 
+#include "pbbam/Config.h"
+
 #include <string>
 
-#include "pbbam/Config.h"
 #include "pbbam/vcf/VcfFile.h"
 
 namespace PacBio {
@@ -23,10 +24,7 @@ void SortFile(const VcfFile& file, const std::string& outputFilename);
 /// \param inputFilename
 /// \param outputFilename
 ///
-inline void SortFile(const std::string& inputFilename, const std::string& outputFilename)
-{
-    SortFile(VcfFile{inputFilename}, outputFilename);
-}
+void SortFile(const std::string& inputFilename, const std::string& outputFilename);
 
 }  // namespace VCF
 }  // namespace PacBio
