@@ -7,13 +7,15 @@
 #ifndef VIRTUALPOLYMERASEBAMRECORD_H
 #define VIRTUALPOLYMERASEBAMRECORD_H
 
+#include "pbbam/Config.h"
+
 #include "pbbam/virtual/VirtualZmwBamRecord.h"
 
 namespace PacBio {
 namespace BAM {
 
 /// \deprecated Use VirtualZmwBamRecord instead.
-typedef VirtualZmwBamRecord VirtualPolymeraseBamRecord;
+using VirtualPolymeraseBamRecord = VirtualZmwBamRecord;
 
 }  // namespace BAM
 }  // namespace PacBio
