@@ -7,14 +7,15 @@
 #ifndef VIRTUALPOLYMERASEREADER_H
 #define VIRTUALPOLYMERASEREADER_H
 
-#include "pbbam/virtual/VirtualPolymeraseBamRecord.h"
+#include "pbbam/Config.h"
+
 #include "pbbam/virtual/ZmwReadStitcher.h"
 
 namespace PacBio {
 namespace BAM {
 
 /// \deprecated Use ZmwReadStitcher instead.
-typedef ZmwReadStitcher VirtualPolymeraseReader;
+using VirtualPolymeraseReader = ZmwReadStitcher;
 
 }  // namespace BAM
 }  // namespace PacBio
