@@ -7,13 +7,15 @@
 #ifndef ZMWWHITELISTVIRTUALREADER_H
 #define ZMWWHITELISTVIRTUALREADER_H
 
+#include "pbbam/Config.h"
+
 #include "pbbam/virtual/WhitelistedZmwReadStitcher.h"
 
 namespace PacBio {
 namespace BAM {
 
 /// \deprecated Use WhitelistedZmwReadStitcher instead.
-typedef WhitelistedZmwReadStitcher ZmwWhitelistVirtualReader;
+using ZmwWhitelistVirtualReader = WhitelistedZmwReadStitcher;
 
 }  // namespace BAM
 }  // namespace PacBio
