@@ -7,11 +7,13 @@
 #ifndef AUTOVALIDATE_H
 #define AUTOVALIDATE_H
 
+#include "pbbam/Config.h"
+
 // \brief Auto-validation
 //
 // To validate BAM components (header, records, etc.) you can either use the
 // Validator API provided, or enable auto-validation. To compile pbbam for
-// auto-validation, add the -DPacBioBAM_auto_validate=ON option to your cmake
+// auto-validation, add the -Dauto-validate=true option to your Meson
 // invocation.
 //
 //
