@@ -13,23 +13,22 @@
 #include <cstdint>
 #include <memory>
 #include <sstream>
+#include <string>
 
 #include <htslib/sam.h>
 
 #include "Autovalidate.h"
 #include "FileUtils.h"
 #include "MemoryUtils.h"
-#include "pbbam/MakeUnique.h"
 #include "pbbam/PbiFile.h"
 
 namespace PacBio {
 namespace BAM {
-namespace internal {
 
-class BamFilePrivate
+class BamFile::BamFilePrivate
 {
 public:
-    BamFilePrivate(std::string fn) : filename_{std::move(fn)}, firstAlignmentOffset_{-1}
+    explicit BamFilePrivate(std::string fn) : filename_{std::move(fn)}, firstAlignmentOffset_{-1}
     {
         // ensure we've updated htslib verbosity with requested verbosity here
         hts_verbose = (PacBio::BAM::HtslibVerbosity == -1 ? 0 : PacBio::BAM::HtslibVerbosity);
@@ -47,16 +46,17 @@ public:
             // -1: some other error
             std::ostringstream e;
             if (eofCheck == 0)
-                e << fn << " : is missing EOF block\n";
+                e << "BamFile: missing EOF block in " << fn;
             else
-                e << fn << " : unknown error while checking EOF block\n";
+                e << "BamFile: unknown error encountered while checking EOF in " << fn
+                  << " (status code = " << eofCheck << ')';
             throw std::runtime_error{e.str()};
         }
 #endif
 
         // attempt fetch header
-        std::unique_ptr<bam_hdr_t, internal::HtslibHeaderDeleter> hdr(sam_hdr_read(f.get()));
-        header_ = internal::BamHeaderMemory::FromRawData(hdr.get());
+        std::unique_ptr<bam_hdr_t, HtslibHeaderDeleter> hdr(sam_hdr_read(f.get()));
+        header_ = BamHeaderMemory::FromRawData(hdr.get());
 
         // cache first alignment offset
         firstAlignmentOffset_ = bgzf_tell(f->fp.bgzf);
@@ -77,41 +77,35 @@ public:
         return RawEOFCheck(f) == 1;
     }
 
-    int RawEOFCheck(const std::unique_ptr<samFile, internal::HtslibFileDeleter>& f) const
+    int RawEOFCheck(const std::unique_ptr<samFile, HtslibFileDeleter>& f) const
     {
         assert(f);
         assert(f->fp.bgzf);
         return bgzf_check_EOF(f->fp.bgzf);
     }
 
-    std::unique_ptr<samFile, internal::HtslibFileDeleter> RawOpen() const
+    std::unique_ptr<samFile, HtslibFileDeleter> RawOpen() const
     {
-        std::unique_ptr<samFile, internal::HtslibFileDeleter> f(sam_open(filename_.c_str(), "rb"));
-        if (!f || !f->fp.bgzf) throw std::runtime_error{"could not open BAM file: " + filename_};
-        if (f->format.format != bam) throw std::runtime_error{"expected BAM, unknown format"};
+        std::unique_ptr<samFile, HtslibFileDeleter> f(sam_open(filename_.c_str(), "rb"));
+        if (!f || !f->fp.bgzf) throw std::runtime_error{"BamFile: could not open: " + filename_};
+        if (f->format.format != bam)
+            throw std::runtime_error{"BamFile: expected BAM, encountered un supported format: " +
+                                     filename_};
         return f;
     }
 
-public:
     std::string filename_;
     BamHeader header_;
     int64_t firstAlignmentOffset_;
 };
 
-}  // namespace internal
-
-// ------------------------
-// BamFile implementation
-// ------------------------
-
-BamFile::BamFile(std::string filename)
-    : d_{std::make_unique<internal::BamFilePrivate>(std::move(filename))}
+BamFile::BamFile(std::string filename) : d_{std::make_unique<BamFilePrivate>(std::move(filename))}
 {
 }
 
 BamFile::BamFile(const BamFile& other) : d_{other.d_->DeepCopy()} {}
 
-BamFile::BamFile(BamFile&& other) : d_{std::move(other.d_)} {}
+BamFile::BamFile(BamFile&&) noexcept = default;
 
 BamFile& BamFile::operator=(const BamFile& other)
 {
@@ -121,22 +115,21 @@ BamFile& BamFile::operator=(const BamFile& other)
     return *this;
 }
 
-BamFile& BamFile::operator=(BamFile&& other)
-{
-    if (this != &other) {
-        d_ = std::move(other.d_);
-    }
-    return *this;
-}
+BamFile& BamFile::operator=(BamFile&&) noexcept = default;
 
-BamFile::~BamFile() {}
+BamFile::~BamFile() = default;
 
 void BamFile::CreatePacBioIndex() const { PbiFile::CreateFrom(*this); }
 
 void BamFile::CreateStandardIndex() const
 {
-    if (bam_index_build(d_->filename_.c_str(), 0) != 0)
-        throw std::runtime_error{"could not build BAI index"};
+    const auto ret = bam_index_build(d_->filename_.c_str(), 0);
+    if (ret != 0) {
+        std::ostringstream s;
+        s << "BamFile: could not create *.bai index for file: " << d_->filename_
+          << " (status code = " << ret << ')';
+        throw std::runtime_error{s.str()};
+    }
 }
 
 void BamFile::EnsurePacBioIndexExists() const
@@ -161,17 +154,14 @@ const BamHeader& BamFile::Header() const { return d_->header_; }
 
 bool BamFile::IsPacBioBAM() const { return !d_->header_.PacBioBamVersion().empty(); }
 
-bool BamFile::PacBioIndexExists() const
-{
-    return internal::FileUtils::Exists(PacBioIndexFilename());
-}
+bool BamFile::PacBioIndexExists() const { return FileUtils::Exists(PacBioIndexFilename()); }
 
 std::string BamFile::PacBioIndexFilename() const { return d_->filename_ + ".pbi"; }
 
 bool BamFile::PacBioIndexIsNewer() const
 {
-    const auto bamTimestamp = internal::FileUtils::LastModified(Filename());
-    const auto pbiTimestamp = internal::FileUtils::LastModified(PacBioIndexFilename());
+    const auto bamTimestamp = FileUtils::LastModified(Filename());
+    const auto pbiTimestamp = FileUtils::LastModified(PacBioIndexFilename());
     return bamTimestamp <= pbiTimestamp;
 }
 
@@ -189,17 +179,14 @@ uint32_t BamFile::ReferenceLength(const int id) const
 
 std::string BamFile::ReferenceName(const int id) const { return d_->header_.SequenceName(id); }
 
-bool BamFile::StandardIndexExists() const
-{
-    return internal::FileUtils::Exists(StandardIndexFilename());
-}
+bool BamFile::StandardIndexExists() const { return FileUtils::Exists(StandardIndexFilename()); }
 
 std::string BamFile::StandardIndexFilename() const { return d_->filename_ + ".bai"; }
 
 bool BamFile::StandardIndexIsNewer() const
 {
-    const auto bamTimestamp = internal::FileUtils::LastModified(Filename());
-    const auto baiTimestamp = internal::FileUtils::LastModified(StandardIndexFilename());
+    const auto bamTimestamp = FileUtils::LastModified(Filename());
+    const auto baiTimestamp = FileUtils::LastModified(StandardIndexFilename());
     return bamTimestamp <= baiTimestamp;
 }
 
