@@ -16,7 +16,7 @@
 
 namespace PacBio {
 namespace BAM {
-namespace internal {
+namespace {
 
 template <typename T>
 inline void appendBamValue(const T& value, kstring_t* str)
@@ -57,7 +57,7 @@ std::vector<T> readBamMultiValue(const uint8_t* src, size_t& offset)
     return result;
 }
 
-}  // namespace internal
+}  // anonymous
 
 TagCollection BamTagCodec::Decode(const std::vector<uint8_t>& data)
 {
@@ -75,9 +75,6 @@ TagCollection BamTagCodec::Decode(const std::vector<uint8_t>& data)
         tagName.append(1, pData[i++]);
         tagName.append(1, pData[i++]);
 
-        using internal::readBamMultiValue;
-        using internal::readBamValue;
-
         const auto tagType = static_cast<char>(pData[i++]);
         switch (tagType) {
             case 'A':
@@ -146,15 +143,16 @@ TagCollection BamTagCodec::Decode(const std::vector<uint8_t>& data)
 
                     // unknown subTagType
                     default:
-                        throw std::runtime_error{"unsupported array-tag-type encountered: " +
-                                                 std::string{1, subTagType}};
+                        throw std::runtime_error{
+                            "BamTagCodec: unsupported array-tag-type encountered: " +
+                            std::string{1, subTagType}};
                 }
                 break;
             }
 
             // unknown tagType
             default:
-                throw std::runtime_error{"unsupported tag-type encountered: " +
+                throw std::runtime_error{"BamTagCodec: unsupported tag-type encountered: " +
                                          std::string{1, tagType}};
         }
     }
@@ -169,7 +167,9 @@ std::vector<uint8_t> BamTagCodec::Encode(const TagCollection& tags)
     for (const auto& tagIter : tags) {
 
         const auto& name = tagIter.first;
-        if (name.size() != 2) throw std::runtime_error{"malformatted tag name: " + name};
+        if (name.size() != 2)
+            throw std::runtime_error{"BamTagCodec: tag name (" + name +
+                                     ") must have 2 characters only"};
 
         const auto& tag = tagIter.second;
         if (tag.IsNull()) continue;
@@ -187,9 +187,6 @@ std::vector<uint8_t> BamTagCodec::Encode(const TagCollection& tags)
             }
         }
 
-        using internal::appendBamMultiValue;
-        using internal::appendBamValue;
-
         // "<TYPE>:<DATA>" for all other data
         switch (tag.Type()) {
             case TagDataType::INT8: {
@@ -284,7 +281,7 @@ std::vector<uint8_t> BamTagCodec::Encode(const TagCollection& tags)
             // unsupported tag type
             default: {
                 free(str.s);
-                throw std::runtime_error{"unsupported tag-type encountered: " +
+                throw std::runtime_error{"BamTagCodec: unsupported tag-type encountered: " +
                                          std::to_string(static_cast<uint16_t>(tag.Type()))};
             }
         }
@@ -299,9 +296,6 @@ std::vector<uint8_t> BamTagCodec::Encode(const TagCollection& tags)
 
 Tag BamTagCodec::FromRawData(uint8_t* rawData)
 {
-    using internal::readBamMultiValue;
-    using internal::readBamValue;
-
     size_t offset = 0;
     const auto tagType = static_cast<char>(*rawData++);
     switch (tagType) {
@@ -357,15 +351,16 @@ Tag BamTagCodec::FromRawData(uint8_t* rawData)
 
                 // unknown subTagType
                 default:
-                    throw std::runtime_error{"unsupported array-tag-type encountered: " +
-                                             std::string{1, subTagType}};
+                    throw std::runtime_error{
+                        "BamTagCodec: unsupported array-tag-type encountered: " +
+                        std::string{1, subTagType}};
             }
             break;
         }
 
         // unknown tagType
         default:
-            throw std::runtime_error{"unsupported tag-type encountered: " +
+            throw std::runtime_error{"BamTagCodec: unsupported tag-type encountered: " +
                                      std::string{1, tagType}};
     }
     return Tag();  // to avoid compiler warning
@@ -384,10 +379,6 @@ std::vector<uint8_t> BamTagCodec::ToRawData(const Tag& tag, const TagModifier& a
 
     // for all others
     else {
-
-        using internal::appendBamMultiValue;
-        using internal::appendBamValue;
-
         switch (tag.Type()) {
 
             // single, numeric values
@@ -460,7 +451,7 @@ std::vector<uint8_t> BamTagCodec::ToRawData(const Tag& tag, const TagModifier& a
             // unsupported tag type
             default: {
                 free(str.s);
-                throw std::runtime_error{"unsupported tag-type encountered: " +
+                throw std::runtime_error{"BamTagCodec: unsupported tag-type encountered: " +
                                          std::to_string(static_cast<uint16_t>(tag.Type()))};
             }
         }
@@ -499,13 +490,14 @@ uint8_t BamTagCodec::TagTypeCode(const Tag& tag, const TagModifier& additionalMo
                 break;
             default:
                 // non integers not allowed
-                throw std::runtime_error{"tag-type not convertible to ASCII, tag-type: " +
-                                         std::to_string(static_cast<uint16_t>(tag.Type()))};
+                throw std::runtime_error{
+                    "BamTagCodec: tag-type not convertible to ASCII, tag-type: " +
+                    std::to_string(static_cast<uint16_t>(tag.Type()))};
         }
 
         // ensure value is in valid ASCII char range
         if (value < 33 || value > 126)
-            throw std::runtime_error{"invalid integer value for ASCII char, value: " +
+            throw std::runtime_error{"BamTagCodec: invalid integer value for ASCII char, value: " +
                                      std::to_string(value)};
 
         return static_cast<uint8_t>('A');
@@ -544,7 +536,7 @@ uint8_t BamTagCodec::TagTypeCode(const Tag& tag, const TagModifier& additionalMo
             return static_cast<uint8_t>('B');
 
         default:
-            throw std::runtime_error{"unsupported tag-type encountered: " +
+            throw std::runtime_error{"BamTagCodec: unsupported tag-type encountered: " +
                                      std::to_string(static_cast<uint16_t>(tag.Type()))};
     }
     return 0;  // to avoid compiler warning
