@@ -11,14 +11,14 @@
 #include <cstdint>
 
 #include "pbbam/CompositeBamReader.h"
-#include "pbbam/MakeUnique.h"
 #include "pbbam/PbiFilterTypes.h"
 
 namespace PacBio {
 namespace BAM {
 
-struct BarcodeQuery::BarcodeQueryPrivate
+class BarcodeQuery::BarcodeQueryPrivate
 {
+public:
     BarcodeQueryPrivate(const int16_t barcode, const DataSet& dataset)
         : reader_{PbiBarcodeFilter{barcode}, dataset}
     {
@@ -32,7 +32,7 @@ BarcodeQuery::BarcodeQuery(const int16_t barcode, const DataSet& dataset)
 {
 }
 
-BarcodeQuery::~BarcodeQuery() {}
+BarcodeQuery::~BarcodeQuery() = default;
 
 bool BarcodeQuery::GetNext(BamRecord& r) { return d_->reader_.GetNext(r); }
 
