@@ -3,21 +3,21 @@
 #ifndef CHEMISTRYTABLE_H
 #define CHEMISTRYTABLE_H
 
+#include "pbbam/Config.h"
+
 #include <array>
 #include <string>
 #include <vector>
 
 namespace PacBio {
 namespace BAM {
-namespace internal {
 
-typedef std::vector<std::array<std::string, 4>> ChemistryTable;
+using ChemistryTable = std::vector<std::array<std::string, 5>>;
 
-extern const ChemistryTable BuiltInChemistryTable;
+const ChemistryTable& BuiltInChemistryTable();
 
 const ChemistryTable& GetChemistryTableFromEnv();
 
-}  // namespace internal
 }  // namespace BAM
 }  // namespace PacBio
 
