@@ -13,8 +13,9 @@
 namespace PacBio {
 namespace BAM {
 
-struct EntireFileQuery::EntireFileQueryPrivate
+class EntireFileQuery::EntireFileQueryPrivate
 {
+public:
     EntireFileQueryPrivate(const DataSet &dataset) : reader_(dataset) {}
 
     SequentialCompositeBamReader reader_;
@@ -25,7 +26,7 @@ EntireFileQuery::EntireFileQuery(const DataSet &dataset)
 {
 }
 
-EntireFileQuery::~EntireFileQuery() {}
+EntireFileQuery::~EntireFileQuery() = default;
 
 bool EntireFileQuery::GetNext(BamRecord &r) { return d_->reader_.GetNext(r); }
 
