@@ -9,13 +9,13 @@
 #include "pbbam/FastaSequenceQuery.h"
 
 #include "pbbam/CompositeFastaReader.h"
-#include "pbbam/MakeUnique.h"
 
 namespace PacBio {
 namespace BAM {
 
-struct FastaSequenceQuery::FastaSequenceQueryPrivate
+class FastaSequenceQuery::FastaSequenceQueryPrivate
 {
+public:
     FastaSequenceQueryPrivate(const DataSet& dataset) : reader_{dataset} {}
 
     CompositeFastaReader reader_;
@@ -26,7 +26,7 @@ FastaSequenceQuery::FastaSequenceQuery(const DataSet& dataset)
 {
 }
 
-FastaSequenceQuery::~FastaSequenceQuery() {}
+FastaSequenceQuery::~FastaSequenceQuery() = default;
 
 bool FastaSequenceQuery::GetNext(FastaSequence& seq) { return d_->reader_.GetNext(seq); }
 
