@@ -5,20 +5,24 @@
 #include "FileProducer.h"
 
 #include <cstdio>
-#include <exception>
+
+#include <stdexcept>
 
 namespace PacBio {
 namespace BAM {
-namespace internal {
 
 FileProducer::FileProducer(std::string targetFilename)
-    : FileProducer(std::move(targetFilename), targetFilename + ".tmp")
+    : FileProducer(targetFilename, targetFilename + ".tmp")
 {
 }
 
 FileProducer::FileProducer(std::string targetFilename, std::string tempFilename)
     : targetFilename_{std::move(targetFilename)}, tempFilename_{std::move(tempFilename)}
 {
+    if (targetFilename_.empty()) {
+        throw std::runtime_error{"FileProducer error: cannot write to file with empty name"};
+    }
+
     // override renaming if writing to stdout
     //
     // setting temp filename to '-' keeps consistent interfaces
@@ -35,6 +39,5 @@ FileProducer::~FileProducer()
     }
 }
 
-}  // namespace internal
 }  // namespace BAM
 }  // namespace PacBio
