@@ -3,14 +3,16 @@
 #ifndef FOFNREADER_H
 #define FOFNREADER_H
 
+#include "pbbam/Config.h"
+
 #include <iosfwd>
 #include <string>
 #include <vector>
-#include "pbbam/DataSet.h"
+
+#include <pbbam/DataSet.h>
 
 namespace PacBio {
 namespace BAM {
-namespace internal {
 
 class FofnReader
 {
@@ -18,7 +20,6 @@ public:
     static std::vector<std::string> Files(std::istream& in);
 };
 
-}  // namespace internal
 }  // namespace BAM
 }  // namespace PacBio
 
