@@ -22,8 +22,8 @@
 #include <boost/numeric/conversion/cast.hpp>
 
 #include "FileUtils.h"
-#include "StringUtils.h"
 #include "pbbam/PbiFilterTypes.h"
+#include "pbbam/StringUtilities.h"
 
 namespace PacBio {
 namespace BAM {
@@ -48,6 +48,7 @@ enum class BuiltIn
   , NumInsertedBasesFilter
   , NumMatchesFilter
   , NumMismatchesFilter
+  , QIdFilter
   , QueryEndFilter
   , QueryLengthFilter
   , QueryNameFilter
@@ -81,6 +82,7 @@ static const std::unordered_map<std::string, BuiltIn> builtInLookup =
     { "identity",      BuiltIn::IdentityFilter },
     { "cx",            BuiltIn::LocalContextFilter },
     { "movie",         BuiltIn::MovieNameFilter },
+    { "qid",           BuiltIn::QIdFilter },
     { "qe",            BuiltIn::QueryEndFilter },
     { "qend",          BuiltIn::QueryEndFilter },
     { "length",        BuiltIn::QueryLengthFilter },
@@ -102,13 +104,15 @@ static const std::unordered_map<std::string, BuiltIn> builtInLookup =
 
 static const std::unordered_map<std::string, LocalContextFlags> contextFlagNames =
 {
-    { "NO_LOCAL_CONTEXT", LocalContextFlags::NO_LOCAL_CONTEXT },
-    { "ADAPTER_BEFORE",   LocalContextFlags::ADAPTER_BEFORE },
-    { "ADAPTER_AFTER",    LocalContextFlags::ADAPTER_AFTER },
-    { "BARCODE_BEFORE",   LocalContextFlags::BARCODE_BEFORE },
-    { "BARCODE_AFTER",    LocalContextFlags::BARCODE_AFTER },
-    { "FORWARD_PASS",     LocalContextFlags::FORWARD_PASS },
-    { "REVERSE_PASS",     LocalContextFlags::REVERSE_PASS }
+    { "NO_LOCAL_CONTEXT",   LocalContextFlags::NO_LOCAL_CONTEXT },
+    { "ADAPTER_BEFORE",     LocalContextFlags::ADAPTER_BEFORE },
+    { "ADAPTER_AFTER",      LocalContextFlags::ADAPTER_AFTER },
+    { "BARCODE_BEFORE",     LocalContextFlags::BARCODE_BEFORE },
+    { "BARCODE_AFTER",      LocalContextFlags::BARCODE_AFTER },
+    { "FORWARD_PASS",       LocalContextFlags::FORWARD_PASS },
+    { "REVERSE_PASS",       LocalContextFlags::REVERSE_PASS },
+    { "ADAPTER_BEFORE_BAD", LocalContextFlags::ADAPTER_BEFORE_BAD},
+    { "ADAPTER_AFTER_BAD",  LocalContextFlags::ADAPTER_AFTER_BAD}
 };
 // clang-format off
 
@@ -125,7 +129,7 @@ static inline bool isList(const std::string& value) { return value.find(',') !=
 
 static PbiFilter CreateBarcodeFilter(std::string value, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for barcode filter property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for barcode filter property"};
 
     if (isBracketed(value)) {
         value.erase(0, 1);
@@ -133,8 +137,8 @@ static PbiFilter CreateBarcodeFilter(std::string value, const Compare::Type comp
     }
 
     if (isList(value)) {
-        std::vector<std::string> barcodes = internal::Split(value, ',');
-        if (barcodes.size() != 2) throw std::runtime_error{"only 2 barcode values expected"};
+        std::vector<std::string> barcodes = Split(value, ',');
+        if (barcodes.size() != 2) throw std::runtime_error{"PbiFilter: only 2 barcode values expected"};
         return PbiBarcodesFilter{boost::numeric_cast<int16_t>(std::stoi(barcodes.at(0))),
                                  boost::numeric_cast<int16_t>(std::stoi(barcodes.at(1))),
                                  compareType};
@@ -144,7 +148,7 @@ static PbiFilter CreateBarcodeFilter(std::string value, const Compare::Type comp
 
 static PbiFilter CreateBarcodeForwardFilter(std::string value, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for barcode_forward filter property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for barcode_forward filter property"};
 
     if (isBracketed(value)) {
         value.erase(0, 1);
@@ -152,7 +156,7 @@ static PbiFilter CreateBarcodeForwardFilter(std::string value, const Compare::Ty
     }
 
     if (isList(value)) {
-        std::vector<std::string> tokens = internal::Split(value, ',');
+        std::vector<std::string> tokens = Split(value, ',');
         std::vector<int16_t> barcodes;
         barcodes.reserve(tokens.size());
         for (const auto& t : tokens)
@@ -164,7 +168,7 @@ static PbiFilter CreateBarcodeForwardFilter(std::string value, const Compare::Ty
 
 static PbiFilter CreateBarcodeReverseFilter(std::string value, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for barcode_reverse filter property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for barcode_reverse filter property"};
 
     if (isBracketed(value)) {
         value.erase(0, 1);
@@ -172,7 +176,7 @@ static PbiFilter CreateBarcodeReverseFilter(std::string value, const Compare::Ty
     }
 
     if (isList(value)) {
-        std::vector<std::string> tokens = internal::Split(value, ',');
+        std::vector<std::string> tokens = Split(value, ',');
         std::vector<int16_t> barcodes;
         barcodes.reserve(tokens.size());
         for (const auto& t : tokens)
@@ -184,7 +188,7 @@ static PbiFilter CreateBarcodeReverseFilter(std::string value, const Compare::Ty
 
 static PbiFilter CreateLocalContextFilter(const std::string& value, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for local context filter property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for local context filter property"};
 
     LocalContextFlags filterValue = LocalContextFlags::NO_LOCAL_CONTEXT;
 
@@ -193,7 +197,7 @@ static PbiFilter CreateLocalContextFilter(const std::string& value, const Compar
 
     // else interpret as flag names
     else {
-        std::vector<std::string> tokens = internal::Split(value, '|');
+        std::vector<std::string> tokens = Split(value, '|');
         for (std::string& token : tokens) {
             boost::algorithm::trim(token);  // trim whitespace
             filterValue = (filterValue | contextFlagNames.at(token));
@@ -205,7 +209,7 @@ static PbiFilter CreateLocalContextFilter(const std::string& value, const Compar
 
 static PbiFilter CreateMovieNameFilter(std::string value, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for movie property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for movie property"};
 
     if (isBracketed(value)) {
         value.erase(0, 1);
@@ -215,18 +219,43 @@ static PbiFilter CreateMovieNameFilter(std::string value, const Compare::Type co
     if (isList(value)) {
 
         if (compareType != Compare::EQUAL && compareType != Compare::NOT_EQUAL)
-            throw std::runtime_error{"unsupported compare type on movie property"};
+            throw std::runtime_error{"PbiFilter: unsupported compare type on movie property"};
 
-        std::vector<std::string> tokens = internal::Split(value, ',');
+        std::vector<std::string> tokens = Split(value, ',');
         return PbiMovieNameFilter{std::move(tokens), compareType};
     } else
         return PbiMovieNameFilter{value, compareType};
 }
 
+static PbiFilter CreateQIdFilter(std::string value, const Compare::Type compareType)
+{
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for qid property"};
+
+    if (isBracketed(value)) {
+        value.erase(0, 1);
+        value.pop_back();
+    }
+
+    if (isList(value)) {
+
+        if (compareType != Compare::EQUAL && compareType != Compare::NOT_EQUAL)
+            throw std::runtime_error{"PbiFilter: unsupported compare type on qid property"};
+
+        std::vector<int32_t> rgIds;
+        for (const auto& t : Split(value, ','))
+            rgIds.push_back(static_cast<int32_t>(std::stoul(t)));
+        return PbiReadGroupFilter{rgIds, compareType};
+    } else {
+        const auto n = static_cast<int32_t>(std::stoul(value));
+        return PbiReadGroupFilter{n, compareType};
+    }
+
+}
+
 static PbiFilter CreateQueryNamesFilterFromFile(const std::string& value, const DataSet& dataset, const Compare::Type compareType)
 {
     if (compareType != Compare::EQUAL && compareType != Compare::NOT_EQUAL)
-        throw std::runtime_error{"unsupported compare type on query name property"};
+        throw std::runtime_error{"PbiFilter: unsupported compare type on query name property"};
 
     // resolve file from dataset, value
     const std::string resolvedFilename = dataset.ResolvePath(value);
@@ -240,11 +269,11 @@ static PbiFilter CreateQueryNamesFilterFromFile(const std::string& value, const
 
 static PbiFilter CreateQueryNameFilter(std::string value, const DataSet& dataset, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for query name property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for query name property"};
 
     // try possible filename first
     const std::string resolvedFilename = dataset.ResolvePath(value);
-    if (internal::FileUtils::Exists(value))
+    if (FileUtils::Exists(value))
         return CreateQueryNamesFilterFromFile(value, dataset, compareType);
 
     // otherwise "normal" qname (single, or list)
@@ -257,9 +286,9 @@ static PbiFilter CreateQueryNameFilter(std::string value, const DataSet& dataset
     if (isList(value)) {
 
         if (compareType != Compare::EQUAL && compareType != Compare::NOT_EQUAL)
-            throw std::runtime_error{"unsupported compare type on query name property"};
+            throw std::runtime_error{"PbiFilter: unsupported compare type on query name property"};
 
-        std::vector<std::string> tokens = internal::Split(value, ',');
+        std::vector<std::string> tokens = Split(value, ',');
         return PbiQueryNameFilter{std::move(tokens), compareType};
     } else
         return PbiQueryNameFilter{value, compareType};
@@ -267,7 +296,7 @@ static PbiFilter CreateQueryNameFilter(std::string value, const DataSet& dataset
 
 static PbiFilter CreateReadGroupFilter(std::string value, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for read group property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for read group property"};
 
     if (isBracketed(value)) {
         value.erase(0, 1);
@@ -277,9 +306,9 @@ static PbiFilter CreateReadGroupFilter(std::string value, const Compare::Type co
     if (isList(value)) {
 
         if (compareType != Compare::EQUAL && compareType != Compare::NOT_EQUAL)
-            throw std::runtime_error{"unsupported compare type on read group property"};
+            throw std::runtime_error{"PbiFilter: unsupported compare type on read group property"};
 
-        std::vector<std::string> tokens = internal::Split(value, ',');
+        std::vector<std::string> tokens = Split(value, ',');
         return PbiReadGroupFilter{std::move(tokens), compareType};
     } else
         return PbiReadGroupFilter{value, compareType};
@@ -287,7 +316,7 @@ static PbiFilter CreateReadGroupFilter(std::string value, const Compare::Type co
 
 static PbiFilter CreateReferenceIdFilter(std::string value, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for reference ID property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for reference ID property"};
 
     if (isBracketed(value)) {
         value.erase(0, 1);
@@ -297,9 +326,9 @@ static PbiFilter CreateReferenceIdFilter(std::string value, const Compare::Type
     if (isList(value)) {
 
         if (compareType != Compare::EQUAL && compareType != Compare::NOT_EQUAL)
-            throw std::runtime_error{"unsupported compare type on reference name ID property"};
+            throw std::runtime_error{"PbiFilter: unsupported compare type on reference name ID property"};
 
-        std::vector<std::string> tokens = internal::Split(value, ',');
+        std::vector<std::string> tokens = Split(value, ',');
         std::vector<int32_t> ids;
         ids.reserve(tokens.size());
         for (const auto& t : tokens)
@@ -311,7 +340,7 @@ static PbiFilter CreateReferenceIdFilter(std::string value, const Compare::Type
 
 static PbiFilter CreateReferenceNameFilter(std::string value, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for reference name property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for reference name property"};
 
     if (isBracketed(value)) {
         value.erase(0, 1);
@@ -321,9 +350,9 @@ static PbiFilter CreateReferenceNameFilter(std::string value, const Compare::Typ
     if (isList(value)) {
 
         if (compareType != Compare::EQUAL && compareType != Compare::NOT_EQUAL)
-            throw std::runtime_error{"unsupported compare type on reference name property"};
+            throw std::runtime_error{"PbiFilter: unsupported compare type on reference name property"};
 
-        std::vector<std::string> tokens = internal::Split(value, ',');
+        std::vector<std::string> tokens = Split(value, ',');
         return PbiReferenceNameFilter{std::move(tokens), compareType};
     } else
         return PbiReferenceNameFilter{value, compareType};
@@ -331,7 +360,7 @@ static PbiFilter CreateReferenceNameFilter(std::string value, const Compare::Typ
 
 static PbiFilter CreateZmwFilter(std::string value, const Compare::Type compareType)
 {
-    if (value.empty()) throw std::runtime_error{"empty value for ZMW filter property"};
+    if (value.empty()) throw std::runtime_error{"PbiFilter: empty value for ZMW filter property"};
 
     if (isBracketed(value)) {
         value.erase(0, 1);
@@ -339,7 +368,7 @@ static PbiFilter CreateZmwFilter(std::string value, const Compare::Type compareT
     }
 
     if (isList(value)) {
-        std::vector<std::string> tokens = internal::Split(value, ',');
+        std::vector<std::string> tokens = Split(value, ',');
         std::vector<int32_t> zmws;
         zmws.reserve(tokens.size());
         for (const auto& t : tokens)
@@ -354,7 +383,7 @@ static PbiFilter CreateZmwModuloFilter(const Property& property)
     if (!property.HasAttribute("Modulo") || !property.HasAttribute("Hash") ||
         property.Name() != "zm")
     {
-        throw std::runtime_error{"Modulo filter not supported on property: "};
+        throw std::runtime_error{"PbiFilter: modulo filter is not supported on property: " + property.Name()};
     }
 
     const auto hashType = property.Attribute("Hash");
@@ -364,7 +393,7 @@ static PbiFilter CreateZmwModuloFilter(const Property& property)
             return FilterHash::UNSIGNED_LONG_CAST;
         if (boost::algorithm::to_lower_copy(hashType) == "boosthashcombine")
             return FilterHash::BOOST_HASH_COMBINE;
-        throw std::runtime_error{"unsuppoerted hash type: " + hashType};
+        throw std::runtime_error{"PbiFilter: unsuppoerted hash type: " + hashType};
     }();
 
     const uint32_t denom = std::stoul(property.Attribute("Modulo"));
@@ -407,6 +436,7 @@ static PbiFilter FromDataSetProperty(const Property& property, const DataSet& da
             case BuiltIn::BarcodeReverseFilter : return CreateBarcodeReverseFilter(value, compareType);
             case BuiltIn::LocalContextFilter   : return CreateLocalContextFilter(value, compareType);
             case BuiltIn::MovieNameFilter      : return CreateMovieNameFilter(value, compareType);
+            case BuiltIn::QIdFilter            : return CreateQIdFilter(value, compareType);
             case BuiltIn::QueryNameFilter      : return CreateQueryNameFilter(value, dataset, compareType);
             case BuiltIn::ReadGroupFilter      : return CreateReadGroupFilter(value, compareType);
             case BuiltIn::ReferenceIdFilter    : return CreateReferenceIdFilter(value, compareType);
@@ -417,7 +447,7 @@ static PbiFilter FromDataSetProperty(const Property& property, const DataSet& da
             case BuiltIn::QueryNamesFromFileFilter : return CreateQueryNamesFilterFromFile(value, dataset, compareType);
 
             default :
-            throw std::runtime_error{""};
+            throw std::runtime_error{"PbiFilter: invalid built-in filter requested"};
         }
         // clang-format on
 
@@ -426,7 +456,7 @@ static PbiFilter FromDataSetProperty(const Property& property, const DataSet& da
 
     } catch (std::exception& e) {
         std::ostringstream s;
-        s << "error: could not create filter from XML Property element:\n"
+        s << "PbiFilter: could not create filter from XML Property element:\n"
           << "  Name:     " << property.Name() << '\n'
           << "  Value:    " << property.Value() << '\n'
           << "  Operator: " << property.Operator() << '\n'
