@@ -9,14 +9,14 @@
 #include "pbbam/ReadAccuracyQuery.h"
 
 #include "pbbam/CompositeBamReader.h"
-#include "pbbam/MakeUnique.h"
 #include "pbbam/PbiFilterTypes.h"
 
 namespace PacBio {
 namespace BAM {
 
-struct ReadAccuracyQuery::ReadAccuracyQueryPrivate
+class ReadAccuracyQuery::ReadAccuracyQueryPrivate
 {
+public:
     ReadAccuracyQueryPrivate(const Accuracy accuracy, const Compare::Type compareType,
                              const DataSet& dataset)
         : reader_{PbiReadAccuracyFilter{accuracy, compareType}, dataset}
@@ -33,7 +33,7 @@ ReadAccuracyQuery::ReadAccuracyQuery(const Accuracy accuracy, const Compare::Typ
 {
 }
 
-ReadAccuracyQuery::~ReadAccuracyQuery() {}
+ReadAccuracyQuery::~ReadAccuracyQuery() = default;
 
 bool ReadAccuracyQuery::GetNext(BamRecord& r) { return d_->reader_.GetNext(r); }
 
