@@ -11,14 +11,14 @@
 #include <cstdint>
 
 #include "pbbam/CompositeBamReader.h"
-#include "pbbam/MakeUnique.h"
 #include "pbbam/PbiFilterTypes.h"
 
 namespace PacBio {
 namespace BAM {
 
-struct SubreadLengthQuery::SubreadLengthQueryPrivate
+class SubreadLengthQuery::SubreadLengthQueryPrivate
 {
+public:
     SubreadLengthQueryPrivate(const int32_t length, const Compare::Type compareType,
                               const DataSet& dataset)
         : reader_(PbiQueryLengthFilter(length, compareType), dataset)
@@ -35,7 +35,7 @@ SubreadLengthQuery::SubreadLengthQuery(const int32_t length, const Compare::Type
 {
 }
 
-SubreadLengthQuery::~SubreadLengthQuery() {}
+SubreadLengthQuery::~SubreadLengthQuery() = default;
 
 bool SubreadLengthQuery::GetNext(BamRecord& r) { return d_->reader_.GetNext(r); }
 
