@@ -25,12 +25,11 @@
 #include "pbbam/BamHeader.h"
 #include "pbbam/BamRecord.h"
 #include "pbbam/EntireFileQuery.h"
-#include "pbbam/MakeUnique.h"
 #include "pbbam/ReadGroupInfo.h"
 
 namespace PacBio {
 namespace BAM {
-namespace internal {
+namespace {
 
 struct ilexcompare_wrapper
 {
@@ -92,7 +91,7 @@ static void ValidateReadGroup(const ReadGroupInfo& rg, std::unique_ptr<Validatio
 
     // valid read type
     if (!rg.ReadType().empty()) {
-        if (internal::AcceptedReadTypes.find(rg.ReadType()) == internal::AcceptedReadTypes.cend())
+        if (AcceptedReadTypes.find(rg.ReadType()) == AcceptedReadTypes.cend())
             errors->AddReadGroupError(id, "read type: " + rg.ReadType() + " is unknown");
     }
 
@@ -296,57 +295,7 @@ void ValidateRecordTagLengths(const BamRecord& b, std::unique_ptr<ValidationErro
             errors->AddTagLengthError(name, "IPD", "ip", b.IPD().size(), expectedLength);
     }
 
-    // NOTE: disabling "internal" tag checks for now, only checking "standard"
-    //       PacBioBAM tags
-
-    //    if (b.HasAltLabelQV()) {
-    //        if (b.AltLabelQV().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "AltLabelQV", "pv", b.AltLabelQV().size(), expectedLength);
-    //    }
-    //    if (b.HasAltLabelTag()) {
-    //        if (b.AltLabelTag().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "AltLabelTag", "pt", b.AltLabelTag().size(), expectedLength);
-    //    }
-    //    if (b.HasLabelQV()) {
-    //        if (b.LabelQV().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "LabelQV", "pq", b.LabelQV().size(), expectedLength);
-    //    }
-    //    if (b.HasPkmean()) {
-    //        if (b.Pkmean().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "Pkmean", "pa", b.Pkmean().size(), expectedLength);
-    //    }
-    //    if (b.HasPkmean2()) {
-    //        if (b.Pkmean2().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "Pkmean2", "ps", b.Pkmean2().size(), expectedLength);
-    //    }
-    //    if (b.HasPkmid()) {
-    //        if (b.Pkmid().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "Pkmid", "pm", b.Pkmid().size(), expectedLength);
-    //    }
-    //    if (b.HasPkmid2()) {
-    //        if (b.Pkmid2().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "Pkmid2", "pi", b.Pkmid2().size(), expectedLength);
-    //    }
-    //    if (b.HasPrePulseFrames()) {
-    //        if (b.PrePulseFrames().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "PrePulseFrames", "pd", b.PrePulseFrames().size(), expectedLength);
-    //    }
-    //    if (b.HasPulseCall()) {
-    //        if (b.PulseCall().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "PulseCall", "pc", b.PulseCall().size(), expectedLength);
-    //    }
-    //    if (b.HasPulseCallWidth()) {
-    //        if (b.PulseCallWidth().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "PulseCallWidth", "px", b.PulseCallWidth().size(), expectedLength);
-    //    }
-    //    if (b.HasPulseMergeQV()) {
-    //        if (b.PulseMergeQV().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "PulseMergeQV", "pg", b.PulseMergeQV().size(), expectedLength);
-    //    }
-    //    if (b.HasPulseWidth()) {
-    //        if (b.PulseWidth().size() != expectedLength)
-    //            errors->AddTagLengthError(name, "PulseWidth", "pw", b.PulseWidth().size(), expectedLength);
-    //    }
+    // NOTE: disabling "internal" tag checks for now, only production tags
 }
 
 void ValidateUnmappedRecord(const BamRecord& b, std::unique_ptr<ValidationErrors>& errors)
@@ -368,39 +317,80 @@ static void ValidateRecord(const BamRecord& b, std::unique_ptr<ValidationErrors>
         ValidateUnmappedRecord(b, errors);
 }
 
-}  // namespace internal
+}  // namespace
 
-using internal::ValidationErrors;
+bool Validator::IsValid(const BamFile& file, const bool entireFile)
+{
+    try {
+        if (entireFile)
+            ValidateEntireFile(file, 1);
+        else
+            ValidateFileMetadata(file, 1);
+        return true;
+    } catch (std::exception&) {
+        return false;
+    }
+}
+
+bool Validator::IsValid(const BamHeader& header)
+{
+    try {
+        Validate(header, 1);
+        return true;
+    } catch (std::exception&) {
+        return false;
+    }
+}
+
+bool Validator::IsValid(const BamRecord& record)
+{
+    try {
+        Validate(record, 1);
+        return true;
+    } catch (std::exception&) {
+        return false;
+    }
+}
+
+bool Validator::IsValid(const ReadGroupInfo& rg)
+{
+    try {
+        Validate(rg, 1);
+        return true;
+    } catch (std::exception&) {
+        return false;
+    }
+}
 
 void Validator::Validate(const BamHeader& header, const size_t maxErrors)
 {
     auto errors = std::make_unique<ValidationErrors>(maxErrors);
-    internal::ValidateHeader(header, "unknown", errors);
+    ValidateHeader(header, "unknown", errors);
     if (!errors->IsEmpty()) errors->ThrowErrors();
 }
 
 void Validator::Validate(const ReadGroupInfo& rg, const size_t maxErrors)
 {
     auto errors = std::make_unique<ValidationErrors>(maxErrors);
-    internal::ValidateReadGroup(rg, errors);
+    ValidateReadGroup(rg, errors);
     if (!errors->IsEmpty()) errors->ThrowErrors();
 }
 
 void Validator::Validate(const BamRecord& b, const size_t maxErrors)
 {
     auto errors = std::make_unique<ValidationErrors>(maxErrors);
-    internal::ValidateRecord(b, errors);
+    ValidateRecord(b, errors);
     if (!errors->IsEmpty()) errors->ThrowErrors();
 }
 
 void Validator::ValidateEntireFile(const BamFile& file, const size_t maxErrors)
 {
     auto errors = std::make_unique<ValidationErrors>(maxErrors);
-    internal::ValidateMetadata(file, errors);
+    ValidateMetadata(file, errors);
 
     EntireFileQuery query(file);
     for (const BamRecord& record : query)
-        internal::ValidateRecord(record, errors);
+        ValidateRecord(record, errors);
 
     if (!errors->IsEmpty()) errors->ThrowErrors();
 }
@@ -408,7 +398,7 @@ void Validator::ValidateEntireFile(const BamFile& file, const size_t maxErrors)
 void Validator::ValidateFileMetadata(const BamFile& file, const size_t maxErrors)
 {
     auto errors = std::make_unique<ValidationErrors>(maxErrors);
-    internal::ValidateMetadata(file, errors);
+    ValidateMetadata(file, errors);
     if (!errors->IsEmpty()) errors->ThrowErrors();
 }
 
