@@ -11,13 +11,12 @@
 #include <sstream>
 #include <stdexcept>
 
-#include "SequenceUtils.h"
+#include "pbbam/StringUtilities.h"
 
 namespace PacBio {
 namespace BAM {
-namespace internal {
 
-const Version Version::Current = Version(3, 0, 5);
+const Version Version::Current = Version(3, 0, 7);
 const Version Version::Minimum = Version(3, 0, 1);
 
 // string must be "<major>.<minor>.<version>"
@@ -25,16 +24,16 @@ Version::Version(const std::string& v) : major_{0}, minor_{0}, revision_{0}
 {
     // parse string
     try {
-        const auto fields = internal::Split(v, '.');
+        const auto fields = Split(v, '.');
         const auto numFields = fields.size();
-        if (numFields == 0) throw std::runtime_error{"invalid version number - empty string"};
+        if (numFields == 0) throw std::runtime_error{"Version: empty string"};
         major_ = std::stoi(fields.at(0));
         if (numFields > 1) {
             minor_ = std::stoi(fields.at(1));
             if (numFields > 2) revision_ = std::stoi(fields.at(2));
         }
     } catch (std::exception&) {
-        throw std::runtime_error{"invalid version number (" + v + "): failed to parse"};
+        throw std::runtime_error{"Version: could not parse: " + v};
     }
 
     // ensure valid numbers
@@ -48,6 +47,5 @@ std::string Version::ToString() const
     return s.str();
 }
 
-}  // namespace internal
 }  // namespace BAM
 }  // namespace PacBio
