@@ -3,6 +3,8 @@
 #ifndef XMLREADER_H
 #define XMLREADER_H
 
+#include "pbbam/Config.h"
+
 #include <iosfwd>
 #include <memory>
 
@@ -10,7 +12,6 @@
 
 namespace PacBio {
 namespace BAM {
-namespace internal {
 
 class XmlReader
 {
@@ -18,7 +19,6 @@ public:
     static std::unique_ptr<DataSetBase> FromStream(std::istream& in);
 };
 
-}  // namespace internal
 }  // namespace BAM
 }  // namespace PacBio
 
