@@ -3,6 +3,8 @@
 #ifndef XMLWRITER_H
 #define XMLWRITER_H
 
+#include "pbbam/Config.h"
+
 #include <iosfwd>
 #include <memory>
 
@@ -11,8 +13,6 @@ namespace BAM {
 
 class DataSetBase;
 
-namespace internal {
-
 class XmlWriter
 {
 public:
@@ -20,7 +20,6 @@ public:
     static void ToStream(const std::unique_ptr<DataSetBase>& dataset, std::ostream& out);
 };
 
-}  // namespace internal
 }  // namespace BAM
 }  // namespace PacBio
 
