@@ -15,16 +15,16 @@
 #include "MemoryUtils.h"
 #include "pbbam/BamRecord.h"
 #include "pbbam/CompositeBamReader.h"
-#include "pbbam/MakeUnique.h"
 #include "pbbam/PbiFilterTypes.h"
 
 namespace PacBio {
 namespace BAM {
 
-struct ZmwGroupQuery::ZmwGroupQueryPrivate
+class ZmwGroupQuery::ZmwGroupQueryPrivate
 {
     using ReaderType = PbiFilterCompositeBamReader<Compare::Zmw>;
 
+public:
     ZmwGroupQueryPrivate(const std::vector<int32_t>& zmwWhitelist, const DataSet& dataset)
         : whitelist_(zmwWhitelist.cbegin(), zmwWhitelist.cend())
     {
@@ -60,6 +60,7 @@ struct ZmwGroupQuery::ZmwGroupQueryPrivate
         return true;
     }
 
+private:
     std::deque<int32_t> whitelist_;
     std::unique_ptr<ReaderType> reader_;
 };
@@ -69,7 +70,7 @@ ZmwGroupQuery::ZmwGroupQuery(const std::vector<int32_t>& zmwWhitelist, const Dat
 {
 }
 
-ZmwGroupQuery::~ZmwGroupQuery() {}
+ZmwGroupQuery::~ZmwGroupQuery() = default;
 
 bool ZmwGroupQuery::GetNext(std::vector<BamRecord>& records) { return d_->GetNext(records); }
 
