@@ -11,14 +11,14 @@
 #include <cstdint>
 
 #include "pbbam/CompositeBamReader.h"
-#include "pbbam/MakeUnique.h"
 #include "pbbam/PbiFilterTypes.h"
 
 namespace PacBio {
 namespace BAM {
 
-struct ZmwQuery::ZmwQueryPrivate
+class ZmwQuery::ZmwQueryPrivate
 {
+public:
     ZmwQueryPrivate(std::vector<int32_t> zmwWhitelist, const DataSet& dataset)
         : reader_{PbiZmwFilter{std::move(zmwWhitelist)}, dataset}
     {
@@ -32,7 +32,7 @@ ZmwQuery::ZmwQuery(std::vector<int32_t> zmwWhitelist, const DataSet& dataset)
 {
 }
 
-ZmwQuery::~ZmwQuery() {}
+ZmwQuery::~ZmwQuery() = default;
 
 bool ZmwQuery::GetNext(BamRecord& r) { return d_->reader_.GetNext(r); }
 
