@@ -1,8 +1,11 @@
 // Author: Derek Barnett
 
+#include "../PbbamInternalConfig.h"
+
 #include <pbbam/vcf/VcfFormat.h>
 
 #include <cassert>
+#include <cmath>
 #include <fstream>
 #include <iostream>
 #include <sstream>
@@ -47,7 +50,7 @@ std::string QuotedText(const std::string& d) { return "\"" + d + "\""; }
 std::string UnquotedText(const std::string& d)
 {
     if (d.size() < 2 || d.front() != '"' || d.back() != '"')
-        throw std::runtime_error{"VCF format error: description text not quoted: " + d};
+        throw std::runtime_error{"VcfFormat: description text is not quoted: " + d};
     return d.substr(1, d.size() - 2);
 }
 
@@ -167,7 +170,7 @@ ContigDefinition VcfFormat::ParsedContigDefinition(std::string line)
     // substring between brackets
     const auto lastBracketPos = line.find_last_of('>');
     if (lastBracketPos == std::string::npos)
-        throw std::runtime_error{"VCF format error: malformed ##contig line: " + line};
+        throw std::runtime_error{"VcfFormat: malformed ##contig line: " + line};
     line = std::string(line.cbegin() + 10, line.cbegin() + lastBracketPos);
 
     std::string id;
@@ -177,7 +180,7 @@ ContigDefinition VcfFormat::ParsedContigDefinition(std::string line)
     for (const auto& field : fields) {
         const auto tokens = PacBio::BAM::Split(field, '=');
         if (tokens.size() != 2) {
-            throw std::runtime_error{"VCF format error: malformed ##contig line: " + line};
+            throw std::runtime_error{"VcfFormat: malformed ##contig line: " + line};
         }
         if (tokens[0] == Tokens::id)
             id = tokens[1];
@@ -196,7 +199,7 @@ FilterDefinition VcfFormat::ParsedFilterDefinition(std::string line)
     // substring between brackets
     const auto lastBracketPos = line.find_last_of('>');
     if (lastBracketPos == std::string::npos)
-        throw std::runtime_error{"VCF format error: malformed FILTER line: " + line};
+        throw std::runtime_error{"VcfFormat: malformed FILTER line: " + line};
     line = std::string(line.cbegin() + 10, line.cbegin() + lastBracketPos);
 
     std::string id;
@@ -206,14 +209,14 @@ FilterDefinition VcfFormat::ParsedFilterDefinition(std::string line)
     for (const auto& field : fields) {
         const auto tokens = PacBio::BAM::Split(field, '=');
         if (tokens.size() != 2) {
-            throw std::runtime_error{"VCF format error: malformed FILTER line: " + line};
+            throw std::runtime_error{"VcfFormat: malformed FILTER line: " + line};
         }
         if (tokens[0] == Tokens::id)
             id = tokens[1];
         else if (tokens[0] == Tokens::description) {
             description = UnquotedText(tokens[1]);
         } else
-            throw std::runtime_error{"VCF format error: unrecognized FILTER field: " + tokens[0]};
+            throw std::runtime_error{"VcfFormat: unrecognized FILTER field: " + tokens[0]};
     }
 
     return FilterDefinition{std::move(id), std::move(description)};
@@ -227,7 +230,7 @@ FormatDefinition VcfFormat::ParsedFormatDefinition(std::string line)
     // substring between brackets
     const auto lastBracketPos = line.find_last_of('>');
     if (lastBracketPos == std::string::npos)
-        throw std::runtime_error{"VCF format error: malformed FORMAT line: " + line};
+        throw std::runtime_error{"VcfFormat: malformed FORMAT line: " + line};
     line = std::string(line.cbegin() + 10, line.cbegin() + lastBracketPos);
 
     std::string id;
@@ -239,7 +242,7 @@ FormatDefinition VcfFormat::ParsedFormatDefinition(std::string line)
     for (const auto& field : fields) {
         const auto tokens = PacBio::BAM::Split(field, '=');
         if (tokens.size() != 2) {
-            throw std::runtime_error{"VCF format error: malformed FORMAT line: " + line};
+            throw std::runtime_error{"VcfFormat: malformed FORMAT line: " + line};
         }
         if (tokens[0] == Tokens::id)
             id = tokens[1];
@@ -250,7 +253,7 @@ FormatDefinition VcfFormat::ParsedFormatDefinition(std::string line)
         else if (tokens[0] == Tokens::description) {
             description = UnquotedText(tokens[1]);
         } else
-            throw std::runtime_error{"VCF format error: unrecognized FORMAT field: " + tokens[0]};
+            throw std::runtime_error{"VcfFormat: unrecognized FORMAT field: " + tokens[0]};
     }
 
     return FormatDefinition{std::move(id), std::move(number), std::move(type),
@@ -261,7 +264,7 @@ GeneralDefinition VcfFormat::ParsedGeneralDefinition(const std::string& line)
 {
     const auto tokens = PacBio::BAM::Split(line, '=');
     if (tokens.size() != 2 || tokens[0].find(Tokens::double_hash) != 0) {
-        throw std::runtime_error{"VCF format error: malformed header line: " + line};
+        throw std::runtime_error{"VcfFormat: malformed header line: " + line};
     }
 
     return GeneralDefinition{tokens[0].substr(2), tokens[1]};
@@ -275,7 +278,7 @@ InfoDefinition VcfFormat::ParsedInfoDefinition(std::string line)
     // substring between brackets
     const auto lastBracketPos = line.find_last_of('>');
     if (lastBracketPos == std::string::npos)
-        throw std::runtime_error{"VCF format error: malformed INFO line: " + line};
+        throw std::runtime_error{"VcfFormat: malformed INFO line: " + line};
     line = std::string(line.cbegin() + 8, line.cbegin() + lastBracketPos);
 
     std::string id;
@@ -289,7 +292,7 @@ InfoDefinition VcfFormat::ParsedInfoDefinition(std::string line)
     for (const auto& field : fields) {
         const auto tokens = PacBio::BAM::Split(field, '=');
         if (tokens.size() != 2) {
-            throw std::runtime_error{"VCF format error: malformed INFO line: " + line};
+            throw std::runtime_error{"VcfFormat: malformed INFO line: " + line};
         }
         if (tokens[0] == Tokens::id)
             id = tokens[1];
@@ -304,7 +307,7 @@ InfoDefinition VcfFormat::ParsedInfoDefinition(std::string line)
         } else if (tokens[0] == Tokens::version) {
             version = UnquotedText(tokens[1]);
         } else
-            throw std::runtime_error{"VCF format error: unrecognized INFO field: " + tokens[0]};
+            throw std::runtime_error{"VcfFormat: unrecognized INFO field: " + tokens[0]};
     }
 
     return InfoDefinition{std::move(id),          std::move(number), std::move(type),
@@ -323,7 +326,7 @@ VcfHeader VcfFormat::ParsedHeader(const std::string& hdrText)
     {
         auto genDef = ParsedGeneralDefinition(line);
         if (genDef.Id() != Tokens::file_format)
-            throw std::runtime_error{"VCF format error: file must begin with #fileformat line"};
+            throw std::runtime_error{"VcfFormat: file must begin with #fileformat line"};
         hdr.AddGeneralDefinition(std::move(genDef));
     }
 
@@ -371,10 +374,10 @@ VcfHeader VcfFormat::ParsedHeader(const std::string& hdrText)
             chromLineFound = true;
             break;
         } else
-            throw std::runtime_error{"VCF format error: unexpected line found in header:\n" + line};
+            throw std::runtime_error{"VcfFormat: unexpected line found in header:\n" + line};
     }
 
-    if (!chromLineFound) throw std::runtime_error{"VCF format error: CHROM column line is missing"};
+    if (!chromLineFound) throw std::runtime_error{"VcfFormat: CHROM column line is missing"};
 
     return hdr;
 }
@@ -404,7 +407,7 @@ VcfHeader VcfFormat::HeaderFromStream(std::istream& in)
 InfoField VcfFormat::ParsedInfoField(const std::string& text)
 {
     const auto& tokens = PacBio::BAM::Split(text, '=');
-    if (tokens.empty()) throw std::runtime_error{"VCF format error: malformed INFO field: " + text};
+    if (tokens.empty()) throw std::runtime_error{"VcfFormat: malformed INFO field: " + text};
 
     // required ID
     InfoField result;
@@ -434,11 +437,27 @@ std::vector<InfoField> VcfFormat::ParsedInfoFields(const std::string& text)
     return result;
 }
 
+GenotypeField VcfFormat::ParsedGenotypeField(const std::string& field)
+{
+    GenotypeField result;
+    const auto fieldValues = PacBio::BAM::Split(field, ':');
+    for (const auto& fieldValue : fieldValues) {
+        GenotypeData data;
+        const auto genotypeDataValues = PacBio::BAM::Split(fieldValue, ',');
+        if (genotypeDataValues.size() == 1)
+            data.value = genotypeDataValues.at(0);
+        else
+            data.values = genotypeDataValues;
+        result.data.push_back(std::move(data));
+    }
+    return result;
+}
+
 VcfVariant VcfFormat::ParsedVariant(const std::string& line)
 {
     const auto fields = PacBio::BAM::Split(line, '\t');
     if (fields.size() < 7)
-        throw std::runtime_error{"VCF format error: record is missing required fields: " + line};
+        throw std::runtime_error{"VcfFormat: record is missing required fields: " + line};
 
     // CHROM POS ID REF ALT REF
     auto chrom = fields.at(0);
@@ -463,34 +482,12 @@ VcfVariant VcfFormat::ParsedVariant(const std::string& line)
 
     // GENOTYPE (samples)
     if (fields.size() > 9) {
-        std::vector<std::string> genotypeIds;
-        const auto& formatField = fields.at(8);
-        const auto formatFields = PacBio::BAM::Split(formatField, ':');
-        for (auto&& genotypeId : formatFields)
-            genotypeIds.push_back(genotypeId);
-        var.GenotypeIds(std::move(genotypeIds));
+        var.GenotypeIds(PacBio::BAM::Split(fields.at(8), ':'));
 
-        // per-sample
-        std::vector<GenotypeField> sampleGenotypes;
-        for (size_t i = 9; i < fields.size(); ++i) {
-
-            GenotypeField g;
-
-            const auto& sampleField = fields.at(i);
-            const auto fieldValues = PacBio::BAM::Split(sampleField, ':');
-            for (const auto fieldValue : fieldValues) {
-                GenotypeData data;
-                const auto genotypeDataValues = PacBio::BAM::Split(fieldValue, ',');
-                if (genotypeDataValues.size() == 1)
-                    data.value = genotypeDataValues.at(0);
-                else
-                    data.values = genotypeDataValues;
-                g.data.push_back(std::move(data));
-            }
-
-            sampleGenotypes.push_back(std::move(g));
-        }
-        var.Genotypes(std::move(sampleGenotypes));
+        std::vector<GenotypeField> genotypes;
+        for (size_t i = 9; i < fields.size(); ++i)
+            genotypes.emplace_back(ParsedGenotypeField(fields.at(i)));
+        var.Genotypes(std::move(genotypes));
     }
 
     return var;
@@ -500,30 +497,36 @@ std::string VcfFormat::FormattedInfoField(const InfoField& field)
 {
     std::ostringstream out;
     out << field.id;
-    if (field.value.is_initialized()) {
+    if (field.value.is_initialized())
         out << '=' << field.value.get();
-    } else if (field.values.is_initialized()) {
-        out << '=';
-        bool first = true;
-        for (const auto& value : field.values.get()) {
-            if (!first) out << ',';
-            out << value;
-            first = false;
-        }
-    }
+    else if (field.values.is_initialized())
+        out << '=' << PacBio::BAM::Join(field.values.get(), ',');
     return out.str();
 }
 
 std::string VcfFormat::FormattedInfoFields(const std::vector<InfoField>& fields)
 {
-    std::ostringstream out;
-    bool first = true;
-    for (const auto field : fields) {
-        if (!first) out << ';';
-        out << FormattedInfoField(field);
-        first = false;
+    std::vector<std::string> result;
+    for (const auto& field : fields)
+        result.push_back(FormattedInfoField(field));
+    return PacBio::BAM::Join(result, ';');
+}
+
+std::string VcfFormat::FormattedGenotypeField(const GenotypeField& field)
+{
+    std::string result;
+    bool firstDataEntry = true;
+    for (const auto& d : field.data) {
+        if (!firstDataEntry) result += ':';
+        if (d.value.is_initialized())
+            result += d.value.get();
+        else {
+            assert(d.values.is_initialized());
+            result += PacBio::BAM::Join(d.values.get(), ',');
+        }
+        firstDataEntry = false;
     }
-    return out.str();
+    return result;
 }
 
 std::string VcfFormat::FormattedVariant(const VcfVariant& var)
@@ -536,34 +539,10 @@ std::string VcfFormat::FormattedVariant(const VcfVariant& var)
 
     const auto& genotypeIds = var.GenotypeIds();
     if (!genotypeIds.empty()) {
-        out << '\t';
-        bool firstId = true;
-        for (const auto id : genotypeIds) {
-            if (!firstId) out << ':';
-            out << id;
-            firstId = false;
-        }
-
-        const auto& sampleGenotypes = var.Genotypes();
-        for (const auto sampleGenotype : sampleGenotypes) {
-            out << '\t';
-            bool firstDataEntry = true;
-            for (const auto& d : sampleGenotype.data) {
-                if (!firstDataEntry) out << ':';
-                if (d.value.is_initialized())
-                    out << d.value.get();
-                else {
-                    assert(d.values.is_initialized());
-                    bool firstDatapoint = true;
-                    for (const auto& datapoint : d.values.get()) {
-                        if (!firstDatapoint) out << ',';
-                        out << datapoint;
-                        firstDatapoint = false;
-                    }
-                }
-                firstDataEntry = false;
-            }
-        }
+        out << '\t' << PacBio::BAM::Join(genotypeIds, ':');
+        const auto& genotypes = var.Genotypes();
+        for (const auto& genotype : genotypes)
+            out << '\t' << FormattedGenotypeField(genotype);
     }
     return out.str();
 }
