@@ -1,11 +1,14 @@
 // Author: Derek Barnett
 
+#include "../PbbamInternalConfig.h"
+
 #include <pbbam/vcf/VcfWriter.h>
 
+#include <cassert>
 #include <fstream>
 #include <iostream>
+#include <type_traits>
 
-#include <pbbam/MakeUnique.h>
 #include <pbbam/vcf/VcfFormat.h>
 #include <pbbam/vcf/VcfHeader.h>
 #include <pbbam/vcf/VcfVariant.h>
@@ -14,10 +17,15 @@
 namespace PacBio {
 namespace VCF {
 
-struct VcfWriter::VcfWriterPrivate : public PacBio::BAM::internal::FileProducer
+static_assert(!std::is_copy_constructible<VcfWriter>::value,
+              "VcfWriter(const VcfWriter&) is not = delete");
+static_assert(!std::is_copy_assignable<VcfWriter>::value,
+              "VcfWriter& operator=(const VcfWriter&) is not = delete");
+
+struct VcfWriter::VcfWriterPrivate : public PacBio::BAM::FileProducer
 {
     VcfWriterPrivate(std::string fn, const VcfHeader& header)
-        : PacBio::BAM::internal::FileProducer{std::move(fn)}, out_{TempFilename()}
+        : PacBio::BAM::FileProducer{std::move(fn)}, out_{TempFilename()}
     {
         out_ << VcfFormat::FormattedHeader(header) << '\n';
     }
@@ -36,9 +44,13 @@ VcfWriter::VcfWriter(std::string fn, const VcfHeader& header)
 {
 }
 
-bool VcfWriter::Write(const VcfVariant& var) { return d_->Write(var); }
+VcfWriter::VcfWriter(VcfWriter&&) noexcept = default;
 
-VcfWriter::~VcfWriter() {}
+VcfWriter& VcfWriter::operator=(VcfWriter&&) noexcept = default;
+
+VcfWriter::~VcfWriter() = default;
+
+bool VcfWriter::Write(const VcfVariant& var) { return d_->Write(var); }
 
 }  // namespace VCF
 }  // namespace PacBio
