@@ -17,7 +17,7 @@ namespace BamHeaderTests {
 
 struct BamHdrDeleter
 {
-    void operator()(bam_hdr_t* hdr)
+    void operator()(bam_hdr_t* hdr) const
     {
         if (hdr) bam_hdr_destroy(hdr);
         hdr = nullptr;
