@@ -7,9 +7,13 @@
 
 #include <gtest/gtest.h>
 
+#include "PbbamTestData.h"
+
+#include <pbbam/BamReader.h>
 #include <pbbam/BamRecord.h>
 #include <pbbam/BamRecordView.h>
 #include <pbbam/BamTagCodec.h>
+#include <pbbam/EntireFileQuery.h>
 
 // clang-format off
 
@@ -20,7 +24,7 @@ typedef std::vector<uint16_t> f_data;
 
 namespace BamRecordClippingTests {
 
-static 
+static
 ReadGroupInfo MakeReadGroup(const FrameCodec codec,
                             const std::string& movieName,
                             const std::string& readType)
@@ -176,26 +180,18 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
     const std::string s2_cigar_clipped = "3=3D4=";
     const std::string s3_cigar_clipped = "2=1D2I2D3=";
 
+    const std::string s1_rev_cigar_clipped = "7=";
+    const std::string s2_rev_cigar_clipped = "4=3D3=";
+    const std::string s3_rev_cigar_clipped = "3=1D2I2D2=";
+
     const BamRecord prototype = BamRecordClippingTests::MakeRecord(qStart, qEnd, seq, quals, tagBases, tagQuals, frames,
                                                   pulseCall, pulseBases, pulseQuals, pulseFrames);
 
-    BamRecord s0 = prototype; // unmapped record
-    BamRecord s1 = prototype.Mapped(tId, tPos, Strand::FORWARD, s1_cigar, mapQual);
-    BamRecord s2 = prototype.Mapped(tId, tPos, Strand::FORWARD, s2_cigar, mapQual);
-    BamRecord s3 = prototype.Mapped(tId, tPos, Strand::FORWARD, s3_cigar, mapQual);
-    BamRecord s1_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s1_cigar, mapQual);
-    BamRecord s2_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s2_cigar, mapQual);
-    BamRecord s3_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s3_cigar, mapQual);
+    {
+        SCOPED_TRACE("s0");
 
-    s0.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s1.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s2.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s3.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s1_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s2_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s3_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-
-    {   // s0
+        BamRecord s0 = prototype; // unmapped record
+        s0.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_FALSE(s0.IsMapped());
         EXPECT_EQ(clipStart, s0.QueryStart());
@@ -223,8 +219,11 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
         EXPECT_EQ(frames_clipped,    view.IPD().Data());
         EXPECT_EQ(pulseCall_clipped, view.PulseCalls());
     }
+    {
+        SCOPED_TRACE("s1 - FORWARD");
 
-    {   // s1 - FORWARD
+        BamRecord s1 = prototype.Mapped(tId, tPos, Strand::FORWARD, s1_cigar, mapQual);
+        s1.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s1.IsMapped());
         EXPECT_EQ(Strand::FORWARD, s1.AlignedStrand());
@@ -255,19 +254,31 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
         EXPECT_EQ(frames_clipped,    view.IPD().Data());
         EXPECT_EQ(pulseCall_clipped, view.PulseCalls());
     }
+    {
+        SCOPED_TRACE("s1 - REVERSE");
 
-    {   // s1 - REVERSE
+        BamRecord s1_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s1_cigar, mapQual);
 
         EXPECT_TRUE(s1_rev.IsMapped());
         EXPECT_EQ(Strand::REVERSE, s1_rev.AlignedStrand());
-        EXPECT_EQ(clipStart, s1_rev.QueryStart());
-        EXPECT_EQ(clipEnd,   s1_rev.QueryEnd());
-        EXPECT_EQ(clipStart, s1_rev.AlignedStart());    // queryStart (no soft clips)
-        EXPECT_EQ(clipEnd,   s1_rev.AlignedEnd());      // alignStart + seqLength
+        EXPECT_EQ(500,  s1_rev.QueryStart());
+        EXPECT_EQ(510,  s1_rev.QueryEnd());
+        EXPECT_EQ(500,  s1_rev.AlignedStart());    // queryStart (no soft clips)
+        EXPECT_EQ(510,  s1_rev.AlignedEnd());      // alignStart + seqLength
+        EXPECT_EQ(tPos, s1_rev.ReferenceStart());        // 100 + startOffset
+        EXPECT_EQ(110,  s1_rev.ReferenceEnd());          // RefStart + 7=
+        EXPECT_EQ(s1_cigar, s1_rev.CigarData().ToStdString());
+
+        s1_rev.Clip(ClipType::CLIP_TO_QUERY, 502, 509);
+        EXPECT_TRUE(s1_rev.IsMapped());
+        EXPECT_EQ(Strand::REVERSE, s1_rev.AlignedStrand());
+        EXPECT_EQ(502, s1_rev.QueryStart());
+        EXPECT_EQ(509, s1_rev.QueryEnd());
+        EXPECT_EQ(502, s1_rev.AlignedStart());    // queryStart (no soft clips)
+        EXPECT_EQ(509, s1_rev.AlignedEnd());      // alignStart + seqLength
         EXPECT_EQ(102, s1_rev.ReferenceStart());        // 100 + startOffset
         EXPECT_EQ(109, s1_rev.ReferenceEnd());          // RefStart + 7=
-
-        EXPECT_EQ(s1_cigar_clipped, s1_rev.CigarData().ToStdString());
+        EXPECT_EQ(s1_rev_cigar_clipped, s1_rev.CigarData().ToStdString());
 
         const BamRecordView view
         {
@@ -277,7 +288,6 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
             false,
             PulseBehavior::ALL
         };
-
         EXPECT_EQ(seq_rev_clipped,       view.Sequence());
         EXPECT_EQ(quals_rev_clipped,     view.Qualities().Fastq());
         EXPECT_EQ(tagBases_rev_clipped,  view.DeletionTags());
@@ -287,8 +297,11 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
         EXPECT_EQ(frames_rev_clipped,    view.IPD().Data());
         EXPECT_EQ(pulseCall_rev_clipped, view.PulseCalls());
     }
+    {
+        SCOPED_TRACE("s2 - FORWARD");
 
-    {   // s2 - FORWARD
+        BamRecord s2 = prototype.Mapped(tId, tPos, Strand::FORWARD, s2_cigar, mapQual);
+        s2.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s2.IsMapped());
         EXPECT_EQ(Strand::FORWARD, s2.AlignedStrand());
@@ -318,8 +331,11 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
         EXPECT_EQ(pulseQuals_clipped, view.AltLabelQVs().Fastq());
         EXPECT_EQ(frames_clipped,   view.IPD().Data());
     }
+    {
+        SCOPED_TRACE("s2 - REVERSE");
 
-    {   // s2 - REVERSE
+        BamRecord s2_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s2_cigar, mapQual);
+        s2_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s2_rev.IsMapped());
         EXPECT_EQ(Strand::REVERSE, s2_rev.AlignedStrand());
@@ -330,7 +346,7 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
         EXPECT_EQ(102, s2_rev.ReferenceStart());        // 100 + startOffset
         EXPECT_EQ(112, s2_rev.ReferenceEnd());          // RefStart + 7= + 3D
 
-        EXPECT_EQ(s2_cigar_clipped, s2_rev.CigarData().ToStdString());
+        EXPECT_EQ(s2_rev_cigar_clipped, s2_rev.CigarData().ToStdString());
 
         const BamRecordView view
         {
@@ -350,8 +366,11 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
         EXPECT_EQ(frames_rev_clipped,    view.IPD().Data());
         EXPECT_EQ(pulseCall_rev_clipped, view.PulseCalls());
     }
+    {
+        SCOPED_TRACE("s3 - FORWARD");
 
-    {   // s3 - FORWARD
+        BamRecord s3 = prototype.Mapped(tId, tPos, Strand::FORWARD, s3_cigar, mapQual);
+        s3.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s3.IsMapped());
         EXPECT_EQ(Strand::FORWARD, s3.AlignedStrand());
@@ -382,8 +401,11 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
         EXPECT_EQ(frames_clipped,    view.IPD().Data());
         EXPECT_EQ(pulseCall_clipped, view.PulseCalls());
     }
+    {
+        SCOPED_TRACE("s3 - REVERSE");
 
-    {   // s3 - REVERSE
+        BamRecord s3_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s3_cigar, mapQual);
+        s3_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s3_rev.IsMapped());
         EXPECT_EQ(Strand::REVERSE, s3_rev.AlignedStrand());
@@ -394,7 +416,7 @@ TEST(BamRecordClippingTest, ClipToQuery_Basic)
         EXPECT_EQ(102, s3_rev.ReferenceStart());         // 100 + startOffset
         EXPECT_EQ(110, s3_rev.ReferenceEnd());           // RefStart + 5= + 3D
 
-        EXPECT_EQ(s3_cigar_clipped, s3_rev.CigarData().ToStdString());
+        EXPECT_EQ(s3_rev_cigar_clipped, s3_rev.CigarData().ToStdString());
 
         const BamRecordView view
         {
@@ -443,6 +465,7 @@ TEST(BamRecordClippingTest, ClipToQuery_WithSoftClips)
     const std::string s1_tagBases_clipped = s1_seq_clipped;
     const std::string s1_tagQuals_clipped = s1_quals_clipped;
     const f_data s1_frames_clipped   = { 10, 10, 20, 20, 30, 40, 40 };
+    const std::string s1_cigar_rev_clipped = "6=1S";
     const std::string s1_seq_rev_clipped   = "AACGGTT";
     const std::string s1_quals_rev_clipped = "?]?]?]?";
     const std::string s1_tagBases_rev_clipped = s1_seq_rev_clipped;
@@ -456,6 +479,7 @@ TEST(BamRecordClippingTest, ClipToQuery_WithSoftClips)
     const std::string s2_tagBases_clipped = s2_seq_clipped;
     const std::string s2_tagQuals_clipped = s2_quals_clipped;
     const f_data s2_frames_clipped   = { 10, 10, 20, 20, 30, 40, 40 };
+    const std::string s2_cigar_rev_clipped = "1=3D5=1S";
     const std::string s2_seq_rev_clipped   = "AACGGTT";
     const std::string s2_quals_rev_clipped = "?]?]?]?";
     const std::string s2_tagBases_rev_clipped = s2_seq_rev_clipped;
@@ -469,6 +493,7 @@ TEST(BamRecordClippingTest, ClipToQuery_WithSoftClips)
     const std::string s3_tagBases_clipped = s3_seq_clipped;
     const std::string s3_tagQuals_clipped = s3_quals_clipped;
     const f_data s3_frames_clipped   = { 10, 10, 20, 20, 30, 40, 40 };
+    const std::string s3_cigar_rev_clipped = "1D2I2D4=1S";
     const std::string s3_seq_rev_clipped   = "AACGGTT";
     const std::string s3_quals_rev_clipped = "?]?]?]?";
     const std::string s3_tagBases_rev_clipped = s3_seq_rev_clipped;
@@ -477,46 +502,16 @@ TEST(BamRecordClippingTest, ClipToQuery_WithSoftClips)
 
     const BamRecord prototype = BamRecordClippingTests::MakeRecord(qStart, qEnd, seq, quals, tagBases, tagQuals, frames,
                                                   seq, tagBases, tagQuals, frames);
-    BamRecord s1 = prototype.Mapped(tId, tPos, Strand::FORWARD, s1_cigar, mapQual);
-    BamRecord s2 = prototype.Mapped(tId, tPos, Strand::FORWARD, s2_cigar, mapQual);
-    BamRecord s3 = prototype.Mapped(tId, tPos, Strand::FORWARD, s3_cigar, mapQual);
-    BamRecord s1_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s1_cigar, mapQual);
-    BamRecord s2_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s2_cigar, mapQual);
-    BamRecord s3_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s3_cigar, mapQual);
 
-    // sanity checks before clipping
-    EXPECT_TRUE(s1.IsMapped());
-    EXPECT_EQ(tPos, s1.ReferenceStart());
-    EXPECT_EQ(tPos + 10, s1.ReferenceEnd()); // 10=
+    {
+        SCOPED_TRACE("s1 - FORWARD");
 
-    EXPECT_TRUE(s1_rev.IsMapped());
-    EXPECT_EQ(tPos, s1_rev.ReferenceStart());
-    EXPECT_EQ(tPos + 10, s1_rev.ReferenceEnd()); // 10=
+        BamRecord s1 = prototype.Mapped(tId, tPos, Strand::FORWARD, s1_cigar, mapQual);
+        EXPECT_TRUE(s1.IsMapped());
+        EXPECT_EQ(100, s1.ReferenceStart());
+        EXPECT_EQ(110, s1.ReferenceEnd()); // 10=
 
-    EXPECT_TRUE(s2.IsMapped());
-    EXPECT_EQ(tPos, s2.ReferenceStart());
-    EXPECT_EQ(tPos + 13, s2.ReferenceEnd());   // 5= + 3D + 5=
-
-    EXPECT_TRUE(s2_rev.IsMapped());
-    EXPECT_EQ(tPos, s2_rev.ReferenceStart());
-    EXPECT_EQ(tPos + 13, s2_rev.ReferenceEnd());   // 5= + 3D + 5=
-
-    EXPECT_TRUE(s3.IsMapped());
-    EXPECT_EQ(tPos, s3.ReferenceStart());
-    EXPECT_EQ(tPos + 11, s3.ReferenceEnd());   // 4= + 1D + 2D + 4=
-
-    EXPECT_TRUE(s3_rev.IsMapped());
-    EXPECT_EQ(tPos, s3_rev.ReferenceStart());
-    EXPECT_EQ(tPos + 11, s3_rev.ReferenceEnd());   // 4= + 1D + 2D + 4=
-
-    s1.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s2.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s3.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s1_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s2_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-    s3_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
-
-    {   // s1 - FORWARD
+        s1.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s1.IsMapped());
         EXPECT_EQ(Strand::FORWARD, s1.AlignedStrand());
@@ -544,19 +539,26 @@ TEST(BamRecordClippingTest, ClipToQuery_WithSoftClips)
         EXPECT_EQ(s1_tagQuals_clipped, view.DeletionQVs().Fastq());
         EXPECT_EQ(s1_frames_clipped,   view.IPD().Data());
     }
+    {
+        SCOPED_TRACE("s1 - REVERSE");
 
-    {   // s1 - REVERSE
+        BamRecord s1_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s1_cigar, mapQual);
+        EXPECT_TRUE(s1_rev.IsMapped());
+        EXPECT_EQ(100, s1_rev.ReferenceStart());
+        EXPECT_EQ(110, s1_rev.ReferenceEnd()); // 10=
+
+        s1_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s1_rev.IsMapped());
         EXPECT_EQ(Strand::REVERSE, s1_rev.AlignedStrand());
         EXPECT_EQ(clipStart, s1_rev.QueryStart());
         EXPECT_EQ(clipEnd,   s1_rev.QueryEnd());
-        EXPECT_EQ(clipStart, s1_rev.AlignedStart());    // queryStart (no soft clips)
-        EXPECT_EQ(clipEnd,   s1_rev.AlignedEnd());      // alignStart + seqLength
+        EXPECT_EQ(503, s1_rev.AlignedStart());    // queryStart (no soft clips)
+        EXPECT_EQ(509,   s1_rev.AlignedEnd());      // alignStart + seqLength
         EXPECT_EQ(tPos,      s1_rev.ReferenceStart());  // 100 + startOffset
-        EXPECT_EQ(tPos + 7,  s1_rev.ReferenceEnd());    // RefStart + 7=
+        EXPECT_EQ(tPos + 6,  s1_rev.ReferenceEnd());    // RefStart + 7=
 
-        EXPECT_EQ(s1_cigar_clipped, s1_rev.CigarData().ToStdString());
+        EXPECT_EQ(s1_cigar_rev_clipped, s1_rev.CigarData().ToStdString());
 
         const BamRecordView view
         {
@@ -573,8 +575,15 @@ TEST(BamRecordClippingTest, ClipToQuery_WithSoftClips)
         EXPECT_EQ(s1_tagQuals_rev_clipped, view.DeletionQVs().Fastq());
         EXPECT_EQ(s1_frames_rev_clipped,   view.IPD().Data());
     }
+    {
+        SCOPED_TRACE("s2 - FORWARD");
 
-    {   // s2 - FORWARD
+        BamRecord s2 = prototype.Mapped(tId, tPos, Strand::FORWARD, s2_cigar, mapQual);
+        EXPECT_TRUE(s2.IsMapped());
+        EXPECT_EQ(100, s2.ReferenceStart());
+        EXPECT_EQ(113, s2.ReferenceEnd());   // 5= + 3D + 5=
+
+        s2.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s2.IsMapped());
         EXPECT_EQ(Strand::FORWARD, s2.AlignedStrand());
@@ -602,19 +611,26 @@ TEST(BamRecordClippingTest, ClipToQuery_WithSoftClips)
         EXPECT_EQ(s2_tagQuals_clipped, view.DeletionQVs().Fastq());
         EXPECT_EQ(s2_frames_clipped,   view.IPD().Data());
     }
+    {
+        SCOPED_TRACE("s2 - REVERSE");
 
-    {   // s2 - REVERSE
+        BamRecord s2_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s2_cigar, mapQual);
+        EXPECT_TRUE(s2_rev.IsMapped());
+        EXPECT_EQ(100, s2_rev.ReferenceStart());
+        EXPECT_EQ(113, s2_rev.ReferenceEnd());   // 5= + 3D + 5=
+
+        s2_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s2_rev.IsMapped());
         EXPECT_EQ(Strand::REVERSE, s2_rev.AlignedStrand());
         EXPECT_EQ(clipStart, s2_rev.QueryStart());
         EXPECT_EQ(clipEnd,   s2_rev.QueryEnd());
-        EXPECT_EQ(clipStart, s2_rev.AlignedStart());    // queryStart (no soft clips left)
-        EXPECT_EQ(clipEnd,   s2_rev.AlignedEnd());      // alignStart + seqLength
+        EXPECT_EQ(503, s2_rev.AlignedStart());    // queryStart (no soft clips left)
+        EXPECT_EQ(509,   s2_rev.AlignedEnd());      // alignStart + seqLength
         EXPECT_EQ(tPos,      s2_rev.ReferenceStart());  // 100 + startOffset
-        EXPECT_EQ(tPos + 10, s2_rev.ReferenceEnd());    // RefStart + 5=3D2=
+        EXPECT_EQ(tPos + 9, s2_rev.ReferenceEnd());    // RefStart + 5=3D2=
 
-        EXPECT_EQ(s2_cigar_clipped, s2_rev.CigarData().ToStdString());
+        EXPECT_EQ(s2_cigar_rev_clipped, s2_rev.CigarData().ToStdString());
 
         const BamRecordView view
         {
@@ -632,8 +648,15 @@ TEST(BamRecordClippingTest, ClipToQuery_WithSoftClips)
         EXPECT_EQ(s2_tagQuals_rev_clipped, view.AltLabelQVs().Fastq());
         EXPECT_EQ(s2_frames_rev_clipped,   view.IPD().Data());
     }
+    {
+        SCOPED_TRACE("s3 - FORWARD");
 
-    {   // s3 - FORWARD
+        BamRecord s3 = prototype.Mapped(tId, tPos, Strand::FORWARD, s3_cigar, mapQual);
+        EXPECT_TRUE(s3.IsMapped());
+        EXPECT_EQ(100, s3.ReferenceStart());
+        EXPECT_EQ(111, s3.ReferenceEnd());   // 4= + 1D + 2D + 4=
+
+        s3.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
         EXPECT_TRUE(s3.IsMapped());
         EXPECT_EQ(Strand::FORWARD, s3.AlignedStrand());
@@ -663,18 +686,26 @@ TEST(BamRecordClippingTest, ClipToQuery_WithSoftClips)
         EXPECT_EQ(s3_tagQuals_clipped, view.AltLabelQVs().Fastq());
         EXPECT_EQ(s3_frames_clipped,   view.IPD().Data());
     }
+    {
+        SCOPED_TRACE("s3 - REVERSE");
+
+        BamRecord s3_rev = prototype.Mapped(tId, tPos, Strand::REVERSE, s3_cigar, mapQual);
+        EXPECT_TRUE(s3_rev.IsMapped());
+        EXPECT_EQ(100, s3_rev.ReferenceStart());
+        EXPECT_EQ(111, s3_rev.ReferenceEnd());   // 4= + 1D + 2D + 4=
+
+        s3_rev.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd);
 
-    {   // s3 - REVERSE
         EXPECT_TRUE(s3_rev.IsMapped());
         EXPECT_EQ(Strand::REVERSE, s3_rev.AlignedStrand());
         EXPECT_EQ(clipStart, s3_rev.QueryStart());
         EXPECT_EQ(clipEnd,   s3_rev.QueryEnd());
-        EXPECT_EQ(clipStart, s3_rev.AlignedStart());    // queryStart (no soft clips left)
-        EXPECT_EQ(clipEnd,   s3_rev.AlignedEnd());      // alignStart + seqLength
+        EXPECT_EQ(503, s3_rev.AlignedStart());    // queryStart + 1S
+        EXPECT_EQ(509,   s3_rev.AlignedEnd());      // alignStart + seqLength
         EXPECT_EQ(tPos,      s3_rev.ReferenceStart());  // 100 + startOffset
-        EXPECT_EQ(tPos + 8,  s3_rev.ReferenceEnd());    // RefStart + 4=1D2D1=
+        EXPECT_EQ(tPos + 7,  s3_rev.ReferenceEnd());    // RefStart + 4=1D2D1=
 
-        EXPECT_EQ(s3_cigar_clipped, s3_rev.CigarData().ToStdString());
+        EXPECT_EQ(s3_cigar_rev_clipped, s3_rev.CigarData().ToStdString());
 
         const BamRecordView view
         {
@@ -1776,6 +1807,10 @@ TEST(BamRecordTest, ClipEncodedFrames)
     const std::string s2_cigar_clipped = "3=3D4=";
     const std::string s3_cigar_clipped = "2=1D2I2D3=";
 
+    const std::string s1_cigar_rev_clipped = "7=";
+    const std::string s2_cigar_rev_clipped = "4=3D3=";
+    const std::string s3_cigar_rev_clipped = "3=1D2I2D2=";
+
     const BamRecord prototype = BamRecordClippingTests::MakeRecord(qStart, qEnd, seq, quals, tagBases, tagQuals, frames,
                                                   pulseCall, pulseBases, pulseQuals, pulseFrames, FrameCodec::V1);
 
@@ -1867,7 +1902,7 @@ TEST(BamRecordTest, ClipEncodedFrames)
         EXPECT_EQ(102, s1_rev.ReferenceStart());        // 100 + startOffset
         EXPECT_EQ(109, s1_rev.ReferenceEnd());          // RefStart + 7=
 
-        EXPECT_EQ(s1_cigar_clipped, s1_rev.CigarData().ToStdString());
+        EXPECT_EQ(s1_cigar_rev_clipped, s1_rev.CigarData().ToStdString());
 
         const BamRecordView view
         {
@@ -1930,7 +1965,7 @@ TEST(BamRecordTest, ClipEncodedFrames)
         EXPECT_EQ(102, s2_rev.ReferenceStart());        // 100 + startOffset
         EXPECT_EQ(112, s2_rev.ReferenceEnd());          // RefStart + 7= + 3D
 
-        EXPECT_EQ(s2_cigar_clipped, s2_rev.CigarData().ToStdString());
+        EXPECT_EQ(s2_cigar_rev_clipped, s2_rev.CigarData().ToStdString());
 
         const BamRecordView view
         {
@@ -1994,7 +2029,7 @@ TEST(BamRecordTest, ClipEncodedFrames)
         EXPECT_EQ(102, s3_rev.ReferenceStart());         // 100 + startOffset
         EXPECT_EQ(110, s3_rev.ReferenceEnd());           // RefStart + 5= + 3D
 
-        EXPECT_EQ(s3_cigar_clipped, s3_rev.CigarData().ToStdString());
+        EXPECT_EQ(s3_cigar_rev_clipped, s3_rev.CigarData().ToStdString());
 
         const BamRecordView view
         {
@@ -2014,6 +2049,352 @@ TEST(BamRecordTest, ClipEncodedFrames)
         EXPECT_EQ(frames_rev_clipped,    view.IPD().Data());
         EXPECT_EQ(pulseCall_rev_clipped, view.PulseCalls());
     }
+}
+
+TEST(BamRecordClippingTest, ExciseSoftClipsFromFramesWithDeletions)
+{
+    const std::string expectedName{"m141008_060349_42194_c100704972550000001823137703241586_s1_p0/14/2409_2745"};
+    const PacBio::BAM::Strand expectedStrand = PacBio::BAM::Strand::FORWARD;
+    const std::string expectedCigar{
+        "20S11=1I47=1I2=1I6=1I22=1I2=1I9=1I29=1D6=1I16=1I6=1I7=1I8=2I5=1I5=1I11=1I5=5I2=3I1=1I1=1I1=3I5=2D19=1I14=1I17=28S"};
+    const std::string expectedRawSeq{
+        "CCCCGGGATTCCTCTAGATGCATCAGGTAAGAAAAGTACGATGCTACAGCTTGTGACTGGTGCGGCACTT"
+        "TTGGCTGAGTTTATCCTGTGCCACCTCATGTATTCTGCCCTAGACAGTCGGTCTTGCACGCCATTACTAG"
+        "ACCGACAAAATGGAACCGGGGCCCTTAAACCCCGTTCGAAGGCGTAAGCAAGGAAGATAGGGTTTTATGA"
+        "AACTCTTCCCAGTCAATAATACCAAAAAAACCCCAACCAAGATCGTGACGGATTGCAGAGCGAATCCTAT"
+        "CCGCGCTCGCAATAATTTAGTGTTGATCCAAGCTTGCTGAGGACTAGTAAAGCTTC"};
+    const std::string expectedClippedSeq{
+        "CATCAGGTAAGAAAAGTACGATGCTACAGCTTGTGACTGGTGCGGCACTTTTGGCTGAGTTTATCCTGTG"
+        "CCACCTCATGTATTCTGCCCTAGACAGTCGGTCTTGCACGCCATTACTAGACCGACAAAATGGAACCGGG"
+        "GCCCTTAAACCCCGTTCGAAGGCGTAAGCAAGGAAGATAGGGTTTTATGAAACTCTTCCCAGTCAATAAT"
+        "ACCAAAAAAACCCCAACCAAGATCGTGACGGATTGCAGAGCGAATCCTATCCGCGCTCGCAATAATTTAG"
+        "TGTTGATC"};
+    const std::vector<uint8_t> expectedRawIpds{
+        17,3,8,3,4,1,14,8,2,1,21,3,1,17,22,13,10,9,89,7,4,5,3,17,8,8,18,58,14,
+        25,8,5,9,1,5,0,20,16,15,9,78,19,2,20,23,12,2,5,7,3,5,61,19,12,13,6,65,
+        18,105,2,34,94,3,38,69,16,5,76,1,21,5,3,2,0,32,23,26,9,3,4,18,2,2,12,19,
+        33,63,11,4,25,3,7,7,3,26,48,28,34,1,2,6,31,17,29,68,5,20,79,6,12,10,3,
+        43,72,21,65,8,45,17,14,13,20,7,3,5,8,0,17,11,65,6,7,8,3,6,11,4,1,80,4,
+        16,21,12,4,2,8,1,25,22,36,18,34,11,5,4,33,3,12,1,14,8,22,4,8,76,8,5,18,
+        32,5,33,47,255,36,9,26,2,6,47,0,35,8,8,0,5,37,40,1,11,8,39,60,8,42,0,3,
+        6,11,12,20,24,15,1,10,10,38,25,63,21,28,0,4,17,0,31,23,13,41,23,42,0,7,
+        33,7,23,11,50,30,2,44,21,182,44,105,231,33,255,59,189,253,17,13,7,28,40,
+        84,8,13,34,70,214,174,103,5,8,1,8,9,8,1,12,7,4,17,7,45,2,2,7,10,7,19,28,
+        31,3,18,0,42,0,8,2,9,2,1,11,25,1,35,36,1,7,5,17,12,39,8,31,1,40,41,4,18,
+        2,51,14,1,16,255,2,5,83,2,6,2,1,6,9,10,3,31,19,35,6,16,21,12,28,4,10,10,
+        12,1,105,17,2,11};
+    const std::vector<uint8_t> expectedClippedIpds{
+        4,5,3,17,8,8,18,58,14,25,8,5,9,1,5,0,20,16,15,9,78,19,2,20,23,12,2,5,7,
+        3,5,61,19,12,13,6,65,18,105,2,34,94,3,38,69,16,5,76,1,21,5,3,2,0,32,23,
+        26,9,3,4,18,2,2,12,19,33,63,11,4,25,3,7,7,3,26,48,28,34,1,2,6,31,17,29,
+        68,5,20,79,6,12,10,3,43,72,21,65,8,45,17,14,13,20,7,3,5,8,0,17,11,65,6,
+        7,8,3,6,11,4,1,80,4,16,21,12,4,2,8,1,25,22,36,18,34,11,5,4,33,3,12,1,14,
+        8,22,4,8,76,8,5,18,32,5,33,47,255,36,9,26,2,6,47,0,35,8,8,0,5,37,40,1,
+        11,8,39,60,8,42,0,3,6,11,12,20,24,15,1,10,10,38,25,63,21,28,0,4,17,0,31,
+        23,13,41,23,42,0,7,33,7,23,11,50,30,2,44,21,182,44,105,231,33,255,59,
+        189,253,17,13,7,28,40,84,8,13,34,70,214,174,103,5,8,1,8,9,8,1,12,7,4,17,
+        7,45,2,2,7,10,7,19,28,31,3,18,0,42,0,8,2,9,2,1,11,25,1,35,36,1,7,5,17,
+        12,39,8,31,1,40,41,4,18,2,51,14,1,16,255};
+
+    const std::string fn{PbbamTestsConfig::Data_Dir + "/softclip_deletions.bam"};
+    BamRecord record;
+    BamReader reader{fn};
+    ASSERT_TRUE(reader.GetNext(record));
+
+    EXPECT_EQ(expectedName, record.FullName());
+    EXPECT_EQ(expectedStrand, record.AlignedStrand());
+    EXPECT_EQ(expectedCigar, record.CigarData().ToStdString());
+
+    const auto rawSeq = record.Sequence(PacBio::BAM::Orientation::GENOMIC);
+    const auto clippedSeq = record.Sequence(PacBio::BAM::Orientation::GENOMIC, false, true);
+    EXPECT_EQ(expectedRawSeq, rawSeq);
+    EXPECT_EQ(expectedClippedSeq, clippedSeq);
+
+    ASSERT_TRUE(record.HasIPD());
+    const auto rawIpds = record.IPD(PacBio::BAM::Orientation::GENOMIC).Encode();
+    const auto clippedIpds = record.IPD(PacBio::BAM::Orientation::GENOMIC, false, true).Encode();
+    EXPECT_EQ(expectedRawIpds, rawIpds);
+    EXPECT_EQ(expectedClippedIpds, clippedIpds);
+}
+
+TEST(BamRecordTest, ClipToQuery_Stranded)
+{
+    using namespace PacBio::BAM;
+
+    const std::string bamFile{PbbamTestsConfig::Data_Dir + "/clip_to_query.bam"};
+
+    bool first = true;
+    EntireFileQuery query{bamFile};
+    for (auto& i : query)
+    {
+        Strand expectedStrand;
+        std::string scope;
+        if (first) {
+            expectedStrand = Strand::FORWARD;
+            scope = "First record (FORWARD strand)";
+        } else {
+            expectedStrand = Strand::REVERSE;
+            scope = "Second record (REVERSE strand)";
+        }
+
+        SCOPED_TRACE(scope);
+
+        // initial
+        EXPECT_EQ(2, i.ReferenceStart());
+        EXPECT_EQ(7, i.ReferenceEnd());
+        EXPECT_EQ(0, i.QueryStart());
+        EXPECT_EQ(8, i.QueryEnd());
+        EXPECT_EQ(expectedStrand, i.AlignedStrand());
+        EXPECT_EQ("1S4=1I1=1S", i.CigarData().ToStdString());
+
+        // first clip
+        i.Clip(ClipType::CLIP_TO_REFERENCE, 3, 6);
+        EXPECT_EQ(3, i.ReferenceStart());
+        EXPECT_EQ(6, i.ReferenceEnd());
+        EXPECT_EQ(2, i.QueryStart());
+        EXPECT_EQ(6, i.QueryEnd());
+        EXPECT_EQ(expectedStrand, i.AlignedStrand());
+        EXPECT_EQ("3=1I", i.CigarData().ToStdString());
+
+        // second clip
+        Position qS;
+        Position qE;
+        if (first) {
+            qS = i.QueryStart();
+            qE = i.QueryEnd() - 1;
+        } else {
+            qS = i.QueryStart() + 1;
+            qE = i.QueryEnd();
+        }
+        i.Clip(ClipType::CLIP_TO_QUERY, qS, qE);
+        EXPECT_EQ(3, i.ReferenceStart());
+        EXPECT_EQ(6, i.ReferenceEnd());
+        EXPECT_EQ(qS, i.QueryStart());
+        EXPECT_EQ(qE, i.QueryEnd());
+        EXPECT_EQ(expectedStrand, i.AlignedStrand());
+        EXPECT_EQ("3=", i.CigarData().ToStdString());
+
+        first = false;
+    }
+}
+
+TEST(BamRecordTest, ClippingFlankingInserts_IgnoredOnClipToQuery)
+{
+    const Position qStart = 500;
+    const Position qEnd   = 515;
+    const std::string seq      = "TTAACCGTTAGCAAA";
+    const std::string quals    = "--?]?]?]?]?*+++";
+    const std::string tagBases = "TTAACCGTTAGCAAA";
+    const std::string tagQuals = "--?]?]?]?]?*+++";
+    const f_data frames = { 40, 40, 10, 10, 20, 20, 30, 40, 40, 10, 30, 20, 10, 10, 10 };
+
+    const BamRecord prototype =
+        BamRecordClippingTests::MakeRecord(qStart, qEnd, seq, quals, tagBases, tagQuals,
+                                           frames, seq, tagBases, tagQuals, frames);
+
+    {   // aligned forward
+
+        const int32_t  tId     = 0;
+        const Position tPos    = 100;
+        const uint8_t  mapQual = 80;
+        const Cigar cigar{"4I5=6I"};
+
+        BamRecord s = prototype.Mapped(tId, tPos, Strand::FORWARD, cigar, mapQual);
+        EXPECT_TRUE(s.IsMapped());
+        EXPECT_EQ(100, s.ReferenceStart());
+        EXPECT_EQ(105, s.ReferenceEnd());
+
+        const size_t clipStart = 502;
+        const size_t clipEnd = 512;
+        const bool exciseFlankingInserts = true;
+
+        s.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd, exciseFlankingInserts);
+
+        EXPECT_TRUE(s.IsMapped());
+        EXPECT_EQ(Strand::FORWARD, s.AlignedStrand());
+        EXPECT_EQ("2I5=3I", s.CigarData().ToStdString());
+
+        EXPECT_EQ(clipStart, s.QueryStart());
+        EXPECT_EQ(clipEnd,   s.QueryEnd());
+        EXPECT_EQ(clipStart, s.AlignedStart());
+        EXPECT_EQ(clipEnd,   s.AlignedEnd());
+        EXPECT_EQ(100,       s.ReferenceStart());
+        EXPECT_EQ(105,       s.ReferenceEnd());
+    }
+    {   // aligned reverse
+
+        const int32_t  tId     = 0;
+        const Position tPos    = 100;
+        const uint8_t  mapQual = 80;
+        const Cigar cigar{"4I5=6I"};
+
+        BamRecord s = prototype.Mapped(tId, tPos, Strand::REVERSE, cigar, mapQual);
+        EXPECT_TRUE(s.IsMapped());
+        EXPECT_EQ(100, s.ReferenceStart());
+        EXPECT_EQ(105, s.ReferenceEnd());
+
+        const size_t clipStart = 502;
+        const size_t clipEnd = 512;
+        const bool exciseFlankingInserts = true;
+
+        s.Clip(ClipType::CLIP_TO_QUERY, clipStart, clipEnd, exciseFlankingInserts);
+
+        EXPECT_TRUE(s.IsMapped());
+        EXPECT_EQ(Strand::REVERSE, s.AlignedStrand());
+        EXPECT_EQ("1I5=4I", s.CigarData().ToStdString());
+
+        EXPECT_EQ(clipStart, s.QueryStart());
+        EXPECT_EQ(clipEnd,   s.QueryEnd());
+        EXPECT_EQ(clipStart, s.AlignedStart());
+        EXPECT_EQ(clipEnd,   s.AlignedEnd());
+        EXPECT_EQ(100,       s.ReferenceStart());
+        EXPECT_EQ(105,       s.ReferenceEnd());
+    }
+}
+
+TEST(BamRecordTest, ClipToReference_Forward_ExciseFlankingInserts)
+{
+    const Position qStart = 500;
+    const Position qEnd   = 526;
+    const std::string seq      = "TTAACCGTTAGCAAATTAACCGTTAG";
+    const std::string quals    = "--?]?]?]?]?*+++--?]?]?]?]?";
+    const std::string tagBases = "TTAACCGTTAGCAAATTAACCGTTAG";
+    const std::string tagQuals = "--?]?]?]?]?*+++--?]?]?]?]?";
+    const f_data frames = {
+        40, 40, 10, 10, 20, 20, 30, 40, 40, 10,
+        30, 20, 10, 10, 10, 40, 40, 10, 10, 20,
+        20, 30, 40, 40, 10, 30 };
+
+    const BamRecord prototype =
+        BamRecordClippingTests::MakeRecord(qStart, qEnd, seq, quals, tagBases, tagQuals,
+                                           frames, seq, tagBases, tagQuals, frames);
+
+    const int32_t  tId     = 0;
+    const Position tPos    = 100;
+    const uint8_t  mapQual = 80;
+    const Cigar cigar{"3=6I10=6I1="};
+
+    const size_t clipStart = 103;
+    const size_t clipEnd = 113;
+
+    // ----------------
+    // keep inserts
+
+    bool exciseFlankingInserts = false;
+
+    BamRecord withInserts = prototype.Mapped(tId, tPos, Strand::FORWARD, cigar, mapQual);
+    EXPECT_TRUE(withInserts.IsMapped());
+    EXPECT_EQ(100, withInserts.ReferenceStart());
+    EXPECT_EQ(114, withInserts.ReferenceEnd());
+
+    withInserts.Clip(ClipType::CLIP_TO_REFERENCE, clipStart, clipEnd, exciseFlankingInserts);
+
+    EXPECT_TRUE(withInserts.IsMapped());
+    EXPECT_EQ(Strand::FORWARD, withInserts.AlignedStrand());
+    EXPECT_EQ("6I10=6I", withInserts.CigarData().ToStdString());
+
+    EXPECT_EQ(503, withInserts.QueryStart());
+    EXPECT_EQ(525, withInserts.QueryEnd());
+    EXPECT_EQ(503, withInserts.AlignedStart());
+    EXPECT_EQ(525, withInserts.AlignedEnd());
+    EXPECT_EQ(103, withInserts.ReferenceStart());
+    EXPECT_EQ(113, withInserts.ReferenceEnd());
+
+    // -----------------
+    // excise inserts
+
+    exciseFlankingInserts = true;
+
+    BamRecord withoutInserts = prototype.Mapped(tId, tPos, Strand::FORWARD, cigar, mapQual);
+    EXPECT_TRUE(withoutInserts.IsMapped());
+    EXPECT_EQ(100, withoutInserts.ReferenceStart());
+    EXPECT_EQ(114, withoutInserts.ReferenceEnd());
+
+    withoutInserts.Clip(ClipType::CLIP_TO_REFERENCE, clipStart, clipEnd, exciseFlankingInserts);
+
+    EXPECT_TRUE(withoutInserts.IsMapped());
+    EXPECT_EQ(Strand::FORWARD, withoutInserts.AlignedStrand());
+    EXPECT_EQ("10=", withoutInserts.CigarData().ToStdString());
+
+    EXPECT_EQ(509, withoutInserts.QueryStart());
+    EXPECT_EQ(519, withoutInserts.QueryEnd());
+    EXPECT_EQ(509, withoutInserts.AlignedStart());
+    EXPECT_EQ(519, withoutInserts.AlignedEnd());
+    EXPECT_EQ(103, withoutInserts.ReferenceStart());
+    EXPECT_EQ(113, withoutInserts.ReferenceEnd());
+}
+
+TEST(BamRecordTest, ClipToReference_Reverse_ExciseFlankingInserts)
+{
+    const Position qStart = 500;
+    const Position qEnd   = 526;
+    const std::string seq      = "TTAACCGTTAGCAAATTAACCGTTAG";
+    const std::string quals    = "--?]?]?]?]?*+++--?]?]?]?]?";
+    const std::string tagBases = "TTAACCGTTAGCAAATTAACCGTTAG";
+    const std::string tagQuals = "--?]?]?]?]?*+++--?]?]?]?]?";
+    const f_data frames = {
+        40, 40, 10, 10, 20, 20, 30, 40, 40, 10,
+        30, 20, 10, 10, 10, 40, 40, 10, 10, 20,
+        20, 30, 40, 40, 10, 30 };
+
+    const BamRecord prototype =
+        BamRecordClippingTests::MakeRecord(qStart, qEnd, seq, quals, tagBases, tagQuals,
+                                           frames, seq, tagBases, tagQuals, frames);
+
+    const int32_t  tId     = 0;
+    const Position tPos    = 100;
+    const uint8_t  mapQual = 80;
+    const Cigar cigar{"3=6I10=6I1="};
+
+    const size_t clipStart = 103;
+    const size_t clipEnd = 113;
+
+    // ----------------
+    // keep inserts
+
+    bool exciseFlankingInserts = false;
+
+    BamRecord withInserts = prototype.Mapped(tId, tPos, Strand::REVERSE, cigar, mapQual);
+
+    EXPECT_TRUE(withInserts.IsMapped());
+    EXPECT_EQ(100, withInserts.ReferenceStart());
+    EXPECT_EQ(114, withInserts.ReferenceEnd());
+
+    withInserts.Clip(ClipType::CLIP_TO_REFERENCE, clipStart, clipEnd, exciseFlankingInserts);
+
+    EXPECT_TRUE(withInserts.IsMapped());
+    EXPECT_EQ(Strand::REVERSE, withInserts.AlignedStrand());
+    EXPECT_EQ("6I10=6I", withInserts.CigarData().ToStdString());
+
+    EXPECT_EQ(501, withInserts.QueryStart());
+    EXPECT_EQ(523, withInserts.QueryEnd());
+    EXPECT_EQ(501, withInserts.AlignedStart());
+    EXPECT_EQ(523, withInserts.AlignedEnd());
+    EXPECT_EQ(103, withInserts.ReferenceStart());
+    EXPECT_EQ(113, withInserts.ReferenceEnd());
+
+    // -----------------
+    // excise inserts
+
+    exciseFlankingInserts = true;
+
+    BamRecord withoutInserts = prototype.Mapped(tId, tPos, Strand::REVERSE, cigar, mapQual);
+    EXPECT_TRUE(withoutInserts.IsMapped());
+    EXPECT_EQ(100, withoutInserts.ReferenceStart());
+    EXPECT_EQ(114, withoutInserts.ReferenceEnd());
+
+    withoutInserts.Clip(ClipType::CLIP_TO_REFERENCE, clipStart, clipEnd, exciseFlankingInserts);
+
+    EXPECT_TRUE(withoutInserts.IsMapped());
+    EXPECT_EQ(Strand::REVERSE, withoutInserts.AlignedStrand());
+    EXPECT_EQ("10=", withoutInserts.CigarData().ToStdString());
+
+    EXPECT_EQ(507, withoutInserts.QueryStart());
+    EXPECT_EQ(517, withoutInserts.QueryEnd());
+    EXPECT_EQ(507, withoutInserts.AlignedStart());
+    EXPECT_EQ(517, withoutInserts.AlignedEnd());
+    EXPECT_EQ(103, withoutInserts.ReferenceStart());
+    EXPECT_EQ(113, withoutInserts.ReferenceEnd());
 
 }
 
