@@ -132,7 +132,7 @@ TEST(DataSetIOTest, ThrowsOnNonexistentFofnFile)
     }
     catch(std::runtime_error& e)
     {
-        const std::string msg = "could not open FOFN for reading: does/not/exist.fofn";
+        const std::string msg = "DataSet: could not open FOFN for reading: does/not/exist.fofn";
         EXPECT_EQ(msg, e.what()) ;
         checkedException = true;
     }
@@ -148,7 +148,7 @@ TEST(DataSetIOTest, ThrowsOnNonexistentXmlFile)
     }
     catch(std::runtime_error& e)
     {
-        const std::string msg = "could not open XML file for reading: does/not/exist.xml";
+        const std::string msg = "DataSet: could not open XML file for reading: does/not/exist.xml";
         EXPECT_EQ(msg, e.what()) ;
         checkedException = true;
     }
@@ -164,7 +164,7 @@ TEST(DataSetIOTest, ThrowsOnUnsupportedExtension)
     }
     catch(std::runtime_error& e)
     {
-        const std::string msg = "unsupported extension on input file: bad/extension.foo";
+        const std::string msg = "DataSet: unsupported extension on input file: bad/extension.foo";
         EXPECT_EQ(msg, e.what()) ;
         checkedException = true;
     }
@@ -182,7 +182,7 @@ TEST(DataSetIOTest, ThrowsIfCannotOpenSaveFile)
     }
     catch(std::runtime_error& e)
     {
-        const std::string msg = "could not open XML file for writing: fake_directory_that_should_not_exist/out.xml";
+        const std::string msg = "DataSet: could not open XML file for writing: fake_directory_that_should_not_exist/out.xml";
         EXPECT_EQ(msg, e.what()) ;
         checkedException = true;
     }
@@ -204,25 +204,29 @@ TEST(DataSetIOTest, ToXml)
            .Attribute("xsi:schemaLocation", "http://pacificbiosciences.com/PacBioDatasets.xsd");
 
     // external resources
-    ExternalResource resource1("AlignmentFile.AlignmentBamFile", "file:/mnt/path/to/alignments2.bam");    
+    ExternalResource resource1("AlignmentFile.AlignmentBamFile", "/mnt/path/to/alignments2.bam");
     resource1.Name("Third Alignments BAM");
+    resource1.CreatedAt("2015-01-27T09:00:01");
     resource1.Description("Points to an example Alignments BAM file.");
     resource1.Tags("Example");
     resource1.TimeStampedName("my_tsn");
     resource1.UniqueId("my_uuid");
-    FileIndex pbi1("PacBio.Index.PacBioIndex", "file:/mnt/path/to/alignments2.pbi");
+    FileIndex pbi1("PacBio.Index.PacBioIndex", "/mnt/path/to/alignments2.pbi");
+    pbi1.CreatedAt("2015-01-27T09:00:01");
     pbi1.TimeStampedName("my_tsn");
     pbi1.UniqueId("my_uuid");
     resource1.FileIndices().Add(pbi1);
     dataset.ExternalResources().Add(resource1);
 
-    ExternalResource resource2("AlignmentFile.AlignmentBamFile", "file:./alignments3.bam");
+    ExternalResource resource2("AlignmentFile.AlignmentBamFile", "./alignments3.bam");
+    resource2.CreatedAt("2015-01-27T09:00:01");
     resource2.Name("Fourth Alignments BAM");
     resource2.Description("Points to another example Alignments BAM file, by relative path.");
     resource2.Tags("Example");
     resource2.TimeStampedName("my_tsn");
     resource2.UniqueId("my_uuid");
-    FileIndex pbi2("PacBio.Index.PacBioIndex", "file:/mnt/path/to/alignments3.pbi");
+    FileIndex pbi2("PacBio.Index.PacBioIndex", "./alignments3.pbi");
+    pbi2.CreatedAt("2015-01-27T09:00:01");
     pbi2.TimeStampedName("my_tsn");
     pbi2.UniqueId("my_uuid");
 
@@ -231,6 +235,7 @@ TEST(DataSetIOTest, ToXml)
 
     // sub-datasets with filters
     DataSetBase subDataSet1;
+    subDataSet1.CreatedAt("2015-01-27T09:00:01");
     subDataSet1.Name("HighQuality Read Alignments");
     subDataSet1.TimeStampedName("my_tsn");
     subDataSet1.UniqueId("ab95d0a3-94b8-4918-b3af-a3f81bbe519c");
@@ -240,6 +245,7 @@ TEST(DataSetIOTest, ToXml)
     dataset.SubDataSets().Add(subDataSet1);
 
     DataSetBase subDataSet2;
+    subDataSet2.CreatedAt("2015-01-27T09:00:01");
     subDataSet2.Name("Alignments to chromosome 1");
     subDataSet2.TimeStampedName("my_tsn");
     subDataSet2.UniqueId("ac95d0a3-94b8-4918-b3af-a3f81bbe519c");
@@ -265,33 +271,37 @@ TEST(DataSetIOTest, ToXml)
                 "xmlns:pbds=\"http://pacificbiosciences.com/PacBioDatasets.xsd\">\n"
         "\t<pbbase:ExternalResources>\n"
         "\t\t<pbbase:ExternalResource "
+                "CreatedAt=\"2015-01-27T09:00:01\" "
                 "Description=\"Points to an example Alignments BAM file.\" "
                 "MetaType=\"AlignmentFile.AlignmentBamFile\" "
                 "Name=\"Third Alignments BAM\" "
-                "ResourceId=\"file:/mnt/path/to/alignments2.bam\" "
+                "ResourceId=\"/mnt/path/to/alignments2.bam\" "
                 "Tags=\"Example\" "
                 "TimeStampedName=\"my_tsn\" "
                 "UniqueId=\"my_uuid\" Version=\"3.0.1\">\n"
         "\t\t\t<pbbase:FileIndices>\n"
         "\t\t\t\t<pbbase:FileIndex "
+                "CreatedAt=\"2015-01-27T09:00:01\" "
                 "MetaType=\"PacBio.Index.PacBioIndex\" "
-                "ResourceId=\"file:/mnt/path/to/alignments2.pbi\" "
+                "ResourceId=\"/mnt/path/to/alignments2.pbi\" "
                 "TimeStampedName=\"my_tsn\" "
                 "UniqueId=\"my_uuid\" Version=\"3.0.1\" />\n"
         "\t\t\t</pbbase:FileIndices>\n"
         "\t\t</pbbase:ExternalResource>\n"
         "\t\t<pbbase:ExternalResource "
+                "CreatedAt=\"2015-01-27T09:00:01\" "
                 "Description=\"Points to another example Alignments BAM file, by relative path.\" "
                 "MetaType=\"AlignmentFile.AlignmentBamFile\" "
                 "Name=\"Fourth Alignments BAM\" "
-                "ResourceId=\"file:./alignments3.bam\" "
+                "ResourceId=\"" + dataset.Path() + "/alignments3.bam\" "
                 "Tags=\"Example\" "
                 "TimeStampedName=\"my_tsn\" "
                 "UniqueId=\"my_uuid\" Version=\"3.0.1\">\n"
         "\t\t\t<pbbase:FileIndices>\n"
         "\t\t\t\t<pbbase:FileIndex "
+                "CreatedAt=\"2015-01-27T09:00:01\" "
                 "MetaType=\"PacBio.Index.PacBioIndex\" "
-                "ResourceId=\"file:/mnt/path/to/alignments3.pbi\" "
+                "ResourceId=\"" + dataset.Path() + "/alignments3.pbi\" "
                 "TimeStampedName=\"my_tsn\" "
                 "UniqueId=\"my_uuid\" Version=\"3.0.1\" />\n"
         "\t\t\t</pbbase:FileIndices>\n"
@@ -299,6 +309,7 @@ TEST(DataSetIOTest, ToXml)
         "\t</pbbase:ExternalResources>\n"
         "\t<pbds:DataSets>\n"
         "\t\t<pbds:DataSet "
+                "CreatedAt=\"2015-01-27T09:00:01\" "
                 "MetaType=\"PacBio.DataSet.DataSet\" "
                 "Name=\"HighQuality Read Alignments\" "
                 "TimeStampedName=\"my_tsn\" "
@@ -313,6 +324,7 @@ TEST(DataSetIOTest, ToXml)
         "\t\t\t</pbds:Filters>\n"
         "\t\t</pbds:DataSet>\n"
         "\t\t<pbds:DataSet "
+                "CreatedAt=\"2015-01-27T09:00:01\" "
                 "MetaType=\"PacBio.DataSet.DataSet\" "
                 "Name=\"Alignments to chromosome 1\" "
                 "TimeStampedName=\"my_tsn\" "
@@ -334,6 +346,37 @@ TEST(DataSetIOTest, ToXml)
     EXPECT_EQ(expectedXml, s.str());
 }
 
+TEST(DataSetIOTest, DataSetBaseTypeToXml)
+{
+    // top-level data
+    ContigSet dataset;
+    dataset.CreatedAt("2015-01-27T09:00:01");
+    dataset.Name("DataSet_ContigSet");
+    dataset.Tags("barcode moreTags mapping mytags");
+    dataset.TimeStampedName("my_tsn");
+    dataset.UniqueId("b095d0a3-94b8-4918-b3af-a3f81bbe519c");
+
+    // write dataset
+    const std::string expectedXml{
+        "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+        "<pbds:ContigSet "
+                "CreatedAt=\"2015-01-27T09:00:01\" "
+                "MetaType=\"PacBio.DataSet.ContigSet\" "
+                "Name=\"DataSet_ContigSet\" "
+                "Tags=\"barcode moreTags mapping mytags\" "
+                "TimeStampedName=\"my_tsn\" "
+                "UniqueId=\"b095d0a3-94b8-4918-b3af-a3f81bbe519c\" Version=\"3.0.1\" "
+                "xmlns=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" "
+                "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
+                "xsi:schemaLocation=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" "
+                "xmlns:pbds=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" />\n"};
+
+        std::ostringstream s;
+        dataset.SaveToStream(s);
+        EXPECT_EQ(expectedXml, s.str());
+}
+
+
 namespace DataSetIOTests {
 
 static void TestFromXmlString()
@@ -417,7 +460,7 @@ static void TestFromXmlString()
     EXPECT_EQ("http://www.w3.org/2001/XMLSchema-instance",         dataset.Attribute("xmlns:xsi"));
 
     const ExternalResources& resources = dataset.ExternalResources();
-    EXPECT_EQ(2, resources.Size());
+    EXPECT_EQ(2, resources.NumChildren());
 
     const ExternalResource& resource1 = resources[0];
     EXPECT_EQ("Third Alignments BAM",                      resource1.Name());
@@ -1099,10 +1142,10 @@ static void TestReferenceXml()
     EXPECT_EQ(std::string("Tribble"), metadata.ChildText("Organism"));
     EXPECT_EQ(std::string("Diploid"), metadata.ChildText("Ploidy"));
 
-    const internal::DataSetListElement<internal::DataSetElement>& contigs =
-            metadata.Child<internal::DataSetListElement<internal::DataSetElement> >("Contigs");
+    const internal::DataSetElement& contigs = metadata.Child<internal::DataSetElement>("Contigs");
     ASSERT_EQ(1, contigs.NumChildren());
-    const internal::DataSetElement& contig = contigs[0];
+
+    const internal::DataSetElement& contig = contigs.Child<internal::DataSetElement>(0);
     EXPECT_EQ(std::string("gi|229359445|emb|AM181176.4|"), contig.Attribute("Name"));
     EXPECT_EQ(std::string("Pseudomonas fluorescens SBW25 complete genome|quiver"), contig.Attribute("Description"));
     EXPECT_EQ(std::string("6722109"), contig.Attribute("Length"));
@@ -1466,6 +1509,7 @@ TEST(DataSetIOTest, RelativePathCarriedThroughOk_FromString)
             "UniqueId=\"b095d0a3-94b8-4918-b3af-a3f81bbe519c\" "
             "Version=\"2.3.0\" "
             "xmlns=\"http://pacificbiosciences.com/PacBioDataModel.xsd\" "
+            "xmlns:pbbase=\"http://pacificbiosciences.com/PacBioBaseDataModel.xsd\" "
             "xmlns:pbds=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" "
             "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
             "xsi:schemaLocation=\"http://pacificbiosciences.com/PacBioDataModel.xsd\">\n"
@@ -1527,11 +1571,11 @@ TEST(DataSetIOTest, RelativePathCarriedThroughOk_FromFile)
 TEST(DataSetIOTest, DataSetFromRelativeBamFilename)
 {
     // cache initial directory and move to location so we can test relatvie filename ok
-    const std::string startingDirectory = internal::FileUtils::CurrentWorkingDirectory();
+    const std::string startingDirectory = FileUtils::CurrentWorkingDirectory();
 
     const std::string targetDirectory = PbbamTestsConfig::Data_Dir + "/dataset";
     DataSetIOTests::changeCurrentDirectory(targetDirectory);
-    ASSERT_EQ(targetDirectory, internal::FileUtils::CurrentWorkingDirectory());
+    ASSERT_EQ(targetDirectory, FileUtils::CurrentWorkingDirectory());
 
     EXPECT_NO_THROW(
     {
@@ -1545,7 +1589,7 @@ TEST(DataSetIOTest, DataSetFromRelativeBamFilename)
     DataSetIOTests::changeCurrentDirectory(startingDirectory);
 }
 
-TEST(DataaSetIOTest, AllFiles) 
+TEST(DataaSetIOTest, AllFiles)
 {
     // check  BamFiles only
     EXPECT_NO_THROW(
@@ -1560,7 +1604,7 @@ TEST(DataaSetIOTest, AllFiles)
     {
         const DataSet dataset(PbbamTestsConfig::Data_Dir + "/chunking/chunking.subreadset.xml");
         const auto allFiles = dataset.AllFiles();
-        EXPECT_EQ(6, allFiles.size());    
+        EXPECT_EQ(6, allFiles.size());
     });
 }
 
@@ -1578,6 +1622,7 @@ TEST(DataSetIOTest, MetadataDefaultChildrenProperlyOrderedPerXsd)
            .Attribute("xsi:schemaLocation", "http://pacificbiosciences.com/PacBioDatasets.xsd");
 
     ExternalResource ext("Fake.MetaType", "filename");
+    ext.CreatedAt("2015-01-27T09:00:01");
     ext.TimeStampedName("custom_tsn")
        .UniqueId("my_uuid");
     dataset.ExternalResources().Add(ext);
@@ -1587,7 +1632,65 @@ TEST(DataSetIOTest, MetadataDefaultChildrenProperlyOrderedPerXsd)
     DataSetMetadata metadata(numRecords, totalLength);
     dataset.Metadata(metadata);
 
-    const std::string expectedXml{
+    std::ostringstream s;
+    dataset.SaveToStream(s);
+
+    const std::string result = s.str();
+    const size_t xmlnsFound = result.find("xmlns=");
+    const size_t xmlnsXsiFound = result.find("xmlns:xsi=");
+    const size_t xsiSchemaLocationFound = result.find("xsi:schemaLocation=");
+    const size_t xmlnsPbbaseFound = result.find("xmlns:pbbase=");
+    const size_t xmlnsPbdsFound = result.find("xmlns:pbds=");
+
+    EXPECT_TRUE(xmlnsFound < xmlnsXsiFound);
+    EXPECT_TRUE(xmlnsXsiFound < xsiSchemaLocationFound);
+    EXPECT_TRUE(xsiSchemaLocationFound < xmlnsPbbaseFound);
+    EXPECT_TRUE(xmlnsPbbaseFound < xmlnsPbdsFound);
+}
+
+TEST(DataSetIOTest, MakeReferenceSetFromSubdataset)
+{
+    // ReferenceSet with ReferenceSet subdataset
+    const std::string referenceSetXml{
+        "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+        "<pbds:ReferenceSet CreatedAt=\"2015-01-27T09:00:01\" MetaType=\"PacBio.DataSet.ReferenceSet\" "
+                "Name=\"DataSet_ReferenceSet\" Tags=\"barcode moreTags mapping mytags\" "
+                "TimeStampedName=\"my_time_stamped_name\" "
+                "UniqueId=\"b095d0a3-94b8-4918-b3af-a3f81bbe519c\" Version=\"3.0.1\" "
+                "xmlns=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" "
+                "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
+                "xsi:schemaLocation=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" "
+                "xmlns:pbbase=\"http://pacificbiosciences.com/PacBioBaseDataModel.xsd\" "
+                "xmlns:pbds=\"http://pacificbiosciences.com/PacBioDatasets.xsd\">\n"
+        "\t<pbbase:ExternalResources>\n"
+        "\t\t<pbbase:ExternalResource MetaType=\"Fake.MetaType\" ResourceId=\"filename\" TimeStampedName=\"custom_tsn\" UniqueId=\"my_uuid\" Version=\"3.0.1\" />\n"
+        "\t</pbbase:ExternalResources>\n"
+        "\t<pbds:DataSets>\n"
+        "\t\t<pbds:ReferenceSet> CreatedAt=\"2015-01-27T09:00:01\" MetaType=\"PacBio.DataSet.ReferenceSet\" "
+                "Name=\"DataSet_ReferenceSet\" Tags=\"barcode moreTags mapping mytags\" "
+                "TimeStampedName=\"my_time_stamped_name\" "
+                "UniqueId=\"b095d0a3-94b8-4918-b3af-a3f81bbe519c\" Version=\"3.0.1\" "
+                "xmlns=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" "
+                "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
+                "xsi:schemaLocation=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" "
+                "xmlns:pbbase=\"http://pacificbiosciences.com/PacBioBaseDataModel.xsd\" "
+                "xmlns:pbds=\"http://pacificbiosciences.com/PacBioDatasets.xsd\">\n"
+        "\t\t\t<pbds:DataSetMetadata>\n"
+        "\t\t\t\t<pbds:TotalLength>1000</pbds:TotalLength>\n"
+        "\t\t\t\t<pbds:NumRecords>42</pbds:NumRecords>\n"
+        "\t\t\t</pbds:DataSetMetadata>\n"
+        "\t\t</pbds:ReferenceSet>\n"
+        "\t</pbds:DataSets>\n"
+        "\t<pbds:DataSetMetadata>\n"
+        "\t\t<pbds:TotalLength>1000</pbds:TotalLength>\n"
+        "\t\t<pbds:NumRecords>42</pbds:NumRecords>\n"
+        "\t</pbds:DataSetMetadata>\n"
+        "</pbds:ReferenceSet>\n"};
+
+    EXPECT_NO_THROW(DataSet::FromXml(referenceSetXml));
+
+    // AlignmentSet with ReferenceSet subdataset
+    const std::string alignmentSetXml{
         "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
         "<pbds:AlignmentSet CreatedAt=\"2015-01-27T09:00:01\" MetaType=\"PacBio.DataSet.AlignmentSet\" "
                 "Name=\"DataSet_AlignmentSet\" Tags=\"barcode moreTags mapping mytags\" "
@@ -1601,15 +1704,51 @@ TEST(DataSetIOTest, MetadataDefaultChildrenProperlyOrderedPerXsd)
         "\t<pbbase:ExternalResources>\n"
         "\t\t<pbbase:ExternalResource MetaType=\"Fake.MetaType\" ResourceId=\"filename\" TimeStampedName=\"custom_tsn\" UniqueId=\"my_uuid\" Version=\"3.0.1\" />\n"
         "\t</pbbase:ExternalResources>\n"
+        "\t<pbds:DataSets>\n"
+        "\t\t<pbds:ReferenceSet> CreatedAt=\"2015-01-27T09:00:01\" MetaType=\"PacBio.DataSet.ReferenceSet\" "
+                "Name=\"DataSet_ReferenceSet\" Tags=\"barcode moreTags mapping mytags\" "
+                "TimeStampedName=\"my_time_stamped_name\" "
+                "UniqueId=\"b095d0a3-94b8-4918-b3af-a3f81bbe519c\" Version=\"3.0.1\" "
+                "xmlns=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" "
+                "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
+                "xsi:schemaLocation=\"http://pacificbiosciences.com/PacBioDatasets.xsd\" "
+                "xmlns:pbbase=\"http://pacificbiosciences.com/PacBioBaseDataModel.xsd\" "
+                "xmlns:pbds=\"http://pacificbiosciences.com/PacBioDatasets.xsd\">\n"
+        "\t\t\t<pbds:DataSetMetadata>\n"
+        "\t\t\t\t<pbds:TotalLength>1000</pbds:TotalLength>\n"
+        "\t\t\t\t<pbds:NumRecords>42</pbds:NumRecords>\n"
+        "\t\t\t</pbds:DataSetMetadata>\n"
+        "\t\t</pbds:ReferenceSet>\n"
+        "\t</pbds:DataSets>\n"
         "\t<pbds:DataSetMetadata>\n"
         "\t\t<pbds:TotalLength>1000</pbds:TotalLength>\n"
         "\t\t<pbds:NumRecords>42</pbds:NumRecords>\n"
         "\t</pbds:DataSetMetadata>\n"
         "</pbds:AlignmentSet>\n"};
 
-    std::ostringstream s;
-    dataset.SaveToStream(s);
-    EXPECT_EQ(expectedXml, s.str());
+    EXPECT_NO_THROW(DataSet::FromXml(alignmentSetXml));
+}
+
+TEST(DataSetIOTest, AbsolutePathsForGenericAndDerivedDatasets)
+{
+    DataSet dataset;
+    ReferenceSet referenceDataset;
+
+    dataset.ExternalResources().Add(ExternalResource{"PacBio.SubreadFile.SubreadBamFile", "test.fa"});
+    referenceDataset.ExternalResources().Add(ExternalResource{"PacBio.SubreadFile.SubreadBamFile", "test.fa"});
+
+    const std::string expectedGenericFn{dataset.Path() + "/test.fa"};
+    const std::string expectedReferenceFn{referenceDataset.Path() + "/test.fa"};
+
+    std::ostringstream out;
+    dataset.SaveToStream(out);
+    const std::string genericDatasetXml{out.str()};
+    EXPECT_NE(genericDatasetXml.find(expectedGenericFn), std::string::npos);
+
+    out.str("");
+    referenceDataset.SaveToStream(out);
+    const std::string referenceDatasetXml{out.str()};
+    EXPECT_NE(referenceDatasetXml.find(expectedReferenceFn), std::string::npos);
 }
 
 // clang-format on
