@@ -112,8 +112,7 @@ TEST(PbiFilterQueryTest, QueryOk)
 
 TEST(PbiFilterQueryTest, ZmwRangeFromDatasetOk)
 {
-    const auto expectedMovieName =
-        std::string{"m150404_101626_42267_c100807920800000001823174110291514_s1_p0"};
+    const std::string expectedMovieName{"m64004_190414_193017"};
 
     const DataSet ds(PbbamTestsConfig::Data_Dir + "/chunking/chunking.subreadset.xml");
     EXPECT_EQ(3, ds.BamFiles().size());
@@ -134,7 +133,7 @@ TEST(PbiFilterQueryTest, ZmwRangeFromDatasetOk)
 
     {  // sequencing chemistries
         std::set<std::string> chems{ds.SequencingChemistries()};
-        std::set<std::string> expected{"P6-C4"};
+        std::set<std::string> expected{"S/P3-C1/5.0-8M"};
         EXPECT_TRUE(equal(chems.begin(), chems.end(), expected.begin()));
     }
 
@@ -370,33 +369,33 @@ TEST(PbiFilterQueryTest, BarcodeQualityFromXml)
 
     const std::string xml_all = R"_XML_(
 <?xml version="1.0" encoding="utf-8"?>
-<pbds:SubreadSet 
-   xmlns="http://pacificbiosciences.com/PacBioDatasets.xsd" 
-   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
+<pbds:SubreadSet
+   xmlns="http://pacificbiosciences.com/PacBioDatasets.xsd"
+   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:pbbase="http://pacificbiosciences.com/PacBioBaseDataModel.xsd"
    xmlns:pbsample="http://pacificbiosciences.com/PacBioSampleInfo.xsd"
    xmlns:pbmeta="http://pacificbiosciences.com/PacBioCollectionMetadata.xsd"
    xmlns:pbds="http://pacificbiosciences.com/PacBioDatasets.xsd"
-   xsi:schemaLocation="http://pacificbiosciences.com/PacBioDataModel.xsd" 
-   UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe519c" 
-   TimeStampedName="subreadset_150304_231155" 
-   MetaType="PacBio.DataSet.SubreadSet" 
-   Name="DataSet_SubreadSet" 
-   Tags="" 
-   Version="3.0.0" 
-   CreatedAt="2015-01-27T09:00:01"> 
+   xsi:schemaLocation="http://pacificbiosciences.com/PacBioDataModel.xsd"
+   UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe519c"
+   TimeStampedName="subreadset_150304_231155"
+   MetaType="PacBio.DataSet.SubreadSet"
+   Name="DataSet_SubreadSet"
+   Tags=""
+   Version="3.0.0"
+   CreatedAt="2015-01-27T09:00:01">
 <pbbase:ExternalResources>
-   <pbbase:ExternalResource 
-       UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe5193" 
-       TimeStampedName="subread_bam_150304_231155" 
-       MetaType="PacBio.SubreadFile.SubreadBamFile" 
-       ResourceId="m150404_101626_42267_c100807920800000001823174110291514_s1_p0.1.subreads.bam">
+   <pbbase:ExternalResource
+       UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe5193"
+       TimeStampedName="subread_bam_150304_231155"
+       MetaType="PacBio.SubreadFile.SubreadBamFile"
+       ResourceId="m64004_190414_193017.1.subreads.bam">
        <pbbase:FileIndices>
-           <pbbase:FileIndex 
-               UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe5194" 
-               TimeStampedName="bam_index_150304_231155" 
-               MetaType="PacBio.Index.PacBioIndex" 
-               ResourceId="m150404_101626_42267_c100807920800000001823174110291514_s1_p0.1.subreads.bam.pbi"/>
+           <pbbase:FileIndex
+               UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe5194"
+               TimeStampedName="bam_index_150304_231155"
+               MetaType="PacBio.Index.PacBioIndex"
+               ResourceId="m64004_190414_193017.1.subreads.bam.pbi"/>
        </pbbase:FileIndices>
    </pbbase:ExternalResource>
 </pbbase:ExternalResources>
@@ -432,13 +431,13 @@ TEST(PbiFilterQueryTest, BarcodeQualityFromXml)
        UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe5193"
        TimeStampedName="subread_bam_150304_231155"
        MetaType="PacBio.SubreadFile.SubreadBamFile"
-       ResourceId="m150404_101626_42267_c100807920800000001823174110291514_s1_p0.1.subreads.bam">
+       ResourceId="m64004_190414_193017.1.subreads.bam">
        <pbbase:FileIndices>
            <pbbase:FileIndex
                UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe5194"
                TimeStampedName="bam_index_150304_231155"
                MetaType="PacBio.Index.PacBioIndex"
-               ResourceId="m150404_101626_42267_c100807920800000001823174110291514_s1_p0.1.subreads.bam.pbi"/>
+               ResourceId="m64004_190414_193017.1.subreads.bam.pbi"/>
        </pbbase:FileIndices>
    </pbbase:ExternalResource>
 </pbbase:ExternalResources>
@@ -482,6 +481,86 @@ TEST(PbiFilterQueryTest, BarcodeQualityFromXml)
     }
 }
 
+TEST(PbiFilterQueryTest, ReadGroupFilterFromXml)
+{
+    const BamFile file{PbbamTestsConfig::Data_Dir + "/phi29.bam"};
+    const std::string xmlHeader = R"_XML_(
+        <?xml version="1.0" encoding="utf-8"?>
+        <pbds:SubreadSet
+           xmlns="http://pacificbiosciences.com/PacBioDatasets.xsd"
+           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+           xmlns:pbbase="http://pacificbiosciences.com/PacBioBaseDataModel.xsd"
+           xmlns:pbsample="http://pacificbiosciences.com/PacBioSampleInfo.xsd"
+           xmlns:pbmeta="http://pacificbiosciences.com/PacBioCollectionMetadata.xsd"
+           xmlns:pbds="http://pacificbiosciences.com/PacBioDatasets.xsd"
+           xsi:schemaLocation="http://pacificbiosciences.com/PacBioDataModel.xsd"
+           UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe519c"
+           TimeStampedName="subreadset_150304_231155"
+           MetaType="PacBio.DataSet.SubreadSet"
+           Name="DataSet_SubreadSet"
+           Tags=""
+           Version="3.0.0"
+           CreatedAt="2015-01-27T09:00:01">
+        <pbbase:ExternalResources>
+           <pbbase:ExternalResource
+               UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe5193"
+               TimeStampedName="subread_bam_150304_231155"
+               MetaType="PacBio.SubreadFile.SubreadBamFile"
+               ResourceId="phi29.bam">
+               <pbbase:FileIndices>
+                   <pbbase:FileIndex
+                       UniqueId="b095d0a3-94b8-4918-b3af-a3f81bbe5194"
+                       TimeStampedName="bam_index_150304_231155"
+                       MetaType="PacBio.Index.PacBioIndex"
+                       ResourceId="phi29.bam.pbi"/>
+               </pbbase:FileIndices>
+           </pbbase:ExternalResource>
+        </pbbase:ExternalResources>
+        <pbds:Filters>
+            <pbds:Filter>
+                <pbbase:Properties>)_XML_";
+
+    const std::string xmlFooter = R"_XML_(
+                </pbbase:Properties>
+            </pbds:Filter>
+        </pbds:Filters>
+        </pbds:SubreadSet>
+        )_XML_";
+
+    {  // equal
+        const std::string xmlProperty =
+            R"_XML_(<pbbase:Property Name="qid" Operator="==" Value="-1453990154"/>\n)_XML_";
+        const std::string xml = xmlHeader + xmlProperty + xmlFooter;
+        const DataSet ds = DataSet::FromXml(xml);
+        const PbiFilterQuery query{PbiFilter::FromDataSet(ds), file};
+        const auto numReads = query.NumReads();
+        EXPECT_EQ(120, numReads);
+
+        size_t observedReadCount = 0;
+        for (const auto& r : query) {
+            UNUSED(r);
+            ++observedReadCount;
+        }
+        EXPECT_EQ(120, observedReadCount);
+    }
+    {  // not equal
+        const std::string xmlProperty =
+            R"_XML_(<pbbase:Property Name="qid" Operator="!=" Value="-1453990154"/>\n)_XML_";
+        const std::string xml = xmlHeader + xmlProperty + xmlFooter;
+        const DataSet ds = DataSet::FromXml(xml);
+        const PbiFilterQuery query{PbiFilter::FromDataSet(ds), file};
+        const auto numReads = query.NumReads();
+        EXPECT_EQ(0, numReads);
+
+        size_t observedReadCount = 0;
+        for (const auto& r : query) {
+            UNUSED(r);
+            ++observedReadCount;
+        }
+        EXPECT_EQ(0, observedReadCount);
+    }
+}
+
 TEST(PbiFilterQueryTest, ZmwWhitelistFromXml)
 {
     const BamFile file{PbbamTestsConfig::Data_Dir + "/phi29.bam"};
@@ -691,3 +770,150 @@ TEST(PbiFilterQueryTest, TranscriptRecords)
         EXPECT_EQ(4, observed.size());
     }
 }
+
+TEST(PbiFilterQueryTest, BarcodedReadGroupId)
+{
+    const BamFile bamFile{PbbamTestsConfig::Data_Dir + std::string{"/barcoded_read_groups.bam"}};
+
+    {  //  query read group with no barcodes - should catche all, barcoded or not
+        const PbiReadGroupFilter filter{"0d7b28fa"};
+        PbiFilterQuery query{filter, bamFile};
+        size_t count = 0;
+        for (const auto& b : query) {
+            (void)b;
+            ++count;
+        }
+        EXPECT_EQ(5, count);
+    }
+    {  // query read group with barcode label
+
+        const ReadGroupInfo rg{"0d7b28fa/0--0"};
+        PbiReadGroupFilter filter{rg};
+        PbiFilterQuery query{filter, bamFile};
+        size_t count = 0;
+        for (const auto& b : query) {
+            (void)b;
+            ++count;
+        }
+        EXPECT_EQ(1, count);
+    }
+    {  // query multiple read groups with barcode label
+
+        const ReadGroupInfo rg{"0d7b28fa/0--0"};
+        const ReadGroupInfo rg1{"0d7b28fa/1--0"};
+        PbiReadGroupFilter filter{std::vector<ReadGroupInfo>{rg, rg1}};
+        PbiFilterQuery query{filter, bamFile};
+        size_t count = 0;
+        for (const auto& b : query) {
+            (void)b;
+            ++count;
+        }
+        EXPECT_EQ(2, count);
+    }
+}
+
+TEST(PbiFilterQueryTest, CanReusePbiIndexCache)
+{
+    const DataSet ds(PbbamTestsConfig::Data_Dir + "/chunking/chunking.subreadset.xml");
+    const auto indexCache = MakePbiIndexCache(ds);
+
+    {
+        // min ZMW
+        PbiFilterQuery query{PbiZmwFilter{54, Compare::GREATER_THAN}, ds, indexCache};
+        const auto numReads = query.NumReads();
+        EXPECT_EQ(1220, numReads);
+
+        int count = 0;
+        for (const BamRecord& r : query) {
+            EXPECT_GT(r.HoleNumber(), 54);
+            ++count;
+        }
+        EXPECT_EQ(1220, count);
+    }
+
+    {  // max ZMW
+        PbiFilterQuery query{PbiZmwFilter{1816, Compare::LESS_THAN}, ds, indexCache};
+        const auto numReads = query.NumReads();
+        EXPECT_EQ(150, numReads);
+
+        int count = 0;
+        for (const BamRecord& r : query) {
+            EXPECT_LT(r.HoleNumber(), 1816);
+            ++count;
+        }
+        EXPECT_EQ(150, count);
+    }
+}
+
+TEST(PbiFilterQueryTest, ConsistentWhitelistAndBlacklist)
+{
+    const std::string fn{PbbamTestsConfig::Data_Dir + "/dataset/qname_filter.bam"};
+
+    const std::vector<std::string> recordNames{"singleInsertion/0/0_10", "singleInsertion/0/10_20",
+                                               "singleInsertion/1/0_10", "singleInsertion/1/10_20"};
+    const std::vector<std::string> whitelist{"singleInsertion/0/0_10", "singleInsertion/1/0_10"};
+    const std::vector<std::string> blacklist{"singleInsertion/0/10_20", "singleInsertion/1/10_20"};
+
+    {  // sanity check on input
+        PbiFilter filter{};
+        PbiFilterQuery query{filter, fn};
+        EXPECT_EQ(4, query.NumReads());
+
+        size_t i = 0;
+        for (const auto& b : query) {
+            EXPECT_EQ(recordNames.at(i), b.FullName());
+            ++i;
+        }
+    }
+    {
+        // whitelist
+        PbiFilter filter{PbiQueryNameFilter{whitelist}};
+        PbiFilterQuery query{filter, fn};
+        EXPECT_EQ(2, query.NumReads());
+
+        size_t i = 0;
+        for (const auto& b : query) {
+            if (i == 0) {
+                EXPECT_EQ(recordNames.at(0), b.FullName());
+            }
+            if (i == 1) {
+                EXPECT_EQ(recordNames.at(2), b.FullName());
+            }
+            ++i;
+        }
+    }
+    {
+        // !whitelist
+        PbiFilter filter{PbiQueryNameFilter{whitelist, Compare::NOT_CONTAINS}};
+        PbiFilterQuery query{filter, fn};
+        EXPECT_EQ(2, query.NumReads());
+
+        size_t i = 0;
+        for (const auto& b : query) {
+            if (i == 0) {
+                EXPECT_EQ(recordNames.at(1), b.FullName());
+            }
+            if (i == 1) {
+                EXPECT_EQ(recordNames.at(3), b.FullName());
+            }
+            ++i;
+        }
+    }
+    {
+        // blacklist
+        PbiFilter filter{PbiQueryNameFilter{blacklist, Compare::NOT_CONTAINS}};
+        PbiFilterQuery query{filter, fn};
+        EXPECT_EQ(2, query.NumReads());
+
+        size_t i = 0;
+        for (const auto& b : query) {
+            if (i == 0) {
+                EXPECT_EQ(recordNames.at(0), b.FullName());
+            }
+            if (i == 1) {
+                EXPECT_EQ(recordNames.at(2), b.FullName());
+            }
+            ++i;
+        }
+    }
+}
